import * as git from "./git"
import { load, write, getFiles } from "./file_systems"

export class BasePlugin {
  options: any
  name: string

  constructor(options: any, name: string) {
    this.options = options
    this.name = name
  }

  async preFix(): Promise<any> {}

  async fix(_content: any, _filename: any): Promise<any> {
    return null
  }

  async preCheck(): Promise<any> {}

  async check(_content: any, _filename: any): Promise<any> {
    return null
  }

  async postCheck(_analysis: any): Promise<any> {}
}

export class SchemaBugs {
  options: any
  plugins: any

  constructor(options: any) {
    this.options = options
    this.get(this.options.schema)
  }

  get(schema: any): any {
    this.plugins = []
    const bugsDir = this.options["bugs-dir"] || "."
    const files = getFiles([`${bugsDir}/src/bugs/${schema}-*.ts`]).filter(
      (f: any) => this.options.bug == undefined || f.includes(this.options.bug),
    )
    for (const file of files) {
      const module = require(`../${file}`)
      this.plugins.push(new module["Plugin"](this.options))
    }
  }

  async preFix(): Promise<any> {
    for (const plugin of this.plugins) await plugin.preFix()
  }

  async fix(filenames: any): Promise<any> {
    let results: any = {}
    for (const plugin of this.plugins) {
      for (const filename of filenames) {
        const content = load(filename)
        const result = await plugin.fix(content, filename)
        results[plugin.name] = result
        if (result == true) write(content, filename)
        if (this.options.debug)
          console.log(`${plugin.name} ${filename} ${result ? "FIXED" : "OK"}`)
      }
      if (this.options.commit) {
        git.commit(this.options.commit, `Fix ${plugin.name}`)
      }
    }
    return results
  }

  async preCheck(): Promise<void> {
    for (const plugin of this.plugins) await plugin.preCheck()
  }

  async check(content: any, filename: any): Promise<any> {
    let results: any = {}
    for (const plugin of this.plugins)
      results[plugin.name] = await plugin.check(content, filename)
    return results
  }

  async postCheck(analysis: any): Promise<any> {
    for (const plugin of this.plugins) await plugin.postCheck(analysis)
  }
}
