import { load } from "../../file_systems"
import { assert } from "chai"
import { Plugin, remapJo } from "../agenda-00002"

const id = "00002"

suite(`Plugin${id}`)

test(`#Plugin${id}`, async function () {
  Object.assign(remapJo, {
    RUANR5L15S2020IDS22089: "20200140",
  })

  const plugin = new Plugin({ cr: "src/bugs/agenda-00002" })
  const d = `src/bugs/agenda-${id}`

  for (const expectedStatus of [
    "will-be-fixed",
    "already-fixed",
    "ok",
    "needs-fixing",
  ]) {
    const f = `${d}/${expectedStatus}.json`
    const content = load(f)
    await plugin.preCheck()
    const r = await plugin.check(content, f)
    assert.equal(
      r["status"],
      expectedStatus,
      `${f} returns ${r["status"]} ${r["info"]}`,
    )
    await plugin.postCheck({ f: r })
    assert.equal(
      r["status"],
      expectedStatus,
      `${f} returns ${r["status"]} ${r["info"]}`,
    )
  }
  {
    const f = `${d}/null.json`
    const content = load(f)
    await plugin.preCheck()
    const r = await plugin.check(content, f)
    assert.equal(r, null, f)
  }
})

test(`#Plugin${id}checkCandidates`, async function () {
  const plugin = new Plugin({ cr: "src/bugs/agenda-00002" })
  const d = `src/bugs/agenda-${id}`

  const f = `${d}/RUANR5L15S2019IDS21272.json`
  await plugin.preCheck()
  const content = load(f)
  const r = await plugin.check(content, f)
  assert.deepEqual(r["status"], "needs-fixing", f)
  let analysis: any = {}
  analysis[f] = { "agenda-00002": r }
  await plugin.postCheck(analysis)
  assert.deepEqual(
    analysis[f]["agenda-00002"]["info"][1],
    "src/bugs/agenda-00002/test_20190017.asp is a candidate",
    f,
  )
})
