import assert from "assert"

import { cleanBooleanAttribute, cleanXmlArtefacts } from "./xml"
import { AvantAApres } from "../types/amendements"

export function cleanAmendement(amendement: any): void {
  cleanXmlArtefacts(amendement)

  const signataires = amendement.signataires
  {
    cleanXmlArtefacts(signataires)

    const auteur = signataires.auteur
    {
      cleanXmlArtefacts(auteur)
    }

    if (signataires.cosignataires !== undefined) {
      const acteurRef = signataires.cosignataires.acteurRef
      if (Array.isArray(acteurRef)) {
        signataires.cosignatairesRefs = acteurRef
      } else {
        assert(acteurRef)
        signataires.cosignatairesRefs = [acteurRef]
      }
      delete signataires.cosignataires
    }

    let libelle = signataires.libelle
    if (libelle !== undefined) {
      libelle = libelle.split(" ").filter(Boolean).join(" ")
      if (libelle) {
        signataires.libelle = libelle
      } else {
        delete signataires.libelle
      }
    }
  }

  const pointeurFragmentTexte = amendement.pointeurFragmentTexte
  {
    cleanXmlArtefacts(pointeurFragmentTexte)

    const division = pointeurFragmentTexte.division
    if (division !== undefined) {
      cleanXmlArtefacts(division)

      let avantAApres = division.avant_A_Apres
      if (avantAApres !== undefined) {
        delete division.avant_A_Apres
        if (avantAApres === "Après") {
          avantAApres = AvantAApres.Apres
        }
        division.avantAApres = avantAApres
      }
    }

    const amendementStandard = pointeurFragmentTexte.amendementStandard
    if (amendementStandard !== undefined) {
      cleanXmlArtefacts(amendementStandard)

      const alinea = amendementStandard.alinea
      if (alinea !== undefined) {
        cleanXmlArtefacts(alinea)

        let avantAApres = alinea.avant_A_Apres
        if (avantAApres !== undefined) {
          delete alinea.avant_A_Apres
          if (avantAApres === "Après") {
            avantAApres = AvantAApres.Apres
          } else if (
            ![AvantAApres.A, AvantAApres.Apres, AvantAApres.Avant].includes(
              avantAApres,
            )
          ) {
            avantAApres = AvantAApres.A
          }
          alinea.avantAApres = avantAApres
        }

        pointeurFragmentTexte.alinea = alinea
      }
      delete pointeurFragmentTexte.amendementStandard
    }
  }

  const corps = amendement.corps
  if (corps !== undefined) {
    cleanXmlArtefacts(corps)

    const contenuAuteur = corps.contenuAuteur
    if (contenuAuteur !== undefined) {
      cleanXmlArtefacts(contenuAuteur)

      const avantAppel = contenuAuteur.avantAppel
      if (avantAppel !== undefined) {
        cleanXmlArtefacts(avantAppel)

        cleanDispositifAmdtCreditPLF(avantAppel.dispositifAmdtCreditPLF, {
          rectificatif: false,
        })
        cleanDispositifAmdtCreditPLF(avantAppel.dispositifAmdtCreditPLFR, {
          rectificatif: true,
        })

        if (Object.keys(avantAppel).length === 0) {
          delete contenuAuteur.avantAppel
        }
      }

      assert.strictEqual(contenuAuteur.annexeExposeSommaire, undefined)

      cleanDispositifAmdtCreditPLF(contenuAuteur.dispositifAmdtCreditPLF, {
        rectificatif: false,
      })
      cleanDispositifAmdtCreditPLF(contenuAuteur.dispositifAmdtCreditPLFR, {
        rectificatif: true,
      })
    }
  }

  const cycleDeVie = amendement.cycleDeVie
  if (cycleDeVie !== undefined) {
    cleanXmlArtefacts(cycleDeVie)

    const etatDesTraitements = cycleDeVie.etatDesTraitements
    {
      cleanXmlArtefacts(etatDesTraitements)

      const etat = etatDesTraitements.etat
      {
        cleanXmlArtefacts(etat)
      }

      const sousEtat = etatDesTraitements.sousEtat
      if (sousEtat !== undefined) {
        cleanXmlArtefacts(sousEtat)
      }
    }
  }

  if (amendement.representations !== undefined) {
    const representation = amendement.representations.representation
    assert(!Array.isArray(representation))
    amendement.representation = representation
    delete amendement.representations
    {
      cleanXmlArtefacts(representation)

      assert.strictEqual(representation.repSource, undefined)
      assert.strictEqual(representation.offset, undefined)
      assert.strictEqual(representation.dateDispoRepresentation, undefined)

      const statutRepresentation = representation.statutRepresentation
      {
        cleanBooleanAttribute(statutRepresentation, "canonique")
        cleanBooleanAttribute(statutRepresentation, "enregistrement")
        cleanBooleanAttribute(statutRepresentation, "officielle")
        cleanBooleanAttribute(statutRepresentation, "transcription")
        cleanBooleanAttribute(statutRepresentation, "verbatim")
      }
    }
  }

  const loiReference = amendement.loiReference
  if (loiReference !== undefined) {
    cleanXmlArtefacts(loiReference)
  }

  const discussionCommune = amendement.discussionCommune
  if (discussionCommune !== undefined) {
    cleanXmlArtefacts(discussionCommune)
  }

  const discussionIdentique = amendement.discussionIdentique
  if (discussionIdentique !== undefined) {
    cleanXmlArtefacts(discussionIdentique)
  }
}

function cleanDispositifAmdtCreditPLF(
  dispositifAmdtCreditPLF: any,
  { rectificatif }: { rectificatif: boolean },
): void {
  if (dispositifAmdtCreditPLF !== undefined) {
    const listeProgrammes = dispositifAmdtCreditPLF.listeProgrammes?.programme
    if (listeProgrammes !== undefined) {
      if (Array.isArray(listeProgrammes)) {
        dispositifAmdtCreditPLF.listeProgrammes = listeProgrammes
      } else {
        assert(listeProgrammes)
        dispositifAmdtCreditPLF.listeProgrammes = [listeProgrammes]
      }

      for (const programme of dispositifAmdtCreditPLF.listeProgrammes) {
        cleanXmlArtefacts(programme)

        if (programme.lignesCredits !== undefined) {
          let lignesCredits = programme.lignesCredits.ligneCredit
          if (lignesCredits === undefined) {
            delete programme.lignesCredits
          } else {
            if (!Array.isArray(lignesCredits)) {
              assert(lignesCredits)
              lignesCredits = [lignesCredits]
            }
            programme.lignesCredits = lignesCredits
          }
          if (lignesCredits !== undefined) {
            for (const ligneCredit of lignesCredits) {
              if (rectificatif) {
                cleanLigneCreditPLFR(ligneCredit)
              } else {
                cleanXmlArtefacts(ligneCredit)
              }
            }
          }
        }
      }
    }
  }
}

function cleanLigneCreditPLFR(ligneCreditPLFR: any): void {
  cleanXmlArtefacts(ligneCreditPLFR)

  if (ligneCreditPLFR.lignesCredits !== undefined) {
    let lignesCredits = ligneCreditPLFR.lignesCredits.ligneCredit
    if (lignesCredits === undefined) {
      delete ligneCreditPLFR.lignesCredits
    } else {
      if (!Array.isArray(lignesCredits)) {
        assert(lignesCredits)
        lignesCredits = [lignesCredits]
      }
      ligneCreditPLFR.lignesCredits = lignesCredits
    }
    if (lignesCredits !== undefined) {
      for (const ligneCredit of lignesCredits) {
        cleanLigneCreditPLFR(ligneCredit)
      }
    }
  }
}

export function cleanTexteLegislatif(texteLegislatif: any): void {
  cleanXmlArtefacts(texteLegislatif)

  // Note: The following fields in `texteLegislatif.amendements` are currently ignored:
  // * "#text"?:       string;
  // * offset?:        null;
  // * nom?:           Nom;
  // * avant_A_Apres?: DivisionAvantAApres;
  // * dateDepot?:     Date;
  // * documentURI?:   string;
  // Because it seems they should belong to an "amendement".
  let amendement = texteLegislatif.amendements.amendement
  if (Array.isArray(amendement)) {
    texteLegislatif.amendements = amendement
  } else {
    assert(amendement)
    texteLegislatif.amendements = [amendement]
  }

  for (const amendement of texteLegislatif.amendements) {
    cleanAmendement(amendement)
  }
}
