import { glob } from "glob"
import fs from "fs-extra"
import path from "path"

export function* walkDir(
  rootDir: string,
  relativeSplitDir: string[] = [],
): Iterable<string[]> {
  const dir = path.join(rootDir, ...relativeSplitDir)

  if (!fs.existsSync(dir)) {
    return
  }

  for (const filename of fs.readdirSync(dir)) {
    if (filename[0] === ".") {
      continue
    }
    const filePath = path.join(dir, filename)
    const relativeSplitPath = [...relativeSplitDir, filename]
    if (fs.statSync(filePath).isDirectory()) {
      yield* walkDir(rootDir, relativeSplitPath)
    } else {
      yield relativeSplitPath
    }
  }
}

export function load(path: string): any {
  const rawjson: string = fs.readFileSync(path, {
    encoding: "utf8",
  })
  return JSON.parse(rawjson)
}

export function write(something: any, filename: string): any {
  const content = JSON.stringify(something, null, 2)
  fs.ensureDirSync(path.dirname(filename))
  fs.writeFileSync(filename, content, {
    encoding: "utf8",
  })
}

export class GetFileNotFoundError {
  message: string

  constructor(message: string) {
    this.message = message
  }
}

export function getFiles(args: any): any {
  let files: string[] = []
  function _getFiles(fileOrPattern: any) {
    if (glob.hasMagic(fileOrPattern)) {
      const dataFiles = glob.globSync(fileOrPattern, { cwd: process.cwd() })
      // Note: When a legislature ends, the subdirectory of "acteurs" in AMO10 or AMO40
      // may be empty (no more député).
      if (dataFiles.length > 0) files = files.concat(dataFiles)
    } else {
      files.push(fileOrPattern)
      if (!fs.existsSync(fileOrPattern))
        throw new GetFileNotFoundError(`${fileOrPattern} file does not exist`)
    }
  }
  args.forEach(_getFiles)
  return files
}
