// To parse this data:
//
//   import { Convert, Acteur, Deport, Organe, MandatWrapper, ActeursEtOrganes } from "./file";
//
//   const acteur = Convert.toActeur(json);
//   const deport = Convert.toDeport(json);
//   const organe = Convert.toOrgane(json);
//   const mandatWrapper = Convert.toMandatWrapper(json);
//   const acteursEtOrganes = Convert.toActeursEtOrganes(json);
//
// These functions will throw an error if the JSON doesn't
// match the expected interface, even if the JSON is valid.

export interface Acteur {
    acteur: ActeurActeur;
}

export interface ActeurActeur {
    "@xmlns"?:     string;
    uid:           Uid;
    etatCivil:     EtatCivil;
    profession:    Profession;
    uri_hatvp:     Nil | null | string;
    adresses:      Adresses | null;
    mandats?:      Mandats;
    "@xmlns:xsi"?: string;
}

export interface Adresses {
    adresse?:      { [key: string]: null | string }[] | { [key: string]: null | string };
    "@xmlns:xsi"?: string;
    "@xsi:nil"?:   string;
}

export interface EtatCivil {
    ident:         Ident;
    infoNaissance: InfoNaissance;
    dateDeces:     Nil | Date | null;
}

export interface Nil {
    "@xmlns:xsi": string;
    "@xsi:nil":   string;
}

export interface Ident {
    civ:       Civ;
    prenom:    string;
    nom:       string;
    alpha:     string;
    trigramme: Nil | null | string;
}

export enum Civ {
    M = "M.",
    Mme = "Mme",
}

export interface InfoNaissance {
    dateNais:  Nil | Date | null;
    villeNais: Nil | null | string;
    depNais:   Nil | DepNais | null;
    paysNais:  Nil | null | string;
}

export enum DepNais {
    Ain = "Ain",
    Aisne = "Aisne",
    Allier = "Allier",
    AlpesDeHauteProvence = "Alpes-de-Haute-Provence",
    AlpesMaritimes = "Alpes-Maritimes",
    Ardennes = "Ardennes",
    Ardèche = "Ardèche",
    Ariège = "Ariège",
    Aube = "Aube",
    Aude = "Aude",
    Aveyron = "Aveyron",
    BasRhin = "Bas-Rhin",
    BouchesDuRhône = "Bouches-du-Rhône",
    Calvados = "Calvados",
    Cantal = "Cantal",
    Charente = "Charente",
    CharenteMaritime = "Charente-Maritime",
    Cher = "Cher",
    Comores = "Comores",
    Corrèze = "Corrèze",
    Corse = "Corse",
    CorseDuSud = "Corse-du-Sud",
    Creuse = "Creuse",
    CôteDOr = "Côte-d'Or",
    CôtesDArmor = "Côtes-d'Armor",
    CôtesDuNord = "Côtes-du-Nord",
    DeuxSèvres = "Deux-Sèvres",
    Dordogne = "Dordogne",
    Doubs = "Doubs",
    Drôme = "Drôme",
    Essonne = "Essonne",
    Eure = "Eure",
    EureEtLoir = "Eure-et-Loir",
    Finistère = "Finistère",
    FrançaisÉtablisHorsDeFrance = "Français établis hors de France",
    Gard = "Gard",
    Gers = "Gers",
    Gironde = "Gironde",
    Guadeloupe = "Guadeloupe",
    Guyane = "Guyane",
    HautRhin = "Haut-Rhin",
    HauteCorse = "Haute-Corse",
    HauteGaronne = "Haute-Garonne",
    HauteLoire = "Haute-Loire",
    HauteMarne = "Haute-Marne",
    HauteSavoie = "Haute-Savoie",
    HauteSaône = "Haute-Saône",
    HauteVienne = "Haute-Vienne",
    HautesAlpes = "Hautes-Alpes",
    HautesPyrénées = "Hautes-Pyrénées",
    HautsDeSeine = "Hauts-de-Seine",
    Hérault = "Hérault",
    IlleEtVilaine = "Ille-et-Vilaine",
    Indre = "Indre",
    IndreEtLoire = "Indre-et-Loire",
    Isère = "Isère",
    Jura = "Jura",
    Landes = "Landes",
    LoirEtCher = "Loir-et-Cher",
    Loire = "Loire",
    LoireAtlantique = "Loire-Atlantique",
    Loiret = "Loiret",
    Lot = "Lot",
    LotEtGaronne = "Lot-et-Garonne",
    Lozère = "Lozère",
    MaineEtLoire = "Maine-et-Loire",
    Manche = "Manche",
    Marne = "Marne",
    Martinique = "Martinique",
    Mayenne = "Mayenne",
    Mayotte = "Mayotte",
    MeurtheEtMoselle = "Meurthe-et-Moselle",
    Meuse = "Meuse",
    Morbihan = "Morbihan",
    Moselle = "Moselle",
    Nièvre = "Nièvre",
    Nord = "Nord",
    NouvelleCalédonie = "Nouvelle-Calédonie",
    Oise = "Oise",
    Orne = "Orne",
    Paris = "Paris",
    PasDeCalais = "Pas-de-Calais",
    PolynésieFrançaise = "Polynésie Française",
    PuyDeDôme = "Puy-de-Dôme",
    PyrénéesAtlantiques = "Pyrénées-Atlantiques",
    PyrénéesOrientales = "Pyrénées-Orientales",
    Rhône = "Rhône",
    Réunion = "Réunion",
    SaintBarthélemyEtSaintMartin = "Saint-Barthélemy et Saint-Martin",
    SaintMartin = "Saint-Martin",
    SaintPierreEtMiquelon = "Saint-Pierre-et-Miquelon",
    Sarthe = "Sarthe",
    Savoie = "Savoie",
    SaôneEtLoire = "Saône-et-Loire",
    SeineEtMarne = "Seine-et-Marne",
    SeineInférieure = "Seine-Inférieure",
    SeineMaritime = "Seine-Maritime",
    SeineSaintDenis = "Seine-Saint-Denis",
    Somme = "Somme",
    Tarn = "Tarn",
    TarnEtGaronne = "Tarn-et-Garonne",
    TerritoireDeBelfort = "Territoire de Belfort",
    ValDOise = "Val-d'Oise",
    ValDeMarne = "Val-de-Marne",
    Var = "Var",
    Vaucluse = "Vaucluse",
    Vendée = "Vendée",
    Vienne = "Vienne",
    Vosges = "Vosges",
    WallisEtFutuna = "Wallis-et-Futuna",
    Yonne = "Yonne",
    Yvelines = "Yvelines",
}

export interface Mandats {
    mandat: Mandat[] | Mandat;
}

export interface Mandat {
    "@xmlns:xsi"?:         string;
    "@xsi:type"?:          MandatXsiType;
    uid:                   string;
    acteurRef:             string;
    legislature:           null | string;
    typeOrgane:            CodeType;
    dateDebut:             Date;
    datePublication:       Date | null;
    dateFin:               Date | null;
    preseance:             null | string;
    nominPrincipale:       string;
    infosQualite:          InfosQualite;
    organes:               Organes;
    libelle?:              null | string;
    missionSuivanteRef?:   null | string;
    missionPrecedenteRef?: null | string;
    suppleants?:           Suppleants | null;
    chambre?:              null;
    election?:             Election;
    InfosHorsSIAN?:        InfosHorsSian;
    mandature?:            Mandature;
    collaborateurs?:       null[] | Collaborateurs | null;
}

export enum MandatXsiType {
    MandatAvecSuppleantType = "MandatAvecSuppleant_Type",
    MandatMissionType = "MandatMission_Type",
    MandatParlementaireType = "MandatParlementaire_type",
    MandatSimpleType = "MandatSimple_Type",
}

export interface Collaborateurs {
    collaborateur: CollaborateurElement[] | CollaborateurElement;
}

export interface CollaborateurElement {
    qualite:   Civ | null;
    prenom:    string;
    nom:       string;
    dateDebut: null;
    dateFin:   null;
}

export interface Election {
    lieu:                ElectionLieu;
    causeMandat:         null | string;
    refCirconscription?: null | string;
}

export interface ElectionLieu {
    region:         Libelle | null;
    regionType:     Type | null;
    departement:    DepNais | null;
    numDepartement: null | string;
    numCirco:       null | string;
}

export enum Libelle {
    AuvergneRhôneAlpes = "Auvergne-Rhône-Alpes",
    BourgogneFrancheComté = "Bourgogne-Franche-Comté",
    Bretagne = "Bretagne",
    CentreValDeLoire = "Centre-Val de Loire",
    Corse = "Corse",
    FrançaisÉtablisHorsDeFrance = "Français établis hors de France",
    GrandEst = "Grand Est",
    Guadeloupe = "Guadeloupe",
    Guyane = "Guyane",
    HautsDeFrance = "Hauts-de-France",
    IleDeFrance = "Ile-de-France",
    Martinique = "Martinique",
    Mayotte = "Mayotte",
    Normandie = "Normandie",
    NouvelleAquitaine = "Nouvelle-Aquitaine",
    NouvelleCalédonie = "Nouvelle-Calédonie",
    Occitanie = "Occitanie",
    PaysDeLaLoire = "Pays de la Loire",
    PolynésieFrançaise = "Polynésie française",
    ProvenceAlpesCôteDAzur = "Provence-Alpes-Côte d'Azur",
    Réunion = "Réunion",
    SaintBarthélemyEtSaintMartin = "Saint-Barthélemy et Saint-Martin",
    SaintPierreEtMiquelon = "Saint-Pierre-et-Miquelon",
    WallisEtFutuna = "Wallis-et-Futuna",
}

export enum Type {
    CollectivitésDOutreMerEtNouvelleCalédonie = "Collectivités d'outre-mer et Nouvelle-Calédonie",
    Dom = "Dom",
    FrançaisÉtablisHorsDeFrance = "Français établis hors de France",
    Métropolitain = "Métropolitain",
}

export interface InfosQualite {
    codeQualite:   null | string;
    libQualite:    string;
    libQualiteSex: null | string;
}

export interface Mandature {
    datePriseFonction:  Date | null;
    causeFin:           null | string;
    premiereElection:   null | string;
    placeHemicycle:     null | string;
    mandatRemplaceRef?: null | string;
    matriculeVote?:     null | string;
}

export interface Organes {
    organeRef: string[] | string;
}

export interface Suppleants {
    suppleant: Suppleant;
}

export interface Suppleant {
    dateDebut:    Date;
    dateFin:      Date | null;
    suppleantRef: string;
}

export enum CodeType {
    Api = "API",
    Assemblee = "ASSEMBLEE",
    Bureau = "BUREAU",
    Circonscription = "CIRCONSCRIPTION",
    Cjr = "CJR",
    Cmp = "CMP",
    Cnpe = "CNPE",
    Cnps = "CNPS",
    Comnl = "COMNL",
    Comper = "COMPER",
    Comsenat = "COMSENAT",
    Comspsenat = "COMSPSENAT",
    Confpt = "CONFPT",
    Constitu = "CONSTITU",
    Deleg = "DELEG",
    Delegbureau = "DELEGBUREAU",
    Delegsenat = "DELEGSENAT",
    Ga = "GA",
    Ge = "GE",
    Gevi = "GEVI",
    Gouvernement = "GOUVERNEMENT",
    Gp = "GP",
    Groupesenat = "GROUPESENAT",
    Hcj = "HCJ",
    Ministere = "MINISTERE",
    Miseval = "MISEVAL",
    Misinfo = "MISINFO",
    Misinfocom = "MISINFOCOM",
    Misinfopre = "MISINFOPRE",
    Offpar = "OFFPAR",
    Orgaint = "ORGAINT",
    Orgextparl = "ORGEXTPARL",
    Parpol = "PARPOL",
    Presrep = "PRESREP",
    Senat = "SENAT",
}

export interface Profession {
    libelleCourant: Nil | null | string;
    socProcINSEE:   SocProcInsee;
}

export interface SocProcInsee {
    catSocPro: Nil | null | string;
    famSocPro: Nil | FamSocProEnum | null;
}

export enum FamSocProEnum {
    AgriculteursExploitants = "Agriculteurs exploitants",
    ArtisansCommerçantsEtChefsDEntreprise = "Artisans, commerçants et chefs d'entreprise",
    AutresPersonnesSansActivitéProfessionnelle = "Autres personnes sans activité professionnelle",
    CadresEtProfessionsIntellectuellesSupérieures = "Cadres et professions intellectuelles supérieures",
    Employés = "Employés",
    Ouvriers = "Ouvriers",
    ProfessionsIntermédiaires = "Professions Intermédiaires",
    Retraités = "Retraités",
    SansProfessionDéclarée = "Sans profession déclarée",
}

export interface Uid {
    "@xmlns:xsi"?: string;
    "@xsi:type":   UidXsiType;
    "#text":       string;
}

export enum UidXsiType {
    IdActeurType = "IdActeur_type",
}

export interface Deport {
    deport: DeportClass;
}

export interface DeportClass {
    "@xmlns":        string;
    "@xmlns:xsi":    string;
    uid:             string;
    chronotag:       string;
    legislature:     string;
    refActeur:       string;
    dateCreation:    Date;
    datePublication: Date;
    portee:          Instance;
    lecture:         Instance;
    instance:        Instance;
    cible:           Cible;
    explication:     null | string;
}

export interface Cible {
    type:               Instance;
    referenceTextuelle: string;
    references:         null;
}

export interface Instance {
    code:    string;
    libelle: string;
}

export interface Organe {
    organe: OrganeOrgane;
}

export interface OrganeOrgane {
    "@xmlns"?:                string;
    "@xmlns:xsi":             string;
    "@xsi:type"?:             OrganeXsiType;
    uid:                      string;
    codeType:                 CodeType;
    libelle:                  string;
    libelleEdition:           null | string;
    libelleAbrege:            string;
    libelleAbrev:             string;
    viMoDe:                   ViMoDe;
    organeParent?:            null | string;
    organePrecedentRef?:      null | string;
    chambre?:                 null;
    regime?:                  Regime | null;
    legislature?:             null | string;
    regimeJuridique?:         null | string;
    siteInternet?:            null | string;
    nombreReunionsAnnuelles?: null | string;
    secretariat?:             Secretariat;
    numero?:                  null | string;
    lieu?:                    OrganeLieu;
    positionPolitique?:       PositionPolitique | null;
    preseance?:               null | string;
    couleurAssociee?:         null | string;
    listePays?:               ListePays | null;
}

export enum OrganeXsiType {
    GroupePolitiqueType = "GroupePolitique_type",
    OrganeCirconscriptionType = "OrganeCirconscription_type",
    OrganeExterneType = "OrganeExterne_Type",
    OrganeExtraParlementaireType = "OrganeExtraParlementaire_type",
    OrganeParlementaireInternational = "OrganeParlementaireInternational",
    OrganeParlementaireType = "OrganeParlementaire_Type",
}

export interface OrganeLieu {
    region:      Region;
    departement: Departement;
}

export interface Departement {
    codeNatureDep: CodeNatureDep | null;
    code:          null | string;
    libelle:       DepNais | null;
}

export enum CodeNatureDep {
    H = "H",
    M = "M",
    O = "O",
    T = "T",
}

export interface Region {
    type:    Type | null;
    libelle: Libelle | null;
}

export interface ListePays {
    paysRef: string | string[];
}

export enum PositionPolitique {
    Majoritaire = "Majoritaire",
    Minoritaire = "Minoritaire",
    Opposition = "Opposition",
}

export enum Regime {
    AssembléeConsultativeProvisoire = "Assemblée consultative provisoire",
    AssembléesNationalesConstituantes = "Assemblées nationales constituantes",
    The3ÈmeRépublique = "3ème République",
    The4ÈmeRépublique = "4ème République",
    The5ÈmeRépublique = "5ème République",
}

export interface Secretariat {
    secretaire01: null | string;
    secretaire02: null | string;
}

export interface ViMoDe {
    dateDebut:    Date | null;
    dateAgrement: Date | null;
    dateFin:      Date | null;
}

export interface MandatWrapper {
    mandat: Mandat;
}

export interface ActeursEtOrganes {
    export: Export;
}

export interface Export {
    "@xmlns:xsi": string;
    organes:      ExportOrganes;
    acteurs:      Acteurs;
}

export interface Acteurs {
    acteur: ActeurElement[];
}

export interface ActeurElement {
    uid:        Uid;
    etatCivil:  EtatCivil;
    uri_hatvp?: null | string;
    mandats:    Mandats;
}

export interface InfosHorsSian {
    HATVP_URI: null;
}

export interface ExportOrganes {
    organe: OrganeElement[];
}

export interface OrganeElement {
    "@xsi:type":        OrganeXsiType;
    uid:                string;
    codeType:           CodeType;
    libelle:            string;
    libelleEdition:     null | string;
    libelleAbrege:      string;
    libelleAbrev:       string;
    viMoDe:             ViMoDe;
    regimeJuridique?:   null | string;
    regime?:            Regime | null;
    legislature?:       null | string;
    secretariat?:       Secretariat;
    positionPolitique?: PositionPolitique | null;
}

// Converts JSON strings to/from your types
// and asserts the results of JSON.parse at runtime
export class Convert {
    public static toActeur(json: string): Acteur {
        return cast(JSON.parse(json), r("Acteur"));
    }

    public static acteurToJson(value: Acteur): string {
        return JSON.stringify(uncast(value, r("Acteur")), null, 2);
    }

    public static toDeport(json: string): Deport {
        return cast(JSON.parse(json), r("Deport"));
    }

    public static deportToJson(value: Deport): string {
        return JSON.stringify(uncast(value, r("Deport")), null, 2);
    }

    public static toOrgane(json: string): Organe {
        return cast(JSON.parse(json), r("Organe"));
    }

    public static organeToJson(value: Organe): string {
        return JSON.stringify(uncast(value, r("Organe")), null, 2);
    }

    public static toMandatWrapper(json: string): MandatWrapper {
        return cast(JSON.parse(json), r("MandatWrapper"));
    }

    public static mandatWrapperToJson(value: MandatWrapper): string {
        return JSON.stringify(uncast(value, r("MandatWrapper")), null, 2);
    }

    public static toActeursEtOrganes(json: string): ActeursEtOrganes {
        return cast(JSON.parse(json), r("ActeursEtOrganes"));
    }

    public static acteursEtOrganesToJson(value: ActeursEtOrganes): string {
        return JSON.stringify(uncast(value, r("ActeursEtOrganes")), null, 2);
    }
}

function invalidValue(typ: any, val: any, key: any = ''): never {
    if (key) {
        throw Error(`Invalid value for key "${key}". Expected type ${JSON.stringify(typ)} but got ${JSON.stringify(val)}`);
    }
    throw Error(`Invalid value ${JSON.stringify(val)} for type ${JSON.stringify(typ)}`, );
}

function jsonToJSProps(typ: any): any {
    if (typ.jsonToJS === undefined) {
        const map: any = {};
        typ.props.forEach((p: any) => map[p.json] = { key: p.js, typ: p.typ });
        typ.jsonToJS = map;
    }
    return typ.jsonToJS;
}

function jsToJSONProps(typ: any): any {
    if (typ.jsToJSON === undefined) {
        const map: any = {};
        typ.props.forEach((p: any) => map[p.js] = { key: p.json, typ: p.typ });
        typ.jsToJSON = map;
    }
    return typ.jsToJSON;
}

function transform(val: any, typ: any, getProps: any, key: any = ''): any {
    function transformPrimitive(typ: string, val: any): any {
        if (typeof typ === typeof val) return val;
        return invalidValue(typ, val, key);
    }

    function transformUnion(typs: any[], val: any): any {
        // val must validate against one typ in typs
        const l = typs.length;
        for (let i = 0; i < l; i++) {
            const typ = typs[i];
            try {
                return transform(val, typ, getProps);
            } catch (_) {}
        }
        return invalidValue(typs, val);
    }

    function transformEnum(cases: string[], val: any): any {
        if (cases.indexOf(val) !== -1) return val;
        return invalidValue(cases, val);
    }

    function transformArray(typ: any, val: any): any {
        // val must be an array with no invalid elements
        if (!Array.isArray(val)) return invalidValue("array", val);
        return val.map(el => transform(el, typ, getProps));
    }

    function transformDate(val: any): any {
        if (val === null) {
            return null;
        }
        const d = new Date(val);
        if (isNaN(d.valueOf())) {
            return invalidValue("Date", val);
        }
        return d;
    }

    function transformObject(props: { [k: string]: any }, additional: any, val: any): any {
        if (val === null || typeof val !== "object" || Array.isArray(val)) {
            return invalidValue("object", val);
        }
        const result: any = {};
        Object.getOwnPropertyNames(props).forEach(key => {
            const prop = props[key];
            const v = Object.prototype.hasOwnProperty.call(val, key) ? val[key] : undefined;
            result[prop.key] = transform(v, prop.typ, getProps, prop.key);
        });
        Object.getOwnPropertyNames(val).forEach(key => {
            if (!Object.prototype.hasOwnProperty.call(props, key)) {
                result[key] = transform(val[key], additional, getProps, key);
            }
        });
        return result;
    }

    if (typ === "any") return val;
    if (typ === null) {
        if (val === null) return val;
        return invalidValue(typ, val);
    }
    if (typ === false) return invalidValue(typ, val);
    while (typeof typ === "object" && typ.ref !== undefined) {
        typ = typeMap[typ.ref];
    }
    if (Array.isArray(typ)) return transformEnum(typ, val);
    if (typeof typ === "object") {
        return typ.hasOwnProperty("unionMembers") ? transformUnion(typ.unionMembers, val)
            : typ.hasOwnProperty("arrayItems")    ? transformArray(typ.arrayItems, val)
            : typ.hasOwnProperty("props")         ? transformObject(getProps(typ), typ.additional, val)
            : invalidValue(typ, val);
    }
    // Numbers can be parsed by Date but shouldn't be.
    if (typ === Date && typeof val !== "number") return transformDate(val);
    return transformPrimitive(typ, val);
}

function cast<T>(val: any, typ: any): T {
    return transform(val, typ, jsonToJSProps);
}

function uncast<T>(val: T, typ: any): any {
    return transform(val, typ, jsToJSONProps);
}

function a(typ: any) {
    return { arrayItems: typ };
}

function u(...typs: any[]) {
    return { unionMembers: typs };
}

function o(props: any[], additional: any) {
    return { props, additional };
}

function m(additional: any) {
    return { props: [], additional };
}

function r(name: string) {
    return { ref: name };
}

const typeMap: any = {
    "Acteur": o([
        { json: "acteur", js: "acteur", typ: r("ActeurActeur") },
    ], false),
    "ActeurActeur": o([
        { json: "@xmlns", js: "@xmlns", typ: u(undefined, "") },
        { json: "uid", js: "uid", typ: r("Uid") },
        { json: "etatCivil", js: "etatCivil", typ: r("EtatCivil") },
        { json: "profession", js: "profession", typ: r("Profession") },
        { json: "uri_hatvp", js: "uri_hatvp", typ: u(r("Nil"), null, "") },
        { json: "adresses", js: "adresses", typ: u(r("Adresses"), null) },
        { json: "mandats", js: "mandats", typ: u(undefined, r("Mandats")) },
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
    ], false),
    "Adresses": o([
        { json: "adresse", js: "adresse", typ: u(undefined, u(a(m(u(null, ""))), m(u(null, "")))) },
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:nil", js: "@xsi:nil", typ: u(undefined, "") },
    ], false),
    "EtatCivil": o([
        { json: "ident", js: "ident", typ: r("Ident") },
        { json: "infoNaissance", js: "infoNaissance", typ: r("InfoNaissance") },
        { json: "dateDeces", js: "dateDeces", typ: u(r("Nil"), Date, null) },
    ], false),
    "Nil": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: "" },
        { json: "@xsi:nil", js: "@xsi:nil", typ: "" },
    ], false),
    "Ident": o([
        { json: "civ", js: "civ", typ: r("Civ") },
        { json: "prenom", js: "prenom", typ: "" },
        { json: "nom", js: "nom", typ: "" },
        { json: "alpha", js: "alpha", typ: "" },
        { json: "trigramme", js: "trigramme", typ: u(r("Nil"), null, "") },
    ], false),
    "InfoNaissance": o([
        { json: "dateNais", js: "dateNais", typ: u(r("Nil"), Date, null) },
        { json: "villeNais", js: "villeNais", typ: u(r("Nil"), null, "") },
        { json: "depNais", js: "depNais", typ: u(r("Nil"), r("DepNais"), null) },
        { json: "paysNais", js: "paysNais", typ: u(r("Nil"), null, "") },
    ], false),
    "Mandats": o([
        { json: "mandat", js: "mandat", typ: u(a(r("Mandat")), r("Mandat")) },
    ], false),
    "Mandat": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:type", js: "@xsi:type", typ: u(undefined, r("MandatXsiType")) },
        { json: "uid", js: "uid", typ: "" },
        { json: "acteurRef", js: "acteurRef", typ: "" },
        { json: "legislature", js: "legislature", typ: u(null, "") },
        { json: "typeOrgane", js: "typeOrgane", typ: r("CodeType") },
        { json: "dateDebut", js: "dateDebut", typ: Date },
        { json: "datePublication", js: "datePublication", typ: u(Date, null) },
        { json: "dateFin", js: "dateFin", typ: u(Date, null) },
        { json: "preseance", js: "preseance", typ: u(null, "") },
        { json: "nominPrincipale", js: "nominPrincipale", typ: "" },
        { json: "infosQualite", js: "infosQualite", typ: r("InfosQualite") },
        { json: "organes", js: "organes", typ: r("Organes") },
        { json: "libelle", js: "libelle", typ: u(undefined, u(null, "")) },
        { json: "missionSuivanteRef", js: "missionSuivanteRef", typ: u(undefined, u(null, "")) },
        { json: "missionPrecedenteRef", js: "missionPrecedenteRef", typ: u(undefined, u(null, "")) },
        { json: "suppleants", js: "suppleants", typ: u(undefined, u(r("Suppleants"), null)) },
        { json: "chambre", js: "chambre", typ: u(undefined, null) },
        { json: "election", js: "election", typ: u(undefined, r("Election")) },
        { json: "InfosHorsSIAN", js: "InfosHorsSIAN", typ: u(undefined, r("InfosHorsSian")) },
        { json: "mandature", js: "mandature", typ: u(undefined, r("Mandature")) },
        { json: "collaborateurs", js: "collaborateurs", typ: u(undefined, u(a(null), r("Collaborateurs"), null)) },
    ], false),
    "Collaborateurs": o([
        { json: "collaborateur", js: "collaborateur", typ: u(a(r("CollaborateurElement")), r("CollaborateurElement")) },
    ], false),
    "CollaborateurElement": o([
        { json: "qualite", js: "qualite", typ: u(r("Civ"), null) },
        { json: "prenom", js: "prenom", typ: "" },
        { json: "nom", js: "nom", typ: "" },
        { json: "dateDebut", js: "dateDebut", typ: null },
        { json: "dateFin", js: "dateFin", typ: null },
    ], false),
    "Election": o([
        { json: "lieu", js: "lieu", typ: r("ElectionLieu") },
        { json: "causeMandat", js: "causeMandat", typ: u(null, "") },
        { json: "refCirconscription", js: "refCirconscription", typ: u(undefined, u(null, "")) },
    ], false),
    "ElectionLieu": o([
        { json: "region", js: "region", typ: u(r("Libelle"), null) },
        { json: "regionType", js: "regionType", typ: u(r("Type"), null) },
        { json: "departement", js: "departement", typ: u(r("DepNais"), null) },
        { json: "numDepartement", js: "numDepartement", typ: u(null, "") },
        { json: "numCirco", js: "numCirco", typ: u(null, "") },
    ], false),
    "InfosQualite": o([
        { json: "codeQualite", js: "codeQualite", typ: u(null, "") },
        { json: "libQualite", js: "libQualite", typ: "" },
        { json: "libQualiteSex", js: "libQualiteSex", typ: u(null, "") },
    ], false),
    "Mandature": o([
        { json: "datePriseFonction", js: "datePriseFonction", typ: u(Date, null) },
        { json: "causeFin", js: "causeFin", typ: u(null, "") },
        { json: "premiereElection", js: "premiereElection", typ: u(null, "") },
        { json: "placeHemicycle", js: "placeHemicycle", typ: u(null, "") },
        { json: "mandatRemplaceRef", js: "mandatRemplaceRef", typ: u(undefined, u(null, "")) },
        { json: "matriculeVote", js: "matriculeVote", typ: u(undefined, u(null, "")) },
    ], false),
    "Organes": o([
        { json: "organeRef", js: "organeRef", typ: u(a(""), "") },
    ], false),
    "Suppleants": o([
        { json: "suppleant", js: "suppleant", typ: r("Suppleant") },
    ], false),
    "Suppleant": o([
        { json: "dateDebut", js: "dateDebut", typ: Date },
        { json: "dateFin", js: "dateFin", typ: u(Date, null) },
        { json: "suppleantRef", js: "suppleantRef", typ: "" },
    ], false),
    "Profession": o([
        { json: "libelleCourant", js: "libelleCourant", typ: u(r("Nil"), null, "") },
        { json: "socProcINSEE", js: "socProcINSEE", typ: r("SocProcInsee") },
    ], false),
    "SocProcInsee": o([
        { json: "catSocPro", js: "catSocPro", typ: u(r("Nil"), null, "") },
        { json: "famSocPro", js: "famSocPro", typ: u(r("Nil"), r("FamSocProEnum"), null) },
    ], false),
    "Uid": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:type", js: "@xsi:type", typ: r("UidXsiType") },
        { json: "#text", js: "#text", typ: "" },
    ], false),
    "Deport": o([
        { json: "deport", js: "deport", typ: r("DeportClass") },
    ], false),
    "DeportClass": o([
        { json: "@xmlns", js: "@xmlns", typ: "" },
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: "" },
        { json: "uid", js: "uid", typ: "" },
        { json: "chronotag", js: "chronotag", typ: "" },
        { json: "legislature", js: "legislature", typ: "" },
        { json: "refActeur", js: "refActeur", typ: "" },
        { json: "dateCreation", js: "dateCreation", typ: Date },
        { json: "datePublication", js: "datePublication", typ: Date },
        { json: "portee", js: "portee", typ: r("Instance") },
        { json: "lecture", js: "lecture", typ: r("Instance") },
        { json: "instance", js: "instance", typ: r("Instance") },
        { json: "cible", js: "cible", typ: r("Cible") },
        { json: "explication", js: "explication", typ: u(null, "") },
    ], false),
    "Cible": o([
        { json: "type", js: "type", typ: r("Instance") },
        { json: "referenceTextuelle", js: "referenceTextuelle", typ: "" },
        { json: "references", js: "references", typ: null },
    ], false),
    "Instance": o([
        { json: "code", js: "code", typ: "" },
        { json: "libelle", js: "libelle", typ: "" },
    ], false),
    "Organe": o([
        { json: "organe", js: "organe", typ: r("OrganeOrgane") },
    ], false),
    "OrganeOrgane": o([
        { json: "@xmlns", js: "@xmlns", typ: u(undefined, "") },
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: "" },
        { json: "@xsi:type", js: "@xsi:type", typ: u(undefined, r("OrganeXsiType")) },
        { json: "uid", js: "uid", typ: "" },
        { json: "codeType", js: "codeType", typ: r("CodeType") },
        { json: "libelle", js: "libelle", typ: "" },
        { json: "libelleEdition", js: "libelleEdition", typ: u(null, "") },
        { json: "libelleAbrege", js: "libelleAbrege", typ: "" },
        { json: "libelleAbrev", js: "libelleAbrev", typ: "" },
        { json: "viMoDe", js: "viMoDe", typ: r("ViMoDe") },
        { json: "organeParent", js: "organeParent", typ: u(undefined, u(null, "")) },
        { json: "organePrecedentRef", js: "organePrecedentRef", typ: u(undefined, u(null, "")) },
        { json: "chambre", js: "chambre", typ: u(undefined, null) },
        { json: "regime", js: "regime", typ: u(undefined, u(r("Regime"), null)) },
        { json: "legislature", js: "legislature", typ: u(undefined, u(null, "")) },
        { json: "regimeJuridique", js: "regimeJuridique", typ: u(undefined, u(null, "")) },
        { json: "siteInternet", js: "siteInternet", typ: u(undefined, u(null, "")) },
        { json: "nombreReunionsAnnuelles", js: "nombreReunionsAnnuelles", typ: u(undefined, u(null, "")) },
        { json: "secretariat", js: "secretariat", typ: u(undefined, r("Secretariat")) },
        { json: "numero", js: "numero", typ: u(undefined, u(null, "")) },
        { json: "lieu", js: "lieu", typ: u(undefined, r("OrganeLieu")) },
        { json: "positionPolitique", js: "positionPolitique", typ: u(undefined, u(r("PositionPolitique"), null)) },
        { json: "preseance", js: "preseance", typ: u(undefined, u(null, "")) },
        { json: "couleurAssociee", js: "couleurAssociee", typ: u(undefined, u(null, "")) },
        { json: "listePays", js: "listePays", typ: u(undefined, u(r("ListePays"), null)) },
    ], false),
    "OrganeLieu": o([
        { json: "region", js: "region", typ: r("Region") },
        { json: "departement", js: "departement", typ: r("Departement") },
    ], false),
    "Departement": o([
        { json: "codeNatureDep", js: "codeNatureDep", typ: u(r("CodeNatureDep"), null) },
        { json: "code", js: "code", typ: u(null, "") },
        { json: "libelle", js: "libelle", typ: u(r("DepNais"), null) },
    ], false),
    "Region": o([
        { json: "type", js: "type", typ: u(r("Type"), null) },
        { json: "libelle", js: "libelle", typ: u(r("Libelle"), null) },
    ], false),
    "ListePays": o([
        { json: "paysRef", js: "paysRef", typ: u("", a("")) },
    ], false),
    "Secretariat": o([
        { json: "secretaire01", js: "secretaire01", typ: u(null, "") },
        { json: "secretaire02", js: "secretaire02", typ: u(null, "") },
    ], false),
    "ViMoDe": o([
        { json: "dateDebut", js: "dateDebut", typ: u(Date, null) },
        { json: "dateAgrement", js: "dateAgrement", typ: u(Date, null) },
        { json: "dateFin", js: "dateFin", typ: u(Date, null) },
    ], false),
    "MandatWrapper": o([
        { json: "mandat", js: "mandat", typ: r("Mandat") },
    ], false),
    "ActeursEtOrganes": o([
        { json: "export", js: "export", typ: r("Export") },
    ], false),
    "Export": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: "" },
        { json: "organes", js: "organes", typ: r("ExportOrganes") },
        { json: "acteurs", js: "acteurs", typ: r("Acteurs") },
    ], false),
    "Acteurs": o([
        { json: "acteur", js: "acteur", typ: a(r("ActeurElement")) },
    ], false),
    "ActeurElement": o([
        { json: "uid", js: "uid", typ: r("Uid") },
        { json: "etatCivil", js: "etatCivil", typ: r("EtatCivil") },
        { json: "uri_hatvp", js: "uri_hatvp", typ: u(undefined, u(null, "")) },
        { json: "mandats", js: "mandats", typ: r("Mandats") },
    ], false),
    "InfosHorsSian": o([
        { json: "HATVP_URI", js: "HATVP_URI", typ: null },
    ], false),
    "ExportOrganes": o([
        { json: "organe", js: "organe", typ: a(r("OrganeElement")) },
    ], false),
    "OrganeElement": o([
        { json: "@xsi:type", js: "@xsi:type", typ: r("OrganeXsiType") },
        { json: "uid", js: "uid", typ: "" },
        { json: "codeType", js: "codeType", typ: r("CodeType") },
        { json: "libelle", js: "libelle", typ: "" },
        { json: "libelleEdition", js: "libelleEdition", typ: u(null, "") },
        { json: "libelleAbrege", js: "libelleAbrege", typ: "" },
        { json: "libelleAbrev", js: "libelleAbrev", typ: "" },
        { json: "viMoDe", js: "viMoDe", typ: r("ViMoDe") },
        { json: "regimeJuridique", js: "regimeJuridique", typ: u(undefined, u(null, "")) },
        { json: "regime", js: "regime", typ: u(undefined, u(r("Regime"), null)) },
        { json: "legislature", js: "legislature", typ: u(undefined, u(null, "")) },
        { json: "secretariat", js: "secretariat", typ: u(undefined, r("Secretariat")) },
        { json: "positionPolitique", js: "positionPolitique", typ: u(undefined, u(r("PositionPolitique"), null)) },
    ], false),
    "Civ": [
        "M.",
        "Mme",
    ],
    "DepNais": [
        "Ain",
        "Aisne",
        "Allier",
        "Alpes-de-Haute-Provence",
        "Alpes-Maritimes",
        "Ardennes",
        "Ardèche",
        "Ariège",
        "Aube",
        "Aude",
        "Aveyron",
        "Bas-Rhin",
        "Bouches-du-Rhône",
        "Calvados",
        "Cantal",
        "Charente",
        "Charente-Maritime",
        "Cher",
        "Comores",
        "Corrèze",
        "Corse",
        "Corse-du-Sud",
        "Creuse",
        "Côte-d'Or",
        "Côtes-d'Armor",
        "Côtes-du-Nord",
        "Deux-Sèvres",
        "Dordogne",
        "Doubs",
        "Drôme",
        "Essonne",
        "Eure",
        "Eure-et-Loir",
        "Finistère",
        "Français établis hors de France",
        "Gard",
        "Gers",
        "Gironde",
        "Guadeloupe",
        "Guyane",
        "Haut-Rhin",
        "Haute-Corse",
        "Haute-Garonne",
        "Haute-Loire",
        "Haute-Marne",
        "Haute-Savoie",
        "Haute-Saône",
        "Haute-Vienne",
        "Hautes-Alpes",
        "Hautes-Pyrénées",
        "Hauts-de-Seine",
        "Hérault",
        "Ille-et-Vilaine",
        "Indre",
        "Indre-et-Loire",
        "Isère",
        "Jura",
        "Landes",
        "Loir-et-Cher",
        "Loire",
        "Loire-Atlantique",
        "Loiret",
        "Lot",
        "Lot-et-Garonne",
        "Lozère",
        "Maine-et-Loire",
        "Manche",
        "Marne",
        "Martinique",
        "Mayenne",
        "Mayotte",
        "Meurthe-et-Moselle",
        "Meuse",
        "Morbihan",
        "Moselle",
        "Nièvre",
        "Nord",
        "Nouvelle-Calédonie",
        "Oise",
        "Orne",
        "Paris",
        "Pas-de-Calais",
        "Polynésie Française",
        "Puy-de-Dôme",
        "Pyrénées-Atlantiques",
        "Pyrénées-Orientales",
        "Rhône",
        "Réunion",
        "Saint-Barthélemy et Saint-Martin",
        "Saint-Martin",
        "Saint-Pierre-et-Miquelon",
        "Sarthe",
        "Savoie",
        "Saône-et-Loire",
        "Seine-et-Marne",
        "Seine-Inférieure",
        "Seine-Maritime",
        "Seine-Saint-Denis",
        "Somme",
        "Tarn",
        "Tarn-et-Garonne",
        "Territoire de Belfort",
        "Val-d'Oise",
        "Val-de-Marne",
        "Var",
        "Vaucluse",
        "Vendée",
        "Vienne",
        "Vosges",
        "Wallis-et-Futuna",
        "Yonne",
        "Yvelines",
    ],
    "MandatXsiType": [
        "MandatAvecSuppleant_Type",
        "MandatMission_Type",
        "MandatParlementaire_type",
        "MandatSimple_Type",
    ],
    "Libelle": [
        "Auvergne-Rhône-Alpes",
        "Bourgogne-Franche-Comté",
        "Bretagne",
        "Centre-Val de Loire",
        "Corse",
        "Français établis hors de France",
        "Grand Est",
        "Guadeloupe",
        "Guyane",
        "Hauts-de-France",
        "Ile-de-France",
        "Martinique",
        "Mayotte",
        "Normandie",
        "Nouvelle-Aquitaine",
        "Nouvelle-Calédonie",
        "Occitanie",
        "Pays de la Loire",
        "Polynésie française",
        "Provence-Alpes-Côte d'Azur",
        "Réunion",
        "Saint-Barthélemy et Saint-Martin",
        "Saint-Pierre-et-Miquelon",
        "Wallis-et-Futuna",
    ],
    "Type": [
        "Collectivités d'outre-mer et Nouvelle-Calédonie",
        "Dom",
        "Français établis hors de France",
        "Métropolitain",
    ],
    "CodeType": [
        "API",
        "ASSEMBLEE",
        "BUREAU",
        "CIRCONSCRIPTION",
        "CJR",
        "CMP",
        "CNPE",
        "CNPS",
        "COMNL",
        "COMPER",
        "COMSENAT",
        "COMSPSENAT",
        "CONFPT",
        "CONSTITU",
        "DELEG",
        "DELEGBUREAU",
        "DELEGSENAT",
        "GA",
        "GE",
        "GEVI",
        "GOUVERNEMENT",
        "GP",
        "GROUPESENAT",
        "HCJ",
        "MINISTERE",
        "MISEVAL",
        "MISINFO",
        "MISINFOCOM",
        "MISINFOPRE",
        "OFFPAR",
        "ORGAINT",
        "ORGEXTPARL",
        "PARPOL",
        "PRESREP",
        "SENAT",
    ],
    "FamSocProEnum": [
        "Agriculteurs exploitants",
        "Artisans, commerçants et chefs d'entreprise",
        "Autres personnes sans activité professionnelle",
        "Cadres et professions intellectuelles supérieures",
        "Employés",
        "Ouvriers",
        "Professions Intermédiaires",
        "Retraités",
        "Sans profession déclarée",
    ],
    "UidXsiType": [
        "IdActeur_type",
    ],
    "OrganeXsiType": [
        "GroupePolitique_type",
        "OrganeCirconscription_type",
        "OrganeExterne_Type",
        "OrganeExtraParlementaire_type",
        "OrganeParlementaireInternational",
        "OrganeParlementaire_Type",
    ],
    "CodeNatureDep": [
        "H",
        "M",
        "O",
        "T",
    ],
    "PositionPolitique": [
        "Majoritaire",
        "Minoritaire",
        "Opposition",
    ],
    "Regime": [
        "Assemblée consultative provisoire",
        "Assemblées nationales constituantes",
        "3ème République",
        "4ème République",
        "5ème République",
    ],
};
