// To parse this data:
//
//   import { Amendements, AmendementWrapper, Convert, TexteLegislatif } from "./file";
//
//   const amendements = Convert.toAmendements(json);
//   const amendementWrapper = Convert.toAmendementWrapper(json);
//   const texteLegislatif = Convert.toTexteLegislatif(json);
//
// These functions will throw an error if the JSON doesn't
// match the expected interface, even if the JSON is valid.

export interface Amendements {
  textesLegislatifs: TexteLegislatif[]
}

export interface TexteLegislatif {
    "@xmlns:xsi":       string;
    refTexteLegislatif: string;
    amendements:        AmendementsClass;
}

export interface AmendementsClass {
    amendement: Amendement[];
}

export interface AmendementWrapper {
  amendement: Amendement
}

export interface Amendement {
    "@xmlns":                         string;
    uid:                              string;
    chronotag:                        string;
    legislature:                      string;
    identification:                   Identification;
    examenRef:                        string;
    texteLegislatifRef:               string;
    triAmendement:                    AmendementParentRefClass | string;
    cardinaliteAmdtMultiples:         AmendementParentRefClass | string;
    amendementParentRef:              AmendementParentRefClass | string;
    signataires:                      Signataires;
    pointeurFragmentTexte:            PointeurFragmentTexte;
    corps:                            Corps;
    cycleDeVie:                       CycleDeVie;
    representations:                  Representations | null;
    seanceDiscussionRef:              AmendementParentRefClass | string;
    article99:                        string;
    loiReference:                     LoiReference;
    discussionCommune:                Discussion;
    discussionIdentique:              Discussion;
    accordGouvernementDepotHorsDelai: AccordGouvernementDepotHorsDelai;
}

export enum AccordGouvernementDepotHorsDelai {
    Accord = "Accord",
    SansObjet = "Sans objet",
}

export interface AmendementParentRefClass {
    "@xmlns:xsi": string;
    "@xsi:nil":   string;
}

export interface Corps {
    cartoucheInformatif?: AmendementParentRefClass | string;
    contenuAuteur?:       ContenuAuteur;
    "@xmlns:xsi"?:        string;
    "@xsi:nil"?:          string;
}

export interface ContenuAuteur {
    dispositif?:               string;
    avantAppel?:               AvantAppel;
    exposeSommaire?:           string;
    annexeExposeSommaire?:     AmendementParentRefClass;
    "@xmlns:xsi"?:             string;
    "@xsi:nil"?:               string;
    dispositifAmdtCreditPLF?:  DispositifAmdtCreditPLF;
    dispositifAmdtCreditPLFR?: DispositifAmdtCreditPLFR;
}

export interface AvantAppel {
    "@xmlns:xsi"?: string;
    "@xsi:nil"?:   string;
    dispositif?:   string;
    dispositifAmdtCreditPLF?:  DispositifAmdtCreditPLF;
    dispositifAmdtCreditPLFR?: DispositifAmdtCreditPLFR;
}

export interface DispositifAmdtCreditPLF {
    listeProgrammes?: DispositifAmdtCreditPLFListeProgrammes;
    soldeAE?:         string;
    soldeCP?:         string;
    cpEgalAe:         string;
}

export interface DispositifAmdtCreditPLFListeProgrammes {
    programme: PurpleProgramme[];
}

export interface PurpleProgramme {
    libelle:                string;
    programmeRef:           string;
    autorisationEngagement: string;
    creditPaiement:         string;
    action:                 Action;
    lignesCredits:          PurpleLignesCredits;
}

export enum Action {
    Création = "création",
    Modification = "modification",
    Suppression = "suppression",
}

export interface PurpleLignesCredits {
    "@xmlns:xsi"?: string;
    "@xsi:nil"?:   string;
    ligneCredit?:  LigneCreditClass[] | LigneCreditClass;
}

export interface LigneCreditClass {
    id:                     string;
    libelle:                string;
    autorisationEngagement: string;
    creditPaiement:         string;
    action:                 Action;
}

export interface DispositifAmdtCreditPLFR {
    listeProgrammes?: DispositifAmdtCreditPLFRListeProgrammes;
    totalAE?:         Total;
    totalCP?:         Total;
    cpEgalAe:         string;
}

export interface DispositifAmdtCreditPLFRListeProgrammes {
    programme: LigneCreditElement[];
}

export interface LigneCreditLignesCredits {
    "@xmlns:xsi"?: string;
    "@xsi:nil"?:   string;
    ligneCredit?:  LigneCreditElement[] | LigneCreditElement;
}

export interface LigneCreditElement {
    libelle:                              string;
    programmeRef?:                        string;
    autorisationEngagementSupplementaire: string;
    creditPaiementSupplementaire:         string;
    autorisationEngagementAnnule:         string;
    creditPaiementAnnule:                 string;
    action:                               Action;
    lignesCredits?:                       LigneCreditLignesCredits;
    id?:                                  string;
}

export interface Total {
    supplementaire: string;
    annule:         string;
    solde:          string;
}

export interface CycleDeVie {
    dateDepot:          AmendementParentRefClass | Date;
    datePublication:    AmendementParentRefClass | Date;
    soumisArticle40:    string;
    etatDesTraitements: EtatDESTraitements;
    dateSort:           AmendementParentRefClass | Date;
    sort:               AmendementParentRefClass | SortEnum;
}

export interface EtatDESTraitements {
    etat:     Etat;
    sousEtat: SousEtat;
}

export interface Etat {
    code:    Code;
    libelle: Libelle;
}

export enum Code {
    AC = "AC",
    At = "AT",
    Di = "DI",
    Effacé = "effacé",
    Er = "ER",
    Et = "ET",
    IR = "IR",
    Irr127 = "IRR127",
    Irr20 = "IRR20",
    Irr37 = "IRR37",
    Irr38 = "IRR38",
    Irr41 = "IRR41",
    Irr42 = "IRR42",
    Irr45 = "IRR45",
    Irr48 = "IRR48",
    Irr53 = "IRR53",
    Irrai = "IRRAI",
    Irraif = "IRRAIF",
    Irrd = "IRRD",
    Irrhc = "IRRHC",
    Irrhd = "IRRHD",
    Irrsa = "IRRSA",
    R = "R",
    Rt = "RT",
    T = "T",
}

export enum Libelle {
    ADiscuter = "A discuter",
    ADéposer = "A déposer",
    Discuté = "Discuté",
    Effacé = "effacé",
    EnRecevabilité = "En recevabilité",
    EnTraitement = "En traitement",
    Irrecevable = "Irrecevable",
    Irrecevable40 = "Irrecevable 40",
    Retiré = "Retiré",
}

export interface SousEtat {
    code?:         Code;
    libelle?:      SortEnum;
    "@xmlns:xsi"?: string;
    "@xsi:nil"?:   string;
}

export enum SortEnum {
    Adopté = "Adopté",
    AutreIrrecevabilité = "Autre irrecevabilité",
    AutreIrrecevabilité40 = "Autre irrecevabilité 40",
    AutresIrrLOLF = "Autres irr LOLF",
    AutresIrrLOLFSS = "Autres irr LOLFSS",
    Cavalier45 = "Cavalier (45)",
    CavalierBudgétaire = "Cavalier budgétaire",
    CavalierSocial = "Cavalier social",
    ChampDeLHabilitation38 = "Champ de l'habilitation (38)",
    Charge = "Charge",
    Cmp45 = "CMP (45)",
    Crédits = "Crédits",
    DispositionRéglementaire37 = "Disposition réglementaire (37)",
    DomaineDeLaLoi41 = "Domaine de la loi (41)",
    DomaineLoiOrganique127 = "Domaine loi organique (127)",
    Doublon = "Doublon",
    Entonnoir45 = "Entonnoir (45)",
    Gage = "Gage",
    HorsChamp = "Hors champ",
    HorsDélais = "Hors-délais",
    Injonction20 = "Injonction (20)",
    IrrEnPremièrePartie = "Irr en première partie",
    IrrEnSecondePartie = "Irr en seconde partie",
    Lolf = "LOLF",
    Lolfss = "LOLFSS",
    NonSoutenu = "Non soutenu",
    OrdreDuJour48 = "Ordre du jour (48)",
    R = "R",
    RatificationTraité53 = "Ratification traité (53)",
    Rejeté = "Rejeté",
    Retiré = "Retiré",
    RetiréAprèsPublication = "Retiré après publication",
    RetiréAvantPublication = "Retiré avant publication",
    SatisfaitOuInopérant42 = "Satisfait ou inopérant (42)",
    SousAmendement98 = "Sous-amendement (98)",
    Tombé = "Tombé",
}

export interface Discussion {
    "@xmlns:xsi"?: string;
    "@xsi:nil"?:   string;
    idDiscussion?: string;
    typePosition?: TypePosition;
}

export enum TypePosition {
    Dernier = "Dernier",
    Milieu = "Milieu",
    Premier = "Premier",
}

export interface Identification {
    numeroLong:          string;
    numeroOrdreDepot:    string;
    prefixeOrganeExamen: string;
    numeroRect:          string;
}

export interface LoiReference {
    "@xmlns:xsi"?:    string;
    "@xsi:nil"?:      string;
    codeLoi?:         AmendementParentRefClass | null | string;
    divisionCodeLoi?: AmendementParentRefClass | null | string;
}

export interface PointeurFragmentTexte {
    partieAmendableRef:  AmendementParentRefClass;
    division?:           Division;
    amendementStandard?: AmendementStandard;
    missionVisee?:       MissionVisee;
}

export interface AmendementStandard {
    alinea: Alinea;
}

export interface Alinea {
    "@xmlns:xsi"?:      string;
    "@xsi:nil"?:        string;
    avant_A_Apres?:     AmendementParentRefClass | null | string;
    numero?:            AmendementParentRefClass | string;
    alineaDesignation?: string;
}

export interface Division {
    titre:                    null | string;
    articleDesignationCourte: null | string;
    articleDesignation:       null | string;
    type:                     DivisionType;
    avant_A_Apres:            AvantAApres;
    divisionRattachee:        AmendementParentRefClass | string;
    articleAdditionnel:       string;
    chapitreAdditionnel:      string;
    urlDivisionTexteVise:     AmendementParentRefClass | string;
}

export enum AvantAApres {
    A = "A",
    Après = "Après",
    Avant = "Avant",
}

export enum DivisionType {
    Annexe = "ANNEXE",
    Article = "ARTICLE",
    Chapitre = "CHAPITRE",
    Titre = "TITRE",
}

export interface MissionVisee {
    codeEtat:          CodeEtat;
    codeMissionMinefi: string;
    libelleMission:    string;
    missionRef:        string;
}

export enum CodeEtat {
    B = "B",
    C = "C",
    D = "D",
}

export interface Representations {
    representation: Representation;
}

export interface Representation {
    nom:                     Nom;
    typeMime:                TypeMIME;
    statutRepresentation:    StatutRepresentation;
    repSource:               AmendementParentRefClass;
    offset:                  AmendementParentRefClass;
    contenu:                 Contenu;
    dateDispoRepresentation: AmendementParentRefClass;
}

export interface Contenu {
    documentURI: string;
}

export enum Nom {
    PDF = "PDF",
}

export interface StatutRepresentation {
    verbatim:       string;
    canonique:      string;
    officielle:     string;
    transcription:  string;
    enregistrement: string;
}

export interface TypeMIME {
    type:    TypeMIMEType;
    subType: Nom;
}

export enum TypeMIMEType {
    Application = "application",
}

export interface Signataires {
    auteur:        Auteur;
    cosignataires: Cosignataires;
    suffixe:       AmendementParentRefClass | string;
    libelle:       null | string;
}

export interface Auteur {
    typeAuteur:                TypeAuteur;
    gouvernementRef:           AmendementParentRefClass | string;
    acteurRef:                 AmendementParentRefClass | string;
    groupePolitiqueRef:        AmendementParentRefClass | string;
    auteurRapporteurOrganeRef: AmendementParentRefClass | string;
}

export enum TypeAuteur {
    Député = "Député",
    Gouvernement = "Gouvernement",
    Rapporteur = "Rapporteur",
}

export interface Cosignataires {
    "@xmlns:xsi"?: string;
    "@xsi:nil"?:   string;
    acteurRef?:    string[] | string;
}

// Converts JSON strings to/from your types
// and asserts the results of JSON.parse at runtime
export class Convert {
    public static toAmendements(json: string): Amendements {
        return cast(JSON.parse(json), r("Amendements"));
    }

    public static amendementsToJson(value: Amendements): string {
        return JSON.stringify(uncast(value, r("Amendements")), null, 2);
    }

    public static toTexteLegislatif(json: string): TexteLegislatif {
        return cast(JSON.parse(json), r("TexteLegislatif"));
    }

    public static texteLegislatifToJson(value: TexteLegislatif): string {
        return JSON.stringify(uncast(value, r("TexteLegislatif")), null, 2);
    }

    public static toAmendementWrapper(json: string): AmendementWrapper {
        return cast(JSON.parse(json), r("AmendementWrapper"));
    }

    public static amendementWrapperToJson(value: AmendementWrapper): string {
        return JSON.stringify(uncast(value, r("AmendementWrapper")), null, 2);
    }
}

function invalidValue(typ: any, val: any, key: any = ''): never {
    if (key) {
        throw Error(`Invalid value for key "${key}". Expected type ${JSON.stringify(typ)} but got ${JSON.stringify(val)}`);
    }
    throw Error(`Invalid value ${JSON.stringify(val)} for type ${JSON.stringify(typ)}`, );
}

function jsonToJSProps(typ: any): any {
    if (typ.jsonToJS === undefined) {
        const map: any = {};
        typ.props.forEach((p: any) => map[p.json] = { key: p.js, typ: p.typ });
        typ.jsonToJS = map;
    }
    return typ.jsonToJS;
}

function jsToJSONProps(typ: any): any {
    if (typ.jsToJSON === undefined) {
        const map: any = {};
        typ.props.forEach((p: any) => map[p.js] = { key: p.json, typ: p.typ });
        typ.jsToJSON = map;
    }
    return typ.jsToJSON;
}

function transform(val: any, typ: any, getProps: any, key: any = ''): any {
    function transformPrimitive(typ: string, val: any): any {
        if (typeof typ === typeof val) return val;
        return invalidValue(typ, val, key);
    }

    function transformUnion(typs: any[], val: any): any {
        // val must validate against one typ in typs
        const l = typs.length;
        for (let i = 0; i < l; i++) {
            const typ = typs[i];
            try {
                return transform(val, typ, getProps);
            } catch (_) {}
        }
        return invalidValue(typs, val);
    }

    function transformEnum(cases: string[], val: any): any {
        if (cases.indexOf(val) !== -1) return val;
        return invalidValue(cases, val);
    }

    function transformArray(typ: any, val: any): any {
        // val must be an array with no invalid elements
        if (!Array.isArray(val)) return invalidValue("array", val);
        return val.map(el => transform(el, typ, getProps));
    }

    function transformDate(val: any): any {
        if (val === null) {
            return null;
        }
        const d = new Date(val);
        if (isNaN(d.valueOf())) {
            return invalidValue("Date", val);
        }
        return d;
    }

    function transformObject(props: { [k: string]: any }, additional: any, val: any): any {
        if (val === null || typeof val !== "object" || Array.isArray(val)) {
            return invalidValue("object", val);
        }
        const result: any = {};
        Object.getOwnPropertyNames(props).forEach(key => {
            const prop = props[key];
            const v = Object.prototype.hasOwnProperty.call(val, key) ? val[key] : undefined;
            result[prop.key] = transform(v, prop.typ, getProps, prop.key);
        });
        Object.getOwnPropertyNames(val).forEach(key => {
            if (!Object.prototype.hasOwnProperty.call(props, key)) {
                result[key] = transform(val[key], additional, getProps, key);
            }
        });
        return result;
    }

    if (typ === "any") return val;
    if (typ === null) {
        if (val === null) return val;
        return invalidValue(typ, val);
    }
    if (typ === false) return invalidValue(typ, val);
    while (typeof typ === "object" && typ.ref !== undefined) {
        typ = typeMap[typ.ref];
    }
    if (Array.isArray(typ)) return transformEnum(typ, val);
    if (typeof typ === "object") {
        return typ.hasOwnProperty("unionMembers") ? transformUnion(typ.unionMembers, val)
            : typ.hasOwnProperty("arrayItems")    ? transformArray(typ.arrayItems, val)
            : typ.hasOwnProperty("props")         ? transformObject(getProps(typ), typ.additional, val)
            : invalidValue(typ, val);
    }
    // Numbers can be parsed by Date but shouldn't be.
    if (typ === Date && typeof val !== "number") return transformDate(val);
    return transformPrimitive(typ, val);
}

function cast<T>(val: any, typ: any): T {
    return transform(val, typ, jsonToJSProps);
}

function uncast<T>(val: T, typ: any): any {
    return transform(val, typ, jsToJSONProps);
}

function a(typ: any) {
    return { arrayItems: typ };
}

function u(...typs: any[]) {
    return { unionMembers: typs };
}

function o(props: any[], additional: any) {
    return { props, additional };
}

// function m(additional: any) {
//     return { props: [], additional };
// }

function r(name: string) {
    return { ref: name };
}

const typeMap: any = {
    "Amendements": o([
        { json: "textesLegislatifs", js: "textesLegislatifs", typ: a(r("TexteLegislatif")) },
    ], false),
    "TexteLegislatif": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: "" },
        { json: "refTexteLegislatif", js: "refTexteLegislatif", typ: "" },
        { json: "amendements", js: "amendements", typ: r("AmendementsClass") },
    ], false),
    "AmendementsClass": o([
        { json: "amendement", js: "amendement", typ: a(r("Amendement")) },
    ], false),
    "AmendementWrapper": o([
        { json: "amendement", js: "amendement", typ: r("Amendement") },
    ], false),
    "Amendement": o([
        { json: "@xmlns", js: "@xmlns", typ: "" },
        { json: "uid", js: "uid", typ: "" },
        { json: "chronotag", js: "chronotag", typ: "" },
        { json: "legislature", js: "legislature", typ: "" },
        { json: "identification", js: "identification", typ: r("Identification") },
        { json: "examenRef", js: "examenRef", typ: "" },
        { json: "texteLegislatifRef", js: "texteLegislatifRef", typ: "" },
        { json: "triAmendement", js: "triAmendement", typ: u(r("AmendementParentRefClass"), "") },
        { json: "cardinaliteAmdtMultiples", js: "cardinaliteAmdtMultiples", typ: u(r("AmendementParentRefClass"), "") },
        { json: "amendementParentRef", js: "amendementParentRef", typ: u(r("AmendementParentRefClass"), "") },
        { json: "signataires", js: "signataires", typ: r("Signataires") },
        { json: "pointeurFragmentTexte", js: "pointeurFragmentTexte", typ: r("PointeurFragmentTexte") },
        { json: "corps", js: "corps", typ: r("Corps") },
        { json: "cycleDeVie", js: "cycleDeVie", typ: r("CycleDeVie") },
        { json: "representations", js: "representations", typ: u(r("Representations"), null) },
        { json: "seanceDiscussionRef", js: "seanceDiscussionRef", typ: u(r("AmendementParentRefClass"), "") },
        { json: "article99", js: "article99", typ: "" },
        { json: "loiReference", js: "loiReference", typ: r("LoiReference") },
        { json: "discussionCommune", js: "discussionCommune", typ: r("Discussion") },
        { json: "discussionIdentique", js: "discussionIdentique", typ: r("Discussion") },
        { json: "accordGouvernementDepotHorsDelai", js: "accordGouvernementDepotHorsDelai", typ: r("AccordGouvernementDepotHorsDelai") },
    ], false),
    "AmendementParentRefClass": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: "" },
        { json: "@xsi:nil", js: "@xsi:nil", typ: "" },
    ], false),
    "Corps": o([
        { json: "cartoucheInformatif", js: "cartoucheInformatif", typ: u(undefined, u(r("AmendementParentRefClass"), "")) },
        { json: "contenuAuteur", js: "contenuAuteur", typ: u(undefined, r("ContenuAuteur")) },
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:nil", js: "@xsi:nil", typ: u(undefined, "") },
    ], false),
    "ContenuAuteur": o([
        { json: "dispositif", js: "dispositif", typ: u(undefined, "") },
        { json: "avantAppel", js: "avantAppel", typ: u(undefined, r("AvantAppel")) },
        { json: "exposeSommaire", js: "exposeSommaire", typ: u(undefined, "") },
        { json: "annexeExposeSommaire", js: "annexeExposeSommaire", typ: u(undefined, r("AmendementParentRefClass")) },
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:nil", js: "@xsi:nil", typ: u(undefined, "") },
        { json: "dispositifAmdtCreditPLF", js: "dispositifAmdtCreditPLF", typ: u(undefined, r("DispositifAmdtCreditPLF")) },
        { json: "dispositifAmdtCreditPLFR", js: "dispositifAmdtCreditPLFR", typ: u(undefined, r("DispositifAmdtCreditPLFR")) },
    ], false),
    "AvantAppel": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:nil", js: "@xsi:nil", typ: u(undefined, "") },
        { json: "dispositif", js: "dispositif", typ: u(undefined, "") },
        { json: "dispositifAmdtCreditPLF", js: "dispositifAmdtCreditPLF", typ: u(undefined, r("DispositifAmdtCreditPLF")) },
        { json: "dispositifAmdtCreditPLFR", js: "dispositifAmdtCreditPLFR", typ: u(undefined, r("DispositifAmdtCreditPLFR")) },
    ], false),
    "DispositifAmdtCreditPLF": o([
        { json: "listeProgrammes", js: "listeProgrammes", typ: u(undefined, r("DispositifAmdtCreditPLFListeProgrammes")) },
        { json: "soldeAE", js: "soldeAE", typ: u(undefined, "") },
        { json: "soldeCP", js: "soldeCP", typ: u(undefined, "") },
        { json: "cpEgalAe", js: "cpEgalAe", typ: "" },
    ], false),
    "DispositifAmdtCreditPLFListeProgrammes": o([
        { json: "programme", js: "programme", typ: a(r("PurpleProgramme")) },
    ], false),
    "PurpleProgramme": o([
        { json: "libelle", js: "libelle", typ: "" },
        { json: "programmeRef", js: "programmeRef", typ: "" },
        { json: "autorisationEngagement", js: "autorisationEngagement", typ: "" },
        { json: "creditPaiement", js: "creditPaiement", typ: "" },
        { json: "action", js: "action", typ: r("Action") },
        { json: "lignesCredits", js: "lignesCredits", typ: r("PurpleLignesCredits") },
    ], false),
    "PurpleLignesCredits": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:nil", js: "@xsi:nil", typ: u(undefined, "") },
        { json: "ligneCredit", js: "ligneCredit", typ: u(undefined, u(a(r("LigneCreditClass")), r("LigneCreditClass"))) },
    ], false),
    "LigneCreditClass": o([
        { json: "id", js: "id", typ: "" },
        { json: "libelle", js: "libelle", typ: "" },
        { json: "autorisationEngagement", js: "autorisationEngagement", typ: "" },
        { json: "creditPaiement", js: "creditPaiement", typ: "" },
        { json: "action", js: "action", typ: r("Action") },
    ], false),
    "DispositifAmdtCreditPLFR": o([
        { json: "listeProgrammes", js: "listeProgrammes", typ: u(undefined, r("DispositifAmdtCreditPLFRListeProgrammes")) },
        { json: "totalAE", js: "totalAE", typ: u(undefined, r("Total")) },
        { json: "totalCP", js: "totalCP", typ: u(undefined, r("Total")) },
        { json: "cpEgalAe", js: "cpEgalAe", typ: "" },
    ], false),
    "DispositifAmdtCreditPLFRListeProgrammes": o([
        { json: "programme", js: "programme", typ: a(r("LigneCreditElement")) },
    ], false),
    "LigneCreditLignesCredits": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:nil", js: "@xsi:nil", typ: u(undefined, "") },
        { json: "ligneCredit", js: "ligneCredit", typ: u(undefined, u(a(r("LigneCreditElement")), r("LigneCreditElement"))) },
    ], false),
    "LigneCreditElement": o([
        { json: "libelle", js: "libelle", typ: "" },
        { json: "programmeRef", js: "programmeRef", typ: u(undefined, "") },
        { json: "autorisationEngagementSupplementaire", js: "autorisationEngagementSupplementaire", typ: "" },
        { json: "creditPaiementSupplementaire", js: "creditPaiementSupplementaire", typ: "" },
        { json: "autorisationEngagementAnnule", js: "autorisationEngagementAnnule", typ: "" },
        { json: "creditPaiementAnnule", js: "creditPaiementAnnule", typ: "" },
        { json: "action", js: "action", typ: r("Action") },
        { json: "lignesCredits", js: "lignesCredits", typ: u(undefined, r("LigneCreditLignesCredits")) },
        { json: "id", js: "id", typ: u(undefined, "") },
    ], false),
    "Total": o([
        { json: "supplementaire", js: "supplementaire", typ: "" },
        { json: "annule", js: "annule", typ: "" },
        { json: "solde", js: "solde", typ: "" },
    ], false),
    "CycleDeVie": o([
        { json: "dateDepot", js: "dateDepot", typ: u(r("AmendementParentRefClass"), Date) },
        { json: "datePublication", js: "datePublication", typ: u(r("AmendementParentRefClass"), Date) },
        { json: "soumisArticle40", js: "soumisArticle40", typ: "" },
        { json: "etatDesTraitements", js: "etatDesTraitements", typ: r("EtatDESTraitements") },
        { json: "dateSort", js: "dateSort", typ: u(r("AmendementParentRefClass"), Date) },
        { json: "sort", js: "sort", typ: u(r("AmendementParentRefClass"), r("SortEnum")) },
    ], false),
    "EtatDESTraitements": o([
        { json: "etat", js: "etat", typ: r("Etat") },
        { json: "sousEtat", js: "sousEtat", typ: r("SousEtat") },
    ], false),
    "Etat": o([
        { json: "code", js: "code", typ: r("Code") },
        { json: "libelle", js: "libelle", typ: r("Libelle") },
    ], false),
    "SousEtat": o([
        { json: "code", js: "code", typ: u(undefined, r("Code")) },
        { json: "libelle", js: "libelle", typ: u(undefined, r("SortEnum")) },
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:nil", js: "@xsi:nil", typ: u(undefined, "") },
    ], false),
    "Discussion": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:nil", js: "@xsi:nil", typ: u(undefined, "") },
        { json: "idDiscussion", js: "idDiscussion", typ: u(undefined, "") },
        { json: "typePosition", js: "typePosition", typ: u(undefined, r("TypePosition")) },
    ], false),
    "Identification": o([
        { json: "numeroLong", js: "numeroLong", typ: "" },
        { json: "numeroOrdreDepot", js: "numeroOrdreDepot", typ: "" },
        { json: "prefixeOrganeExamen", js: "prefixeOrganeExamen", typ: "" },
        { json: "numeroRect", js: "numeroRect", typ: "" },
    ], false),
    "LoiReference": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:nil", js: "@xsi:nil", typ: u(undefined, "") },
        { json: "codeLoi", js: "codeLoi", typ: u(undefined, u(r("AmendementParentRefClass"), null, "")) },
        { json: "divisionCodeLoi", js: "divisionCodeLoi", typ: u(undefined, u(r("AmendementParentRefClass"), null, "")) },
    ], false),
    "PointeurFragmentTexte": o([
        { json: "partieAmendableRef", js: "partieAmendableRef", typ: r("AmendementParentRefClass") },
        { json: "division", js: "division", typ: u(undefined, r("Division")) },
        { json: "amendementStandard", js: "amendementStandard", typ: u(undefined, r("AmendementStandard")) },
        { json: "missionVisee", js: "missionVisee", typ: u(undefined, r("MissionVisee")) },
    ], false),
    "AmendementStandard": o([
        { json: "alinea", js: "alinea", typ: r("Alinea") },
    ], false),
    "Alinea": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:nil", js: "@xsi:nil", typ: u(undefined, "") },
        { json: "avant_A_Apres", js: "avant_A_Apres", typ: u(undefined, u(r("AmendementParentRefClass"), null, "")) },
        { json: "numero", js: "numero", typ: u(undefined, u(r("AmendementParentRefClass"), "")) },
        { json: "alineaDesignation", js: "alineaDesignation", typ: u(undefined, "") },
    ], false),
    "Division": o([
        { json: "titre", js: "titre", typ: u(null, "") },
        { json: "articleDesignationCourte", js: "articleDesignationCourte", typ: u(null, "") },
        { json: "articleDesignation", js: "articleDesignation", typ: u(null, "") },
        { json: "type", js: "type", typ: r("DivisionType") },
        { json: "avant_A_Apres", js: "avant_A_Apres", typ: r("AvantAApres") },
        { json: "divisionRattachee", js: "divisionRattachee", typ: u(r("AmendementParentRefClass"), "") },
        { json: "articleAdditionnel", js: "articleAdditionnel", typ: "" },
        { json: "chapitreAdditionnel", js: "chapitreAdditionnel", typ: "" },
        { json: "urlDivisionTexteVise", js: "urlDivisionTexteVise", typ: u(r("AmendementParentRefClass"), "") },
    ], false),
    "MissionVisee": o([
        { json: "codeEtat", js: "codeEtat", typ: r("CodeEtat") },
        { json: "codeMissionMinefi", js: "codeMissionMinefi", typ: "" },
        { json: "libelleMission", js: "libelleMission", typ: "" },
        { json: "missionRef", js: "missionRef", typ: "" },
    ], false),
    "Representations": o([
        { json: "representation", js: "representation", typ: r("Representation") },
    ], false),
    "Representation": o([
        { json: "nom", js: "nom", typ: r("Nom") },
        { json: "typeMime", js: "typeMime", typ: r("TypeMIME") },
        { json: "statutRepresentation", js: "statutRepresentation", typ: r("StatutRepresentation") },
        { json: "repSource", js: "repSource", typ: r("AmendementParentRefClass") },
        { json: "offset", js: "offset", typ: r("AmendementParentRefClass") },
        { json: "contenu", js: "contenu", typ: r("Contenu") },
        { json: "dateDispoRepresentation", js: "dateDispoRepresentation", typ: r("AmendementParentRefClass") },
    ], false),
    "Contenu": o([
        { json: "documentURI", js: "documentURI", typ: "" },
    ], false),
    "StatutRepresentation": o([
        { json: "verbatim", js: "verbatim", typ: "" },
        { json: "canonique", js: "canonique", typ: "" },
        { json: "officielle", js: "officielle", typ: "" },
        { json: "transcription", js: "transcription", typ: "" },
        { json: "enregistrement", js: "enregistrement", typ: "" },
    ], false),
    "TypeMIME": o([
        { json: "type", js: "type", typ: r("TypeMIMEType") },
        { json: "subType", js: "subType", typ: r("Nom") },
    ], false),
    "Signataires": o([
        { json: "auteur", js: "auteur", typ: r("Auteur") },
        { json: "cosignataires", js: "cosignataires", typ: r("Cosignataires") },
        { json: "suffixe", js: "suffixe", typ: u(r("AmendementParentRefClass"), "") },
        { json: "libelle", js: "libelle", typ: u(null, "") },
    ], false),
    "Auteur": o([
        { json: "typeAuteur", js: "typeAuteur", typ: r("TypeAuteur") },
        { json: "gouvernementRef", js: "gouvernementRef", typ: u(r("AmendementParentRefClass"), "") },
        { json: "acteurRef", js: "acteurRef", typ: u(r("AmendementParentRefClass"), "") },
        { json: "groupePolitiqueRef", js: "groupePolitiqueRef", typ: u(r("AmendementParentRefClass"), "") },
        { json: "auteurRapporteurOrganeRef", js: "auteurRapporteurOrganeRef", typ: u(r("AmendementParentRefClass"), "") },
    ], false),
    "Cosignataires": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: u(undefined, "") },
        { json: "@xsi:nil", js: "@xsi:nil", typ: u(undefined, "") },
        { json: "acteurRef", js: "acteurRef", typ: u(undefined, u(a(""), "")) },
    ], false),
    "AccordGouvernementDepotHorsDelai": [
        "Accord",
        "Sans objet",
    ],
    "Action": [
        "création",
        "modification",
        "suppression",
    ],
    "Code": [
        "AC",
        "AT",
        "DI",
        "effacé",
        "ER",
        "ET",
        "IR",
        "IRR127",
        "IRR20",
        "IRR37",
        "IRR38",
        "IRR41",
        "IRR42",
        "IRR45",
        "IRR48",
        "IRR53",
        "IRRAI",
        "IRRAIF",
        "IRRD",
        "IRRHC",
        "IRRHD",
        "IRRSA",
        "R",
        "RT",
        "T",
    ],
    "Libelle": [
        "A discuter",
        "A déposer",
        "Discuté",
        "effacé",
        "En recevabilité",
        "En traitement",
        "Irrecevable",
        "Irrecevable 40",
        "Retiré",
    ],
    "SortEnum": [
        "Adopté",
        "Autre irrecevabilité",
        "Autre irrecevabilité 40",
        "Autres irr LOLF",
        "Autres irr LOLFSS",
        "Cavalier (45)",
        "Cavalier budgétaire",
        "Cavalier social",
        "Champ de l'habilitation (38)",
        "Charge",
        "CMP (45)",
        "Crédits",
        "Disposition réglementaire (37)",
        "Domaine de la loi (41)",
        "Domaine loi organique (127)",
        "Doublon",
        "Entonnoir (45)",
        "Gage",
        "Hors champ",
        "Hors-délais",
        "Injonction (20)",
        "Irr en première partie",
        "Irr en seconde partie",
        "LOLF",
        "LOLFSS",
        "Non soutenu",
        "Ordre du jour (48)",
        "R",
        "Ratification traité (53)",
        "Rejeté",
        "Retiré",
        "Retiré après publication",
        "Retiré avant publication",
        "Satisfait ou inopérant (42)",
        "Sous-amendement (98)",
        "Tombé",
    ],
    "TypePosition": [
        "Dernier",
        "Milieu",
        "Premier",
    ],
    "AvantAApres": [
        "A",
        "Après",
        "Avant",
    ],
    "DivisionType": [
        "ANNEXE",
        "ARTICLE",
        "CHAPITRE",
        "TITRE",
    ],
    "CodeEtat": [
        "B",
        "C",
        "D",
    ],
    "Nom": [
        "PDF",
    ],
    "TypeMIMEType": [
        "application",
    ],
    "TypeAuteur": [
        "Député",
        "Gouvernement",
        "Rapporteur",
    ],
};
