// To parse this data:
//
//   import { Convert, Document, DossierParlementaire, DossiersLegislatifs } from "./file";
//
//   const document = Convert.toDocument(json);
//   const dossierParlementaire = Convert.toDossierParlementaire(json);
//   const dossiersLegislatifs = Convert.toDossiersLegislatifs(json);
//
// These functions will throw an error if the JSON doesn't
// match the expected interface, even if the JSON is valid.

import { CodeActe } from "../shared_types/codes_actes"

export interface Document {
    document: DocumentDocument | DocumentSupprime;
}

export interface DocumentDocument {
    "@xmlns":                 string;
    "@xmlns:xsi":             string;
    "@xsi:type":              DocumentXsiType;
    uid:                      string;
    legislature:              null | string;
    cycleDeVie:               CycleDeVie;
    denominationStructurelle: DocumentDenominationStructurelle;
    provenance?:              Provenance;
    titres:                   Titres;
    divisions:                Divisions | null;
    dossierRef:               string;
    redacteur:                null;
    classification:           Classification;
    auteurs:                  Auteurs;
    correction:               Correction | null;
    notice:                   Notice;
    indexation:               Indexation | null;
    imprimerie:               Imprimerie | null;
    coSignataires?:           CoSignataires | null;
    depotAmendements?:        DepotAmendements | null;
    organesReferents?:        OrganesReferents | null;
    rapportPublie?:           string;
}

export interface DocumentSupprime {
    "@xmlns":     string;
    "@xmlns:xsi": string;
    "@xsi:type":  DocumentXsiType;
    uid:          string;
    metadatas:    DocumentSupprimeMetadata;
}

export interface DocumentSupprimeMetadata {
    dateSuppression: Date;
}

export enum DocumentXsiType {
    AccordInternationalType = "accordInternational_Type",
    AvisConseilEtatType = "avisConseilEtat_Type",
    DocumentEtudeImpactType = "documentEtudeImpact_Type",
    RapportParlementaireType = "rapportParlementaire_Type",
    TexteLoiType = "texteLoi_Type",
}

export interface Auteurs {
    auteur: AuteurElement[] | AuteurElement;
}

export interface AuteurElement {
    acteur?: AuteurActeur;
    organe?: AuteurOrgane;
}

export interface AuteurActeur {
    acteurRef: string;
    qualite:   Qualite;
}

export enum Qualite {
    Auteur = "auteur",
    Cosignataire = "cosignataire",
    Rapporteur = "rapporteur",
    RapporteurGénéral = "rapporteur général",
    RapporteurPourAvis = "rapporteur pour avis",
    RapporteurSpécial = "rapporteur spécial",
}

export interface AuteurOrgane {
    organeRef: string;
}

export interface Classification {
    famille:        Famille | null;
    type:           ProcedureParlementaire;
    sousType:       SousType | null;
    statutAdoption: StatutAdoption | null;
}

export interface Famille {
    depot:   ProcedureParlementaire;
    classe:  ProcedureParlementaire;
    espece?: ProcedureParlementaire;
}

export interface ProcedureParlementaire {
    code:    string;
    libelle: string;
}

export interface SousType {
    code:            Code;
    libelle?:        string;
    libelleEdition?: string;
}

export enum Code {
    Accpresrp = "ACCPRESRP",
    Appart1515 = "APPART1515",
    Appart341 = "APPART341",
    Apploi = "APPLOI",
    Aue = "AUE",
    Autratconv = "AUTRATCONV",
    Avecvote = "AVECVOTE",
    Censure492 = "CENSURE492",
    Censure493 = "CENSURE493",
    Comenq = "COMENQ",
    Compa = "COMPA",
    Comspcpte = "COMSPCPTE",
    Const = "CONST",
    Ctrlbudg = "CTRLBUDG",
    Divers = "DIVERS",
    Engresptxt = "ENGRESPTXT",
    Enqu = "ENQU",
    Fin = "FIN",
    Fingest = "FINGEST",
    Finrect = "FINRECT",
    Finssoc = "FINSSOC",
    Finssocrec = "FINSSOCREC",
    Impactlois = "IMPACTLOIS",
    Legdelegan = "LEGDELEGAN",
    Minfocomper = "MINFOCOMPER",
    Modregltan = "MODREGLTAN",
    Noteopecst = "NOTEOPECST",
    Offparl = "OFFPARL",
    Org = "ORG",
    Petition = "PETITION",
    Presidage = "PRESIDAGE",
    Presidass = "PRESIDASS",
    Presidrep = "PRESIDREP",
    Propactcom = "PROPACTCOM",
    Prpdit = "PRPDIT",
    Rect = "RECT",
    Refart11 = "REFART11",
    Rgltbudg = "RGLTBUDG",
    Rgltbudg2 = "RGLTBUDG2",
    Supp = "SUPP",
    Suspours = "SUSPOURS",
    Tvxinstiteurop = "TVXINSTITEUROP",
}

export enum StatutAdoption {
    Adoptcom = "ADOPTCOM",
    Adoptseance = "ADOPTSEANCE",
    Rejetseance = "REJETSEANCE",
}

export interface CoSignataires {
    coSignataire: CoSignataireElement[] | CoSignataireElement;
}

export interface CoSignataireElement {
    acteur?:                CoSignataireActeur;
    dateCosignature:        string;
    dateRetraitCosignature: null | string;
    edite:                  string;
    organe?:                CoSignataireOrgane;
}

export interface CoSignataireActeur {
    acteurRef: string;
}

export interface InitiateursRefs {
    acteurRef?: string;
    organeRef?: string;
}

export interface CoSignataireOrgane {
    organeRef:    string;
    etApparentes: string;
}

export interface Correction {
    typeCorrection:    TypeCorrection;
    niveauCorrection?: string;
}

export enum TypeCorrection {
    Rectifié = "Rectifié",
}

export interface CycleDeVie {
    chrono: Chrono;
}

export interface Chrono {
    dateCreation:       Date;
    dateDepot:          Date | null;
    datePublication:    Date | null;
    datePublicationWeb: Date | null;
}

export enum DocumentDenominationStructurelle {
    Allocution = "Allocution",
    Avis = "Avis",
    Déclaration = "Déclaration",
    Lettre = "Lettre",
    Message = "Message",
    Motion = "Motion",
    ProjetDeLoi = "Projet de loi",
    PropositionDeLoi = "Proposition de loi",
    PropositionDeRésolution = "Proposition de résolution",
    Rapport = "Rapport",
    RapportDInformation = "Rapport d'information",
    Résolution = "Résolution",
}

export interface DepotAmendements {
    amendementsSeance:      AmendementsSeance;
    amendementsCommission?: AmendementsCommission;
}

export interface AmendementsCommission {
    commission: CommissionElement[] | CommissionElement;
}

export interface CommissionElement {
    organeRef:       string;
    amendable:       string;
    dateLimiteDepot: null;
}

export interface AmendementsSeance {
    amendable:       string;
    dateLimiteDepot: null;
}

export interface Divisions {
    division: Division[] | Division;
}

export interface Division {
    "@xsi:type":              DocumentXsiType;
    uid:                      string;
    legislature:              null | string;
    cycleDeVie:               CycleDeVie;
    denominationStructurelle: string;
    titres:                   Titres;
    divisions:                Divisions | null;
    dossierRef:               string;
    redacteur:                null;
    classification:           Classification;
    auteurs:                  Auteurs;
    organesReferents?:        OrganesReferents | null;
    correction:               Correction | null;
    notice:                   Notice;
    indexation:               Indexation | null;
    imprimerie:               Imprimerie | null;
    coSignataires?:           null;
    depotAmendements?:        null;
    rapportPublie?:           string;
}

export interface Imprimerie {
    DIAN?:   null | string;
    ISBN?:   null | string;
    ISSN?:   null;
    nbPage?: null | string;
    prix:    null | string;
}

export interface Notice {
    numNotice?:       string;
    formule?:         string;
    adoptionConforme: string;
}

export interface Titres {
    titrePrincipal:      string;
    titrePrincipalCourt: string;
}

export interface OrganesReferents {
    organeRef: string[] | string;
}

export enum Provenance {
    Commission = "Commission",
    Séance = "Séance",
    TexteDéposé = "Texte Déposé",
}

export interface DossierParlementaire {
    dossierParlementaire: DossierParlementaireDossierParlementaire;
}

export interface DossierParlementaireDossierParlementaire {
    "@xmlns":               string;
    "@xmlns:xsi":           string;
    "@xsi:type":            DossierParlementaireXsiType;
    uid:                    string;
    legislature:            string;
    titreDossier:           TitreDossier;
    procedureParlementaire: ProcedureParlementaire;
    initiateur:             Initiateur | null;
    actesLegislatifs:       ActesLegislatifs | null;
    indexation?:            Indexation;
    fusionDossier:          FusionDossier | null;
    PLF?:                   Plf;
}

export enum DossierParlementaireXsiType {
    DossierCommissionEnqueteType = "DossierCommissionEnquete_Type",
    DossierIniativeExecutifType = "DossierIniativeExecutif_Type",
    DossierLegislatifType = "DossierLegislatif_Type",
    DossierMissionControleType = "DossierMissionControle_Type",
    DossierMissionInformationType = "DossierMissionInformation_Type",
    DossierResolutionAn = "DossierResolutionAN",
}

export interface Plf {
    EtudePLF: EtudePlf[];
}

export interface EtudePlf {
    uid:             string;
    organeRef:       string;
    texteAssocie?:   string;
    rapporteur?:     Rapporteur[] | Rapporteur;
    missionMinefi?:  MissionMinefiElement;
    ordreDIQS:       string;
    ordreCommission: string;
}

export interface Missions {
    mission: MissionMinefiElement[] | MissionMinefiElement;
}

export interface MissionMinefiElement {
    typeMission:  TypeMission;
    codeMission?: string; // 2 uppercase characters
    libelleLong:  string;
    libelleCourt: string;
    typeBudget:   TypeBudget;
    missions?:    Missions;
}

export enum TypeBudget {
    BudgetAnnexe = "Budget annexe",
    BudgetGénéral = "Budget général",
    CompteDeConcoursFinancier = "Compte de concours financier",
    CompteSpécial = "Compte spécial",
    PremièrePartie = "Première partie",
}

export enum TypeMission {
    MissionPrincipale = "mission principale",
    MissionSecondaire = "mission secondaire",
    PartieDeMission = "partie de mission",
}

export interface ActesLegislatifs {
    acteLegislatif: ActeLegislatif[] | ActeLegislatif;
}

export interface ActeLegislatif {
    "@xsi:type":                       ActeLegislatifXsiType;
    uid:                               string;
    codeActe:                          CodeActe;
    libelleActe:                       LibelleActe;
    organeRef?:                        string;
    dateActe:                          Date | null;
    actesLegislatifs:                  ActesLegislatifs | null;
    texteAssocie?:                     string;
    contributionInternaute?:           ContributionInternaute;
    provenance?:                       string;
    depotInitialLectureDefinitiveRef?: null | string;
    initiateur?:                       Initiateur;
    statutConclusion?:                 TypeDeclaration;
    reunionRef?:                       string | null;
    odjRef?:                           string | null;
    odSeancejRef?:                     null;
    voteRefs?:                         VoteRefs | null;
    textesAssocies?:                   TextesAssocies;
    rapporteurs?:                      Rapporteurs;
    reunion?:                          null;
    texteAdopte?:                      null | string;
    casSaisine?:                       TypeDeclaration;
    decision?:                         TypeDeclaration;
    formuleDecision?:                  string;
    initiateurs?:                      InitiateursRefs | null;
    motif?:                            Motif;
    urlConclusion?:                    string;
    numDecision?:                      string;
    anneeDecision?:                    string;
    typeDeclaration?:                  TypeDeclaration;
    typeMotionCensure?:                TypeDeclaration;
    auteurs?:                          ActeLegislatifAuteurs | null;
    typeMotion?:                       TypeDeclaration;
    auteurMotion?:                     string;
    dateRetrait?:                      null;
    texteLoiRef?:                      string;
    infoJO?:                           InfoJoElement | null;
    urlEcheancierLoi?:                 null | string;
    codeLoi?:                          string;
    titreLoi?:                         string | null;
    infoJORect?:                       InfoJoElement[] | InfoJoElement;
    texteEuropeen?:                    TexteEuropeen;
    infoJOCE?:                         InfoJoce;
    statutAdoption?:                   TypeDeclaration;
    urlLegifrance?:                    string;
    referenceNOR?:                     string;
    texteExterneAssocie?:              TexteExterneAssocie;
}

export enum ActeLegislatifXsiType {
    AdoptionEuropeType = "Adoption_Europe_Type",
    ConclusionEtapeCcType = "ConclusionEtapeCC_Type",
    ConvocationCongresType = "ConvocationCongres_Type",
    CreationOrganeTemporaireType = "CreationOrganeTemporaire_Type",
    DecisionMotionCensureType = "DecisionMotionCensure_Type",
    DecisionRecevabiliteBureauType = "DecisionRecevabiliteBureau_Type",
    DecisionType = "Decision_Type",
    DeclarationGouvernementType = "DeclarationGouvernement_Type",
    DepotAccordInternationalType = "DepotAccordInternational_Type",
    DepotAvisConseilEtatType = "DepotAvisConseilEtat_Type",
    DepotInitiativeNavetteType = "DepotInitiativeNavette_Type",
    DepotInitiativeType = "DepotInitiative_Type",
    DepotLettreRectificativeType = "DepotLettreRectificative_Type",
    DepotMotionCensureType = "DepotMotionCensure_Type",
    DepotMotionReferendaireType = "DepotMotionReferendaire_Type",
    DepotRapportType = "DepotRapport_Type",
    DiscussionCommissionType = "DiscussionCommission_Type",
    DiscussionSeancePubliqueType = "DiscussionSeancePublique_Type",
    EtapeType = "Etape_Type",
    EtudeImpactType = "EtudeImpact_Type",
    MotionProcedureType = "MotionProcedure_Type",
    NominRapporteursType = "NominRapporteurs_Type",
    ProcedureAccelereType = "ProcedureAccelere_Type",
    PromulgationType = "Promulgation_Type",
    RenvoiPrealableType = "RenvoiPrealable_Type",
    RenvoiCmpType = "RenvoiCMP_Type",
    RetraitInitiativeType = "RetraitInitiative_Type",
    SaisieComAvisType = "SaisieComAvis_Type",
    SaisieComFondType = "SaisieComFond_Type",
    SaisineConseilConstitType = "SaisineConseilConstit_Type",
}

export interface LibelleActe {
    nomCanonique:  string;
    libelleCourt?: string;
}

export interface Rapporteur {
    acteurRef:      string;
    typeRapporteur: Qualite;
    etudePLFRef?:   string;
}

export interface Rapporteurs {
    rapporteur: Rapporteur[] | Rapporteur;
}

export interface TypeDeclaration {
    fam_code: string;
    libelle?: string;
}

export interface Rapporteurs {
    rapporteur: Rapporteur[] | Rapporteur;
}

export interface TextesAssocies {
    texteAssocie: TexteAssocieElement[] | TexteAssocieElement;
}

export interface TexteAssocieElement {
    typeTexte:       TypeTexte;
    refTexteAssocie: string;
}

export enum TypeTexte {
    Bta = "BTA",
    Tap = "TAP",
}

export interface VoteRefs {
    voteRef: string[] | string;
}

export interface ActeLegislatifAuteurs {
    acteurRef: string[] | string;
}

export interface ContributionInternaute {
    dateFermeture:  null | string;
    dateOuverture?: string;
}

export interface ActeurElement {
    acteurRef: string;
    mandatRef: string;
}

export enum Motif {
    EnApplicationDeLArticle612DeLaConstitution = "En application de l'article 61§2 de la Constitution",
}

export interface InfoJoElement {
    typeJO:        TypeJo;
    dateJO:        string;
    pageJO:        null;
    numJO:         string;
    urlLegifrance?: string;
    referenceNOR?:  string;
}

export enum TypeJo {
    JoLoiDecret = "JO_LOI_DECRET",
}

export interface FusionDossier {
    cause:               Cause;
    dossierAbsorbantRef: string;
}

export enum Cause {
    DossierAbsorbé = "Dossier absorbé",
    ExamenCommun = "Examen commun",
}

export interface Initiateur {
    acteurs?: InitiateurActeurs;
    organes?: Organes;
}

export interface InitiateurActeurs {
    acteur: ActeurElement[] | ActeurElement;
}

export interface Organes {
    organe: OrganesOrgane;
}

export interface OrganesOrgane {
    organeRef: OrganeRefClass;
}

export interface OrganeRefClass {
    uid: string;
}

export interface TitreDossier {
    titre:       string;
    titreChemin: null | string;
    senatChemin: null | string;
}

export interface DossiersLegislatifs {
    export: Export;
}

export interface Export {
    "@xmlns:xsi":        string;
    textesLegislatifs:   TextesLegislatifs;
    dossiersLegislatifs: DossiersLegislatifsClass;
}

export interface DossiersLegislatifsClass {
    dossier: Dossier[];
}

export interface Dossier {
    dossierParlementaire: DossierDossierParlementaire;
}

export interface DossierDossierParlementaire {
    "@xsi:type"?:           DossierParlementaireXsiType;
    uid:                    string;
    legislature:            string;
    titreDossier:           TitreDossier;
    procedureParlementaire: ProcedureParlementaire;
    initiateur:             Initiateur | null;
    actesLegislatifs:       ActesLegislatifs;
    indexation?:            Indexation;
    fusionDossier:          FusionDossier | null;
    PLF?:                   Plf;
}

export interface InfoJoce {
    refJOCE:  string;
    dateJOCE: string;
}

export interface TexteEuropeen {
    typeTexteEuropeen:  string;
    titreTexteEuropeen: string;
}

export interface Indexation {
    themes: Themes;
}

export interface Themes {
    "@niveau": string;
    theme:     Theme;
}

export interface Theme {
    libelleTheme: string;
}

export interface TextesLegislatifs {
    document: DocumentElement[];
}

export interface DocumentElement {
    "@xsi:type":              DocumentXsiType;
    uid:                      string;
    legislature:              null | string;
    cycleDeVie:               CycleDeVie;
    denominationStructurelle: DocumentDenominationStructurelle;
    provenance?:              Provenance;
    titres:                   Titres;
    divisions:                Divisions | null;
    dossierRef:               string;
    redacteur:                null;
    classification:           Classification;
    auteurs:                  Auteurs;
    correction:               Correction | null;
    notice:                   Notice;
    indexation:               Indexation | null;
    imprimerie:               Imprimerie | null;
    coSignataires?:           CoSignataires | null;
    depotAmendements?:        DepotAmendements | null;
    rapportPublie?:           string;
}

export interface TexteExterneAssocie {
    typeDoc:  TexteExterneAssocieTypeDoc;
    texteRef: string;
    titreDoc: string;
    urlDoc:   string;
}

export enum TexteExterneAssocieTypeDoc {
    Decret = "Décret",
}

// Converts JSON strings to/from your types
// and asserts the results of JSON.parse at runtime
export class Convert {
    public static toDocument(json: string): Document {
        return cast(JSON.parse(json), r("Document"));
    }

    public static documentToJson(value: Document): string {
        return JSON.stringify(uncast(value, r("Document")), null, 2);
    }

    public static toDossierParlementaire(json: string): DossierParlementaire {
        return cast(JSON.parse(json), r("DossierParlementaire"));
    }

    public static dossierParlementaireToJson(value: DossierParlementaire): string {
        return JSON.stringify(uncast(value, r("DossierParlementaire")), null, 2);
    }

    public static toDossiersLegislatifs(json: string): DossiersLegislatifs {
        return cast(JSON.parse(json), r("DossiersLegislatifs"));
    }

    public static dossiersLegislatifsToJson(value: DossiersLegislatifs): string {
        return JSON.stringify(uncast(value, r("DossiersLegislatifs")), null, 2);
    }
}

function invalidValue(typ: any, val: any, key: any = ''): never {
    if (key) {
        throw Error(`Invalid value for key "${key}". Expected type ${JSON.stringify(typ)} but got ${JSON.stringify(val)}`);
    }
    throw Error(`Invalid value ${JSON.stringify(val)} for type ${JSON.stringify(typ)}`, );
}

function jsonToJSProps(typ: any): any {
    if (typ.jsonToJS === undefined) {
        const map: any = {};
        typ.props.forEach((p: any) => map[p.json] = { key: p.js, typ: p.typ });
        typ.jsonToJS = map;
    }
    return typ.jsonToJS;
}

function jsToJSONProps(typ: any): any {
    if (typ.jsToJSON === undefined) {
        const map: any = {};
        typ.props.forEach((p: any) => map[p.js] = { key: p.json, typ: p.typ });
        typ.jsToJSON = map;
    }
    return typ.jsToJSON;
}

function transform(val: any, typ: any, getProps: any, key: any = ''): any {
    function transformPrimitive(typ: string, val: any): any {
        if (typeof typ === typeof val) return val;
        return invalidValue(typ, val, key);
    }

    function transformUnion(typs: any[], val: any): any {
        // val must validate against one typ in typs
        const l = typs.length;
        for (let i = 0; i < l; i++) {
            const typ = typs[i];
            try {
                return transform(val, typ, getProps);
            } catch (_) {}
        }
        return invalidValue(typs, val);
    }

    function transformEnum(cases: string[], val: any): any {
        if (cases.indexOf(val) !== -1) return val;
        return invalidValue(cases, val);
    }

    function transformArray(typ: any, val: any): any {
        // val must be an array with no invalid elements
        if (!Array.isArray(val)) return invalidValue("array", val);
        return val.map(el => transform(el, typ, getProps));
    }

    function transformDate(val: any): any {
        if (val === null) {
            return null;
        }
        const d = new Date(val);
        if (isNaN(d.valueOf())) {
            return invalidValue("Date", val);
        }
        return d;
    }

    function transformObject(props: { [k: string]: any }, additional: any, val: any): any {
        if (val === null || typeof val !== "object" || Array.isArray(val)) {
            return invalidValue("object", val);
        }
        const result: any = {};
        Object.getOwnPropertyNames(props).forEach(key => {
            const prop = props[key];
            const v = Object.prototype.hasOwnProperty.call(val, key) ? val[key] : undefined;
            result[prop.key] = transform(v, prop.typ, getProps, prop.key);
        });
        Object.getOwnPropertyNames(val).forEach(key => {
            if (!Object.prototype.hasOwnProperty.call(props, key)) {
                result[key] = transform(val[key], additional, getProps, key);
            }
        });
        return result;
    }

    if (typ === "any") return val;
    if (typ === null) {
        if (val === null) return val;
        return invalidValue(typ, val);
    }
    if (typ === false) return invalidValue(typ, val);
    while (typeof typ === "object" && typ.ref !== undefined) {
        typ = typeMap[typ.ref];
    }
    if (Array.isArray(typ)) return transformEnum(typ, val);
    if (typeof typ === "object") {
        return typ.hasOwnProperty("unionMembers") ? transformUnion(typ.unionMembers, val)
            : typ.hasOwnProperty("arrayItems")    ? transformArray(typ.arrayItems, val)
            : typ.hasOwnProperty("props")         ? transformObject(getProps(typ), typ.additional, val)
            : invalidValue(typ, val);
    }
    // Numbers can be parsed by Date but shouldn't be.
    if (typ === Date && typeof val !== "number") return transformDate(val);
    return transformPrimitive(typ, val);
}

function cast<T>(val: any, typ: any): T {
    return transform(val, typ, jsonToJSProps);
}

function uncast<T>(val: T, typ: any): any {
    return transform(val, typ, jsToJSONProps);
}

function a(typ: any) {
    return { arrayItems: typ };
}

function u(...typs: any[]) {
    return { unionMembers: typs };
}

function o(props: any[], additional: any) {
    return { props, additional };
}

// function m(additional: any) {
//     return { props: [], additional };
// }

function r(name: string) {
    return { ref: name };
}

const typeMap: any = {
    "Document": o([
        { json: "document", js: "document", typ: u(r("DocumentDocument"), r("DocumentSupprime")) },
    ], false),
    "DocumentDocument": o([
        { json: "@xmlns", js: "@xmlns", typ: "" },
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: "" },
        { json: "@xsi:type", js: "@xsi:type", typ: r("DocumentXsiType") },
        { json: "uid", js: "uid", typ: "" },
        { json: "legislature", js: "legislature", typ: u(null, "") },
        { json: "cycleDeVie", js: "cycleDeVie", typ: r("CycleDeVie") },
        { json: "denominationStructurelle", js: "denominationStructurelle", typ: r("DocumentDenominationStructurelle") },
        { json: "provenance", js: "provenance", typ: u(undefined, r("Provenance")) },
        { json: "titres", js: "titres", typ: r("Titres") },
        { json: "divisions", js: "divisions", typ: u(r("Divisions"), null) },
        { json: "dossierRef", js: "dossierRef", typ: "" },
        { json: "redacteur", js: "redacteur", typ: null },
        { json: "classification", js: "classification", typ: r("Classification") },
        { json: "auteurs", js: "auteurs", typ: r("Auteurs") },
        { json: "correction", js: "correction", typ: u(r("Correction"), null) },
        { json: "notice", js: "notice", typ: r("Notice") },
        { json: "indexation", js: "indexation", typ: u(r("Indexation"), null) },
        { json: "imprimerie", js: "imprimerie", typ: u(r("Imprimerie"), null) },
        { json: "coSignataires", js: "coSignataires", typ: u(undefined, u(r("CoSignataires"), null)) },
        { json: "depotAmendements", js: "depotAmendements", typ: u(undefined, u(r("DepotAmendements"), null)) },
        { json: "organesReferents", js: "organesReferents", typ: u(undefined, u(r("OrganesReferents"), null)) },
        { json: "rapportPublie", js: "rapportPublie", typ: u(undefined, "") },
    ], false),
    "DocumentSupprime": o([
        { json: "@xmlns", js: "@xmlns", typ: "" },
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: "" },
        { json: "@xsi:type", js: "@xsi:type", typ: r("DocumentXsiType") },
        { json: "uid", js: "uid", typ: "" },
        { json: "metadatas", js: "metadatas", typ: r("DocumentSupprimeMetadata") },
    ], false),
    "DocumentSupprimeMetadata": o([
        { json: "dateSuppression", js: "dateSuppression", typ: Date },
    ], false),
    "Auteurs": o([
        { json: "auteur", js: "auteur", typ: u(a(r("AuteurElement")), r("AuteurElement")) },
    ], false),
    "AuteurElement": o([
        { json: "acteur", js: "acteur", typ: u(undefined, r("AuteurActeur")) },
        { json: "organe", js: "organe", typ: u(undefined, r("AuteurOrgane")) },
    ], false),
    "AuteurActeur": o([
        { json: "acteurRef", js: "acteurRef", typ: "" },
        { json: "qualite", js: "qualite", typ: r("Qualite") },
    ], false),
    "AuteurOrgane": o([
        { json: "organeRef", js: "organeRef", typ: "" },
    ], false),
    "Classification": o([
        { json: "famille", js: "famille", typ: u(r("Famille"), null) },
        { json: "type", js: "type", typ: r("ProcedureParlementaire") },
        { json: "sousType", js: "sousType", typ: u(r("SousType"), null) },
        { json: "statutAdoption", js: "statutAdoption", typ: u(r("StatutAdoption"), null) },
    ], false),
    "Famille": o([
        { json: "depot", js: "depot", typ: r("ProcedureParlementaire") },
        { json: "classe", js: "classe", typ: r("ProcedureParlementaire") },
        { json: "espece", js: "espece", typ: u(undefined, r("ProcedureParlementaire")) },
    ], false),
    "ProcedureParlementaire": o([
        { json: "code", js: "code", typ: "" },
        { json: "libelle", js: "libelle", typ: "" },
    ], false),
    "SousType": o([
        { json: "code", js: "code", typ: r("Code") },
        { json: "libelle", js: "libelle", typ: u(undefined, "") },
        { json: "libelleEdition", js: "libelleEdition", typ: u(undefined, "") },
    ], false),
    "CoSignataires": o([
        { json: "coSignataire", js: "coSignataire", typ: u(a(r("CoSignataireElement")), r("CoSignataireElement")) },
    ], false),
    "CoSignataireElement": o([
        { json: "acteur", js: "acteur", typ: u(undefined, r("CoSignataireActeur")) },
        { json: "dateCosignature", js: "dateCosignature", typ: "" },
        { json: "dateRetraitCosignature", js: "dateRetraitCosignature", typ: u(null, "") },
        { json: "edite", js: "edite", typ: "" },
        { json: "organe", js: "organe", typ: u(undefined, r("CoSignataireOrgane")) },
    ], false),
    "CoSignataireActeur": o([
        { json: "acteurRef", js: "acteurRef", typ: "" },
    ], false),
    "InitiateursRefs": o([
        { json: "acteurRef", js: "acteurRef", typ: u(undefined, "") },
        { json: "organeRef", js: "organeRef", typ: u(undefined, "") },
    ], false),
    "CoSignataireOrgane": o([
        { json: "organeRef", js: "organeRef", typ: "" },
        { json: "etApparentes", js: "etApparentes", typ: "" },
    ], false),
    "Correction": o([
        { json: "typeCorrection", js: "typeCorrection", typ: r("TypeCorrection") },
        { json: "niveauCorrection", js: "niveauCorrection", typ: u(undefined, "") },
    ], false),
    "CycleDeVie": o([
        { json: "chrono", js: "chrono", typ: r("Chrono") },
    ], false),
    "Chrono": o([
        { json: "dateCreation", js: "dateCreation", typ: Date },
        { json: "dateDepot", js: "dateDepot", typ: u(Date, null) },
        { json: "datePublication", js: "datePublication", typ: u(Date, null) },
        { json: "datePublicationWeb", js: "datePublicationWeb", typ: u(Date, null) },
    ], false),
    "DepotAmendements": o([
        { json: "amendementsSeance", js: "amendementsSeance", typ: r("AmendementsSeance") },
        { json: "amendementsCommission", js: "amendementsCommission", typ: u(undefined, r("AmendementsCommission")) },
    ], false),
    "AmendementsCommission": o([
        { json: "commission", js: "commission", typ: u(a(r("CommissionElement")), r("CommissionElement")) },
    ], false),
    "CommissionElement": o([
        { json: "organeRef", js: "organeRef", typ: "" },
        { json: "amendable", js: "amendable", typ: "" },
        { json: "dateLimiteDepot", js: "dateLimiteDepot", typ: null },
    ], false),
    "AmendementsSeance": o([
        { json: "amendable", js: "amendable", typ: "" },
        { json: "dateLimiteDepot", js: "dateLimiteDepot", typ: null },
    ], false),
    "Divisions": o([
        { json: "division", js: "division", typ: u(a(r("Division")), r("Division")) },
    ], false),
    "Division": o([
        { json: "@xsi:type", js: "@xsi:type", typ: r("DocumentXsiType") },
        { json: "uid", js: "uid", typ: "" },
        { json: "legislature", js: "legislature", typ: u(null, "") },
        { json: "cycleDeVie", js: "cycleDeVie", typ: r("CycleDeVie") },
        { json: "denominationStructurelle", js: "denominationStructurelle", typ: "" },
        { json: "titres", js: "titres", typ: r("Titres") },
        { json: "divisions", js: "divisions", typ: u(r("Divisions"), null) },
        { json: "dossierRef", js: "dossierRef", typ: "" },
        { json: "redacteur", js: "redacteur", typ: null },
        { json: "classification", js: "classification", typ: r("Classification") },
        { json: "auteurs", js: "auteurs", typ: r("Auteurs") },
        { json: "organesReferents", js: "organesReferents", typ: u(undefined, u(r("OrganesReferents"), null)) },
        { json: "correction", js: "correction", typ: u(r("Correction"), null) },
        { json: "notice", js: "notice", typ: r("Notice") },
        { json: "indexation", js: "indexation", typ: u(r("Indexation"), null) },
        { json: "imprimerie", js: "imprimerie", typ: u(r("Imprimerie"), null) },
        { json: "coSignataires", js: "coSignataires", typ: u(undefined, null) },
        { json: "depotAmendements", js: "depotAmendements", typ: u(undefined, null) },
        { json: "rapportPublie", js: "rapportPublie", typ: u(undefined, "") },
    ], false),
    "Imprimerie": o([
        { json: "DIAN", js: "DIAN", typ: u(undefined, u(null, "")) },
        { json: "ISBN", js: "ISBN", typ: u(undefined, u(null, "")) },
        { json: "ISSN", js: "ISSN", typ: u(undefined, null) },
        { json: "nbPage", js: "nbPage", typ: u(undefined, u(null, "")) },
        { json: "prix", js: "prix", typ: u(null, "") },
    ], false),
    "Notice": o([
        { json: "numNotice", js: "numNotice", typ: u(undefined, "") },
        { json: "formule", js: "formule", typ: u(undefined, "") },
        { json: "adoptionConforme", js: "adoptionConforme", typ: "" },
    ], false),
    "Titres": o([
        { json: "titrePrincipal", js: "titrePrincipal", typ: "" },
        { json: "titrePrincipalCourt", js: "titrePrincipalCourt", typ: "" },
    ], false),
    "OrganesReferents": o([
        { json: "organeRef", js: "organeRef", typ: u(a(""), "") },
    ], false),
    "DossierParlementaire": o([
        { json: "dossierParlementaire", js: "dossierParlementaire", typ: r("DossierParlementaireDossierParlementaire") },
    ], false),
    "DossierParlementaireDossierParlementaire": o([
        { json: "@xmlns", js: "@xmlns", typ: "" },
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: "" },
        { json: "@xsi:type", js: "@xsi:type", typ: r("DossierParlementaireXsiType") },
        { json: "uid", js: "uid", typ: "" },
        { json: "legislature", js: "legislature", typ: "" },
        { json: "titreDossier", js: "titreDossier", typ: r("TitreDossier") },
        { json: "procedureParlementaire", js: "procedureParlementaire", typ: r("ProcedureParlementaire") },
        { json: "initiateur", js: "initiateur", typ: u(r("Initiateur"), null) },
        { json: "actesLegislatifs", js: "actesLegislatifs", typ: u(r("ActesLegislatifs"), null) },
        { json: "indexation", js: "indexation", typ: u(undefined, r("Indexation")) },
        { json: "fusionDossier", js: "fusionDossier", typ: u(r("FusionDossier"), null) },
        { json: "PLF", js: "PLF", typ: u(undefined, r("Plf")) },
    ], false),
    "Plf": o([
        { json: "EtudePLF", js: "EtudePLF", typ: a(r("EtudePlf")) },
    ], false),
    "EtudePlf": o([
        { json: "uid", js: "uid", typ: "" },
        { json: "organeRef", js: "organeRef", typ: "" },
        { json: "texteAssocie", js: "texteAssocie", typ: u(undefined, "") },
        { json: "rapporteur", js: "rapporteur", typ: u(undefined, u(a(r("Rapporteur")), r("Rapporteur"))) },
        { json: "missionMinefi", js: "missionMinefi", typ: u(undefined, r("MissionMinefiElement")) },
        { json: "ordreDIQS", js: "ordreDIQS", typ: "" },
        { json: "ordreCommission", js: "ordreCommission", typ: "" },
    ], false),
    "Missions": o([
        { json: "mission", js: "mission", typ: u(a(r("MissionMinefiElement")), r("MissionMinefiElement")) },
    ], false),
    "MissionMinefiElement": o([
        { json: "typeMission", js: "typeMission", typ: r("TypeMission") },
        { json: "codeMission", js: "codeMission", typ: u(undefined, "") },
        { json: "libelleLong", js: "libelleLong", typ: "" },
        { json: "libelleCourt", js: "libelleCourt", typ: "" },
        { json: "typeBudget", js: "typeBudget", typ: r("TypeBudget") },
        { json: "missions", js: "missions", typ: u(undefined, r("Missions")) },
    ], false),
    "ActesLegislatifs": o([
        { json: "acteLegislatif", js: "acteLegislatif", typ: u(a(r("ActeLegislatif")), r("ActeLegislatif")) },
    ], false),
    "ActeLegislatif": o([
        { json: "@xsi:type", js: "@xsi:type", typ: r("ActeLegislatifXsiType") },
        { json: "uid", js: "uid", typ: "" },
        { json: "codeActe", js: "codeActe", typ: r("CodeActe") },
        { json: "libelleActe", js: "libelleActe", typ: r("LibelleActe") },
        { json: "organeRef", js: "organeRef", typ: u(undefined, "") },
        { json: "dateActe", js: "dateActe", typ: u(Date, null) },
        { json: "actesLegislatifs", js: "actesLegislatifs", typ: u(r("ActesLegislatifs"), null) },
        { json: "texteAssocie", js: "texteAssocie", typ: u(undefined, "") },
        { json: "contributionInternaute", js: "contributionInternaute", typ: u(undefined, r("ContributionInternaute")) },
        { json: "provenance", js: "provenance", typ: u(undefined, "") },
        { json: "depotInitialLectureDefinitiveRef", js: "depotInitialLectureDefinitiveRef", typ: u(undefined, u(null, "")) },
        { json: "initiateur", js: "initiateur", typ: u(undefined, r("Initiateur")) },
        { json: "statutConclusion", js: "statutConclusion", typ: u(undefined, r("TypeDeclaration")) },
        { json: "reunionRef", js: "reunionRef", typ: u(undefined, u("", null)) },
        { json: "odjRef", js: "odjRef", typ: u(undefined, u("", null)) },
        { json: "odSeancejRef", js: "odSeancejRef", typ: u(undefined, null) },
        { json: "voteRefs", js: "voteRefs", typ: u(undefined, u(r("VoteRefs"), null)) },
        { json: "textesAssocies", js: "textesAssocies", typ: u(undefined, r("TextesAssocies")) },
        { json: "reunion", js: "reunion", typ: u(undefined, null) },
        { json: "rapporteurs", js: "rapporteurs", typ: u(undefined, r("Rapporteurs")) },
        { json: "texteAdopte", js: "texteAdopte", typ: u(undefined, u(null, "")) },
        { json: "casSaisine", js: "casSaisine", typ: u(undefined, r("TypeDeclaration")) },
        { json: "decision", js: "decision", typ: u(undefined, r("TypeDeclaration")) },
        { json: "formuleDecision", js: "formuleDecision", typ: u(undefined, "") },
        { json: "initiateurs", js: "initiateurs", typ: u(undefined, u(r("InitiateursRefs"), null)) },
        { json: "motif", js: "motif", typ: u(undefined, r("Motif")) },
        { json: "urlConclusion", js: "urlConclusion", typ: u(undefined, "") },
        { json: "numDecision", js: "numDecision", typ: u(undefined, "") },
        { json: "anneeDecision", js: "anneeDecision", typ: u(undefined, "") },
        { json: "typeDeclaration", js: "typeDeclaration", typ: u(undefined, r("TypeDeclaration")) },
        { json: "typeMotionCensure", js: "typeMotionCensure", typ: u(undefined, r("TypeDeclaration")) },
        { json: "auteurs", js: "auteurs", typ: u(undefined, u(r("ActeLegislatifAuteurs"), null)) },
        { json: "typeMotion", js: "typeMotion", typ: u(undefined, r("TypeDeclaration")) },
        { json: "auteurMotion", js: "auteurMotion", typ: u(undefined, "") },
        { json: "dateRetrait", js: "dateRetrait", typ: u(undefined, null) },
        { json: "texteLoiRef", js: "texteLoiRef", typ: u(undefined, "") },
        { json: "infoJO", js: "infoJO", typ: u(undefined, u(r("InfoJoElement"), null)) },
        { json: "urlEcheancierLoi", js: "urlEcheancierLoi", typ: u(undefined, u(null, "")) },
        { json: "codeLoi", js: "codeLoi", typ: u(undefined, "") },
        { json: "titreLoi", js: "titreLoi", typ: u(undefined, u(null, "")) },
        { json: "infoJORect", js: "infoJORect", typ: u(undefined, u(a(r("InfoJoElement")), r("InfoJoElement"))) },
        { json: "texteEuropeen", js: "texteEuropeen", typ: u(undefined, r("TexteEuropeen")) },
        { json: "infoJOCE", js: "infoJOCE", typ: u(undefined, r("InfoJoce")) },
        { json: "statutAdoption", js: "statutAdoption", typ: u(undefined, r("TypeDeclaration")) },
        { json: "urlLegifrance", js: "urlLegifrance", typ: u(undefined, "") },
        { json: "referenceNOR", js: "referenceNOR", typ: u(undefined, "") },
        { json: "texteExterneAssocie", js: "texteExterneAssocie", typ: u(undefined, r("TexteExterneAssocie"))},
    ], false),
    "LibelleActe": o([
        { json: "nomCanonique", js: "nomCanonique", typ: "" },
        { json: "libelleCourt", js: "libelleCourt", typ: u(undefined, "") },
    ], false),
    "Rapporteur": o([
        { json: "acteurRef", js: "acteurRef", typ: "" },
        { json: "typeRapporteur", js: "typeRapporteur", typ: r("Qualite") },
        { json: "etudePLFRef", js: "etudePLFRef", typ: u(undefined, "") },
    ], false),
    "TypeDeclaration": o([
        { json: "fam_code", js: "fam_code", typ: "" },
        { json: "libelle", js: "libelle", typ: u(undefined, "") },
    ], false),
    "Rapporteurs": o([
        { json: "rapporteur", js: "rapporteur", typ: u(a(r("Rapporteur")), r("Rapporteur")) },
    ], false),
    "TextesAssocies": o([
        { json: "texteAssocie", js: "texteAssocie", typ: u(a(r("TexteAssocieElement")), r("TexteAssocieElement")) },
    ], false),
    "TexteAssocieElement": o([
        { json: "typeTexte", js: "typeTexte", typ: r("TypeTexte") },
        { json: "refTexteAssocie", js: "refTexteAssocie", typ: "" },
    ], false),
    "VoteRefs": o([
        { json: "voteRef", js: "voteRef", typ: u(a(""), "") },
    ], false),
    "ActeLegislatifAuteurs": o([
        { json: "acteurRef", js: "acteurRef", typ: u(a(""), "") },
    ], false),
    "ContributionInternaute": o([
        { json: "dateFermeture", js: "dateFermeture", typ: u(null, "") },
        { json: "dateOuverture", js: "dateOuverture", typ: u(undefined, "") },
    ], false),
    "ActeurElement": o([
        { json: "acteurRef", js: "acteurRef", typ: "" },
        { json: "mandatRef", js: "mandatRef", typ: "" },
    ], false),
    "InfoJoElement": o([
        { json: "typeJO", js: "typeJO", typ: r("TypeJo") },
        { json: "dateJO", js: "dateJO", typ: "" },
        { json: "pageJO", js: "pageJO", typ: null },
        { json: "numJO", js: "numJO", typ: "" },
        { json: "urlLegifrance", js: "urlLegifrance", typ: u(undefined, "") },
        { json: "referenceNOR", js: "referenceNOR", typ: u(undefined, "") },
    ], false),
    "FusionDossier": o([
        { json: "cause", js: "cause", typ: r("Cause") },
        { json: "dossierAbsorbantRef", js: "dossierAbsorbantRef", typ: "" },
    ], false),
    "Initiateur": o([
        { json: "acteurs", js: "acteurs", typ: u(undefined, r("InitiateurActeurs")) },
        { json: "organes", js: "organes", typ: u(undefined, r("Organes")) },
    ], false),
    "InitiateurActeurs": o([
        { json: "acteur", js: "acteur", typ: u(a(r("ActeurElement")), r("ActeurElement")) },
    ], false),
    "Organes": o([
        { json: "organe", js: "organe", typ: r("OrganesOrgane") },
    ], false),
    "OrganesOrgane": o([
        { json: "organeRef", js: "organeRef", typ: r("OrganeRefClass") },
    ], false),
    "OrganeRefClass": o([
        { json: "uid", js: "uid", typ: "" },
    ], false),
    "TitreDossier": o([
        { json: "titre", js: "titre", typ: "" },
        { json: "titreChemin", js: "titreChemin", typ: u(null, "") },
        { json: "senatChemin", js: "senatChemin", typ: u(null, "") },
    ], false),
    "DossiersLegislatifs": o([
        { json: "export", js: "export", typ: r("Export") },
    ], false),
    "Export": o([
        { json: "@xmlns:xsi", js: "@xmlns:xsi", typ: "" },
        { json: "textesLegislatifs", js: "textesLegislatifs", typ: r("TextesLegislatifs") },
        { json: "dossiersLegislatifs", js: "dossiersLegislatifs", typ: r("DossiersLegislatifsClass") },
    ], false),
    "DossiersLegislatifsClass": o([
        { json: "dossier", js: "dossier", typ: a(r("Dossier")) },
    ], false),
    "Dossier": o([
        { json: "dossierParlementaire", js: "dossierParlementaire", typ: r("DossierDossierParlementaire") },
    ], false),
    "DossierDossierParlementaire": o([
        { json: "@xsi:type", js: "@xsi:type", typ: u(undefined, r("DossierParlementaireXsiType")) },
        { json: "uid", js: "uid", typ: "" },
        { json: "legislature", js: "legislature", typ: "" },
        { json: "titreDossier", js: "titreDossier", typ: r("TitreDossier") },
        { json: "procedureParlementaire", js: "procedureParlementaire", typ: r("ProcedureParlementaire") },
        { json: "initiateur", js: "initiateur", typ: u(r("Initiateur"), null) },
        { json: "actesLegislatifs", js: "actesLegislatifs", typ: r("ActesLegislatifs") },
        { json: "indexation", js: "indexation", typ: u(undefined, r("Indexation")) },
        { json: "fusionDossier", js: "fusionDossier", typ: u(r("FusionDossier"), null) },
        { json: "PLF", js: "PLF", typ: u(undefined, r("Plf")) },
    ], false),
    "InfoJoce": o([
        { json: "refJOCE", js: "refJOCE", typ: "" },
        { json: "dateJOCE", js: "dateJOCE", typ: "" },
    ], false),
    "TexteEuropeen": o([
        { json: "typeTexteEuropeen", js: "typeTexteEuropeen", typ: "" },
        { json: "titreTexteEuropeen", js: "titreTexteEuropeen", typ: "" },
    ], false),
    "Indexation": o([
        { json: "themes", js: "themes", typ: r("Themes") },
    ], false),
    "Themes": o([
        { json: "@niveau", js: "@niveau", typ: "" },
        { json: "theme", js: "theme", typ: r("Theme") },
    ], false),
    "Theme": o([
        { json: "libelleTheme", js: "libelleTheme", typ: "" },
    ], false),
    "TextesLegislatifs": o([
        { json: "document", js: "document", typ: a(r("DocumentElement")) },
    ], false),
    "DocumentElement": o([
        { json: "@xsi:type", js: "@xsi:type", typ: r("DocumentXsiType") },
        { json: "uid", js: "uid", typ: "" },
        { json: "legislature", js: "legislature", typ: u(null, "") },
        { json: "cycleDeVie", js: "cycleDeVie", typ: r("CycleDeVie") },
        { json: "denominationStructurelle", js: "denominationStructurelle", typ: r("DocumentDenominationStructurelle") },
        { json: "provenance", js: "provenance", typ: u(undefined, r("Provenance")) },
        { json: "titres", js: "titres", typ: r("Titres") },
        { json: "divisions", js: "divisions", typ: u(r("Divisions"), null) },
        { json: "dossierRef", js: "dossierRef", typ: "" },
        { json: "redacteur", js: "redacteur", typ: null },
        { json: "classification", js: "classification", typ: r("Classification") },
        { json: "auteurs", js: "auteurs", typ: r("Auteurs") },
        { json: "correction", js: "correction", typ: u(r("Correction"), null) },
        { json: "notice", js: "notice", typ: r("Notice") },
        { json: "indexation", js: "indexation", typ: u(r("Indexation"), null) },
        { json: "imprimerie", js: "imprimerie", typ: u(r("Imprimerie"), null) },
        { json: "coSignataires", js: "coSignataires", typ: u(undefined, u(r("CoSignataires"), null)) },
        { json: "depotAmendements", js: "depotAmendements", typ: u(undefined, u(r("DepotAmendements"), null)) },
        { json: "rapportPublie", js: "rapportPublie", typ: u(undefined, "") },
    ], false),
    "TexteExterneAssocie": o([
        { json: "typeDoc", js: "typeDoc", typ: r("TexteExterneAssocieTypeDoc") },
        { json: "texteRef", js: "texteRef", typ: "" },
        { json: "titreDoc", js: "titreDoc", typ: "" },
        { json: "urlDoc", js: "urlDoc", typ: "" },
    ], false),
    "DocumentXsiType": [
        "accordInternational_Type",
        "avisConseilEtat_Type",
        "documentEtudeImpact_Type",
        "rapportParlementaire_Type",
        "texteLoi_Type",
    ],
    "Qualite": [
        "auteur",
        "cosignataire",
        "rapporteur",
        "rapporteur général",
        "rapporteur pour avis",
        "rapporteur spécial",
    ],
    "Code": [
        "ACCPRESRP",
        "APPART1515",
        "APPART341",
        "APPLOI",
        "AUE",
        "AUTRATCONV",
        "AVECVOTE",
        "CENSURE492",
        "CENSURE493",
        "COMENQ",
        "COMPA",
        "COMSPCPTE",
        "CONST",
        "CTRLBUDG",
        "DIVERS",
        "ENGRESPTXT",
        "ENQU",
        "FIN",
        "FINGEST",
        "FINRECT",
        "FINSSOC",
        "FINSSOCREC",
        "IMPACTLOIS",
        "LEGDELEGAN",
        "MINFOCOMPER",
        "MODREGLTAN",
        "NOTEOPECST",
        "OFFPARL",
        "ORG",
        "PETITION",
        "PRESIDAGE",
        "PRESIDASS",
        "PRESIDREP",
        "PROPACTCOM",
        "PRPDIT",
        "RECT",
        "REFART11",
        "RGLTBUDG",
        "RGLTBUDG2",
        "SUPP",
        "SUSPOURS",
        "TVXINSTITEUROP",
    ],
    "StatutAdoption": [
        "ADOPTCOM",
        "ADOPTSEANCE",
        "REJETSEANCE",
    ],
    "TypeCorrection": [
        "Rectifié",
    ],
    "DocumentDenominationStructurelle": [
        "Allocution",
        "Avis",
        "Déclaration",
        "Lettre",
        "Message",
        "Motion",
        "Projet de loi",
        "Proposition de loi",
        "Proposition de résolution",
        "Rapport",
        "Rapport d'information",
        "Résolution",
    ],
    "Provenance": [
        "Commission",
        "Séance",
        "Texte Déposé",
    ],
    "DossierParlementaireXsiType": [
        "DossierCommissionEnquete_Type",
        "DossierIniativeExecutif_Type",
        "DossierLegislatif_Type",
        "DossierMissionControle_Type",
        "DossierMissionInformation_Type",
        "DossierResolutionAN",
    ],
    "TypeBudget": [
        "Budget annexe",
        "Budget général",
        "Compte de concours financier",
        "Compte spécial",
        "Première partie",
    ],
    "TypeMission": [
        "mission principale",
        "mission secondaire",
        "partie de mission",
    ],
    "ActeLegislatifXsiType": [
        "Adoption_Europe_Type",
        "ConclusionEtapeCC_Type",
        "ConvocationCongres_Type",
        "CreationOrganeTemporaire_Type",
        "DecisionMotionCensure_Type",
        "DecisionRecevabiliteBureau_Type",
        "Decision_Type",
        "DeclarationGouvernement_Type",
        "DepotAccordInternational_Type",
        "DepotAvisConseilEtat_Type",
        "DepotInitiativeNavette_Type",
        "DepotInitiative_Type",
        "DepotLettreRectificative_Type",
        "DepotMotionCensure_Type",
        "DepotMotionReferendaire_Type",
        "DepotRapport_Type",
        "DiscussionCommission_Type",
        "DiscussionSeancePublique_Type",
        "Etape_Type",
        "EtudeImpact_Type",
        "MotionProcedure_Type",
        "NominRapporteurs_Type",
        "ProcedureAccelere_Type",
        "Promulgation_Type",
        "RenvoiCMP_Type",
        "RenvoiPrealable_Type",
        "RetraitInitiative_Type",
        "SaisieComAvis_Type",
        "SaisieComFond_Type",
        "SaisineConseilConstit_Type",
    ],
    "CodeActe": Object.values(CodeActe),
    "TypeTexte": [
        "BTA",
        "TAP",
    ],
    "Motif": [
        "En application de l'article 61§2 de la Constitution",
    ],
    "TypeJo": [
        "JO_LOI_DECRET",
    ],
    "Cause": [
        "Dossier absorbé",
        "Examen commun",
    ],
    "TexteExterneAssocieTypeDoc": [
        "Décret",
    ],
};
