import commandLineArgs from "command-line-args"

import { EnabledDatasets } from "../datasets"
import { loadAssembleeData } from "../loaders"
import { EtatCycleDeVie } from "../types/agendas"
import { dataDirDefaultOption } from "./shared/cli_helpers"
import { Legislature } from "../types/legislatures"

const optionsDefinitions = [dataDirDefaultOption]
const options = commandLineArgs(optionsDefinitions)

const now = new Date()
const { reunionByUid } = loadAssembleeData(
  options.dataDir,
  EnabledDatasets.Agendas,
  Legislature.DixSept,
)
for (const reunion of Object.values(reunionByUid)) {
  if (
    ![EtatCycleDeVie.Confirmé, EtatCycleDeVie.Eventuel].includes(
      reunion.cycleDeVie.etat,
    )
  ) {
    continue
  }
  const { timestampDebut, typeReunion, uid } = reunion
  if (typeReunion !== "GP") {
    continue
  }
  if (
    timestampDebut.getFullYear() !== now.getFullYear() ||
    timestampDebut.getMonth() !== now.getMonth() ||
    timestampDebut.getDate() !== now.getDate()
  ) {
    continue
  }
  console.log(uid, timestampDebut, JSON.stringify(reunion, null, 2))
}
console.log(now)
