import commandLineArgs from "command-line-args"
import {
  Dataset,
  datasets,
  EnabledDatasets,
  getEnabledDatasets,
} from "../datasets"
import { Legislature } from "../types/legislatures"
import {
  categoriesOption,
  dataDirDefaultOption,
  fetchOption,
  fetchDocumentsOption,
  legislatureOption,
  silentOption,
  parseDocumentsOption,
  documentFormatOption,
} from "./shared/cli_helpers"
import { processDataset } from "./process_open_dataset"

const optionsDefinitions = [
  categoriesOption,
  legislatureOption,
  silentOption,
  dataDirDefaultOption,
  fetchOption,
  fetchDocumentsOption,
  parseDocumentsOption,
  documentFormatOption,
]
const options = commandLineArgs(optionsDefinitions)

async function retrieveOpenData(dataDir: string): Promise<void> {
  const enabledDatasets = getEnabledDatasets(options.categories)

  let chosenDatasets = [
    ...(enabledDatasets & EnabledDatasets.ActeursEtOrganes
      ? datasets.acteursEtOrganes
      : []),
    ...(enabledDatasets & EnabledDatasets.Agendas ? datasets.agendas : []),
    ...(enabledDatasets & EnabledDatasets.Amendements
      ? datasets.amendements
      : []),
    ...(enabledDatasets & EnabledDatasets.DossiersLegislatifs
      ? datasets.dossiersLegislatifs
      : []),
    ...(enabledDatasets & EnabledDatasets.Scrutins ? datasets.scrutins : []),
    ...(enabledDatasets & EnabledDatasets.Questions ? datasets.questions : []),
    ...(enabledDatasets & EnabledDatasets.ComptesRendusSeances
      ? datasets.comptesRendusSeances
      : []),
  ]

  if (options.legislature !== 0 && options.legislature !== undefined) {
    chosenDatasets = chosenDatasets.filter(
      (dataset: Dataset) =>
        dataset.legislature === Legislature.All ||
        options.legislature === dataset.legislature,
    )
  }

  for (const dataset of chosenDatasets) {
    await processDataset({ dataDir, dataset, options })
  }
}

retrieveOpenData(options.dataDir).catch((error) => {
  console.log(error)
  process.exit(1)
})
