/// Deprecated: Use script retrieve_documents.ts instead.

import assert from "assert"
import commandLineArgs from "command-line-args"
import fs from "fs-extra"
import path from "path"

import { EnabledDatasets } from "../datasets"
import * as git from "../git"
import { loadAssembleeData } from "../loaders"
import { TypeDocument } from "../types/dossiers_legislatifs"
import { DocumentUrlFormat, urlFromDocument } from "../urls"
import {
  commitOption,
  dataDirDefaultOption,
  legislatureOption,
  remoteOption,
  silentOption,
  verboseOption,
} from "./shared/cli_helpers"

const optionsDefinitions = [
  commitOption,
  legislatureOption,
  remoteOption,
  silentOption,
  verboseOption,
  dataDirDefaultOption,
  {
    alias: "u",
    help: "UID of first Assemblée's document to retrieve",
    name: "uid",
    type: String,
  },
]
const options = commandLineArgs(optionsDefinitions)

/// Deprecated: Use script retrieve_documents.ts instead.
async function retrieveDocuments(): Promise<number> {
  assert(
    !options.commit || options.uid,
    'Options "commit" & "uid" are incompatible',
  )

  const dataDir = options.dataDir
  const { documentByUid } = loadAssembleeData(
    dataDir,
    EnabledDatasets.DossiersLegislatifs,
    options.legislature,
  )

  const textesLoisDir = path.join(
    dataDir,
    "www.assemblee-nationale.fr",
    options.legislature,
    "textes",
  )
  fs.ensureDirSync(textesLoisDir)
  if (!options.uid) {
    for (const filename of fs.readdirSync(textesLoisDir)) {
      if (filename[0] === ".") {
        continue
      }
      fs.removeSync(path.join(textesLoisDir, filename))
    }
  }

  const textesLois = Object.values(documentByUid)
    .filter((document) => document.xsiType === TypeDocument.TexteLoiType)
    .filter(
      (document) =>
        document.uid.startsWith("PIONAN") || document.uid.startsWith("PRJLAN"),
    )
    .map((document) => {
      return {
        ...document,
        rawDocumentAssembleeUrl: urlFromDocument(
          document,
          DocumentUrlFormat.RawHtml,
        ),
      }
    })
    .sort((a, b) =>
      a.rawDocumentAssembleeUrl === null
        ? -1
        : b.rawDocumentAssembleeUrl === null
          ? 1
          : a.rawDocumentAssembleeUrl.localeCompare(b.rawDocumentAssembleeUrl),
    )

  const firstUid = options.uid
  let skip = !!firstUid
  for (const texteLoi of textesLois) {
    if (skip) {
      if (texteLoi.uid === firstUid) {
        skip = false
      } else {
        continue
      }
    }
    if (!options.silent) {
      console.log(
        `Retrieving bill ${texteLoi.uid} at ${texteLoi.rawDocumentAssembleeUrl}`,
      )
    }
    if (texteLoi.rawDocumentAssembleeUrl === null) {
      continue
    }
    const url = new URL(texteLoi.rawDocumentAssembleeUrl)
    const filePath = path.join(
      dataDir,
      url.hostname,
      ...url.pathname.split("/"),
    )
    assert.strictEqual(textesLoisDir, path.dirname(filePath))

    const response = await fetch(texteLoi.rawDocumentAssembleeUrl)
    const page = await response.text()
    if (!response.ok) {
      if (response.status !== 404) {
        console.error(
          `Error while getting page "${
            texteLoi.rawDocumentAssembleeUrl
          }" (uid: ${texteLoi.uid}):\n\nError:\n${JSON.stringify(
            { code: response.status, message: response.statusText },
            null,
            2,
          )}`,
        )
      }
      continue
    }
    fs.writeFileSync(filePath, page)
  }

  if (options.commit) {
    return git.commitAndPush(textesLoisDir, "Nouvelle moisson", options.remote)
  }
  return 0
}

retrieveDocuments()
  .then((exitCode) => process.exit(exitCode))
  .catch((error) => {
    console.log(error)
    process.exit(1)
  })
