/** Load all big non-split open data files in RAM as a test. */

import assert from "assert"
import commandLineArgs from "command-line-args"

import { EnabledDatasets } from "../datasets"
import { loadAssembleeDataFromBigFiles } from "../loaders"
import { Legislature } from "../types/legislatures"
import { categoriesOption, dataDirDefaultOption } from "./shared/cli_helpers"

const optionsDefinitions = [categoriesOption, dataDirDefaultOption]
const options = commandLineArgs(optionsDefinitions)

options.categories.map((datasetName: string) =>
  assert.notStrictEqual(
    (EnabledDatasets as any as { [name: string]: EnabledDatasets })[
      datasetName
    ],
    undefined,
    `Unknown name of dataset: ${datasetName}`,
  ),
)
const enabledDatasets = options.categories.reduce(
  (enabledDatasets: EnabledDatasets, datasetName: string): EnabledDatasets =>
    enabledDatasets |
    (EnabledDatasets as any as { [name: string]: EnabledDatasets })[
      datasetName
    ],
  EnabledDatasets.None,
)

loadAssembleeDataFromBigFiles(
  options.dataDir,
  enabledDatasets,
  Legislature.All,
  {
    log: true,
  },
)
