
## Generating schemas and documentation (for contributors only)

### Initial generation of TypeScript & JSON schema files from JSON data

```bash
npm install -g quicktype @types/readable-stream
```

```bash
npx quicktype --acronym-style=camel -o src/raw_types/acteurs_et_organes.ts ../assemblee-data/AMO{10,20,30,40,50}_*.json
```

```bash
npx quicktype --acronym-style=camel -o src/raw_types/agendas.ts ../assemblee-data/Agenda_{XIV,XV}.json
```

```bash
npx babel-node --extensions ".ts" --max-old-space-size=8192 --  src/scripts/raw_types_from_amendements.ts ../assemblee-data/
```

```bash
npx quicktype --acronym-style=camel -o src/raw_types/dossiers_legislatifs.ts ../assemblee-data/Dossiers_Legislatifs_{XIV,XV}.json
```

### Updating JSON schema files and validating JSON files

* Convert src/types/*.ts into JSON schemas for comparison purposes

```bash
for f in src/types/*.ts ; do b=$(basename $f .ts) ; npx typescript-json-schema src/types/$b.ts '*' > src/schemas/converted_from_type/$b.json ; done
```

* Manually update src/schemas/*/*.json to account for these differences
* Verify the JSON files validate with the updated schema

```bash
npx babel-node --extensions .ts -- src/scripts/validate_json.ts --repository=$(git rev-parse --show-toplevel) --dataset ../data/assemblee-nettoye/AMO*nettoye
npx babel-node --extensions .ts -- src/scripts/validate_json.ts --repository=$(git rev-parse --show-toplevel) --dataset ../data/assemblee-nettoye/Dossiers_Legislatifs_XV_nettoye
etc.
```

If an error occurs and the schema must be fixed:

* Verify the schema works by using **--dev** to use the schema from the current working directory instead of fetching them from the tag maching the version mentionned in the JSON file. For instance, if the file `acteurs/PA766283.json` has `schemaVersion = "acteur-1.0"` it will use the schema found at [schema-acteur-1.0](https://git.en-root.org/tricoteuses/tricoteuses-assemblee/tree/schema-acteur-1.0) and **not** the current working directory, except if **--dev** is used.
* Once the schema is verified to work, add a tag matching the directory of the schema. For instance for `amendement/Amendement.json` or any of its references (i.e. `amendement/*.json`), set the tag `schema-amendement-X.Y`.
  * If the schema change is backward compatible (i.e. software using the corresponding JSON won't break), increment Y (X.1, X.2, ...)
  * If the schema change is not backward compatible, increment X and set Y to zero (1.0, 2.0, ...)

The tag with the highest version will be used by `src/scripts/clean_reorganized_data.ts` to add a `schemaVersion` field for all JSON files created in a `*_nettoye` repository from that point on. The goal is for a JSON file to validate against an immutable schema identified by a version tag and to all each JSON file to have a different version of the schema.

See the [discussion in the forum](https://forum.en-root.org/t/release-management/85/) for more information and further discussion.

### Helpers to create documentation
All raw files are kept during the process
```bash
$ npx babel-node --extensions .ts -- src/scripts/document_dossiers_legislatifs.ts --data ../data/assemblee-nettoye/Dossiers_Legislatifs_{XIV,XV}_nettoye/dossiers/**/*.json
```

See the [data-site README](https://git.en-root.org/tricoteuses/data-site/blob/master/README.md) for more information about how it is used.
