// To parse this data:
//
//   import { Convert, Acteur, Organe, ActeursEtOrganes } from "./file";
//
//   const acteur = Convert.toActeur(json);
//   const organe = Convert.toOrgane(json);
//   const acteursEtOrganes = Convert.toActeursEtOrganes(json);
//
// These functions will throw an error if the JSON doesn't
// match the expected interface, even if the JSON is valid.

export interface ActeursEtOrganes {
  organes: Organe[]
  acteurs: Acteur[]
}

export interface Acteur {
  schemaVersion?: string
  uid: string
  etatCivil: EtatCivil
  profession?: Profession
  uriHatvp?: string
  adresses?: Adresse[]
  mandats?: Mandat[]
  photo?: Photo
  deports?: Deport[]
}

export interface EtatCivil {
  ident: Ident
  infoNaissance: InfoNaissance
  dateDeces?: Date
}

export interface Ident {
  civ: Civ
  prenom: string
  nom: string
  alpha: string
  trigramme?: string
}

export enum Civ {
  M = "M.",
  Mme = "Mme",
}

export interface InfoNaissance {
  dateNais?: Date
  villeNais?: string
  depNais?: string
  paysNais?: string
}

export type Adresse = AdresseElectronique | AdressePostale

export interface AdresseElectronique {
  xsiType:
    | "AdresseMail_Type"
    | "AdresseSiteWeb_Type"
    | "AdresseTelephonique_Type"
  uid: string
  type: string
  typeLibelle: string
  poids?: string
  adresseDeRattachement?: string
  valElec?: string
}

export interface AdressePostale {
  xsiType: "AdressePostale_Type"
  uid: string
  type: string
  typeLibelle: string
  poids?: string
  intitule?: string
  numeroRue?: string
  nomRue?: string
  complementAdresse?: string
  codePostal?: string
  ville?: string
}

export interface Mandat {
  xsiType?: TypeMandat
  uid: string
  acteurRef: string
  legislature?: string
  typeOrgane: CodeTypeOrgane
  dateDebut: Date
  datePublication?: Date
  dateFin?: Date
  preseance?: string
  nominPrincipale: string
  infosQualite: InfosQualite
  organesRefs: string[]
  organes?: Organe[] // Added by Tricoteuses
  suppleant?: Suppleant
  election?: Election
  mandature?: Mandature
  collaborateurs?: Collaborateur[]
  libelle?: string
  missionSuivanteRef?: string
  missionPrecedenteRef?: string
}

export enum TypeMandat {
  MandatAvecSuppleantType = "MandatAvecSuppleant_Type",
  MandatMissionType = "MandatMission_Type",
  MandatParlementaireType = "MandatParlementaire_type",
  MandatSimpleType = "MandatSimple_Type",
}

export interface Collaborateur {
  qualite?: Civ
  prenom: string
  nom: string
}

export interface Election {
  lieu: Lieu
  causeMandat?: string
  refCirconscription?: string
}

export interface Lieu {
  region?: string
  regionType?: RegionType
  departement?: string
  numDepartement?: string
  numCirco?: string
}

export enum RegionType {
  CollectivitésDOutreMerEtNouvelleCalédonie = "Collectivités d'outre-mer et Nouvelle-Calédonie",
  Dom = "Dom",
  FrançaisÉtablisHorsDeFrance = "Français établis hors de France",
  Métropolitain = "Métropolitain",
}

export interface InfosQualite {
  codeQualite?: string
  libQualite: string
  libQualiteSex?: string
}

export interface Mandature {
  datePriseFonction?: Date
  causeFin?: string
  premiereElection?: string
  placeHemicycle?: string
  mandatRemplaceRef?: string
  matriculeVote?: string
}

export interface Suppleant {
  dateDebut: Date
  dateFin?: Date
  suppleantRef: string
}

export enum CodeTypeOrgane {
  Api = "API", // Assemblée parlementaire internationale
  Assemblee = "ASSEMBLEE", // Assemblée nationale
  Bureau = "BUREAU", // Bureau de l'Assemblée nationale
  Circonscription = "CIRCONSCRIPTION",
  Cjr = "CJR", // Cour de justice de la République
  Cmp = "CMP", // Commission mixte paritaire,
  Cnpe = "CNPE", // Commission d'enquête; what does this acronym mean?
  Cnps = "CNPS", // Commission spéciale; what does this acronym mean?
  Comnl = "COMNL", // Commission spéciale autre; what does this acronym mean?
  Comper = "COMPER", // Commission permanente
  Comsenat = "COMSENAT", // Commission sénatoriale
  Comspsenat = "COMSPSENAT", // Commission spéciale sénatoriale
  Confpt = "CONFPT", // Conférence des présidents
  Constitu = "CONSTITU", // Conseil constitutionnel
  Deleg = "DELEG", // Délégation
  Delegbureau = "DELEGBUREAU", // Délégation du Bureau
  Delegsenat = "DELEGSENAT", // Délégation sénatoriale
  Ga = "GA", // Groupe d'amitié
  Ge = "GE", // Groupe d'études
  Gevi = "GEVI", // Groupe d'études à vocation internationale
  Gouvernement = "GOUVERNEMENT", // Gouvernement
  Gp = "GP", // Groupe politique
  Groupesenat = "GROUPESENAT", // Groupe sénatorial
  Hcj = "HCJ", // Haute Cour de Justice
  Ministere = "MINISTERE", // Ministère
  Miseval = "MISEVAL", // Mission d'évaluation
  Misinfo = "MISINFO", // Mission d'information
  Misinfocom = "MISINFOCOM", // Mission d'information commune
  Misinfopre = "MISINFOPRE", // Missions d’information de la conférence des Présidents
  Offpar = "OFFPAR", // Office parlementaire ou délégation mixte
  Orgaint = "ORGAINT", // Organisme international
  Orgextparl = "ORGEXTPARL", // Organisme extra-parlementaire
  Parpol = "PARPOL", // Parti politique
  Presrep = "PRESREP", // Présidence de la République
  Senat = "SENAT", // Sénat
}

export interface Profession {
  libelleCourant?: string
  socProcInsee: SocProcInsee
}

export interface SocProcInsee {
  catSocPro?: string
  famSocPro?: FamSocPro
}

export enum FamSocPro {
  AgriculteursExploitants = "Agriculteurs exploitants",
  ArtisansCommerçantsEtChefsDEntreprise = "Artisans, commerçants et chefs d'entreprise",
  AutresPersonnesSansActivitéProfessionnelle = "Autres personnes sans activité professionnelle",
  CadresEtProfessionsIntellectuellesSupérieures = "Cadres et professions intellectuelles supérieures",
  Employés = "Employés",
  Ouvriers = "Ouvriers",
  ProfessionsIntermédiaires = "Professions Intermédiaires",
  Retraités = "Retraités",
  SansProfessionDéclarée = "Sans profession déclarée",
}

export interface Deport {
  uid: string
  chronotag: string
  legislature: string
  refActeur: string
  dateCreation: Date
  datePublication: Date
  portee: CodeEtLibelle
  lecture: CodeEtLibelle
  instance: CodeEtLibelle
  cible: Cible
  explication?: string
}

export interface Cible {
  type: CodeEtLibelle
  referenceTextuelle: string
}

export interface CodeEtLibelle {
  code: string
  libelle: string
}

export interface Organe {
  schemaVersion?: string
  xsiType?: TypeOrgane
  uid: string
  codeType: CodeTypeOrgane
  libelle: string
  libelleEdition?: string
  libelleAbrege: string
  libelleAbrev: string
  viMoDe: ViMoDe
  organeParentRef?: string // Renamed by Tricoteuses
  organeParent?: Organe // Added by Tricoteuses
  organePrecedentRef?: string
  organePrecedent?: Organe // Added by Tricoteuses
  regime?: Regime
  legislature?: string
  secretariat?: Secretariat
  numero?: string
  lieu?: OrganeLieu
  regimeJuridique?: string
  siteInternet?: string
  nombreReunionsAnnuelles?: string
  positionPolitique?: PositionPolitique
  preseance?: string
  couleurAssociee?: string
  listePays?: string[]
}

export enum TypeOrgane {
  GroupePolitiqueType = "GroupePolitique_type",
  OrganeCirconscriptionType = "OrganeCirconscription_type",
  OrganeExterneType = "OrganeExterne_Type",
  OrganeExtraParlementaireType = "OrganeExtraParlementaire_type",
  OrganeParlementaireInternational = "OrganeParlementaireInternational",
  OrganeParlementaireType = "OrganeParlementaire_Type",
}

export interface OrganeLieu {
  region: Region
  departement: Departement
}

export interface Departement {
  codeNatureDep?: CodeNatureDep
  code?: string
  libelle?: string
}

export enum CodeNatureDep {
  H = "H",
  M = "M",
  O = "O",
  T = "T",
}

export interface Region {
  type?: RegionType
  libelle?: string
}

export enum PositionPolitique {
  Majoritaire = "Majoritaire",
  Minoritaire = "Minoritaire",
  Opposition = "Opposition",
}

export enum Regime {
  AssembléeConsultativeProvisoire = "Assemblée consultative provisoire",
  AssembléesNationalesConstituantes = "Assemblées nationales constituantes",
  The3ÈmeRépublique = "3ème République",
  The4ÈmeRépublique = "4ème République",
  The5ÈmeRépublique = "5ème République",
}

export interface Secretariat {
  secretaire01?: string
  secretaire02?: string
}

export interface ViMoDe {
  dateDebut?: Date
  dateAgrement?: Date
  dateFin?: Date
}

export interface Photo {
  chemin: string
  cheminMosaique: string
  hauteur: number
  largeur: number
  xMosaique: number
  yMosaique: number
}

// Converts JSON strings to/from your types
// and asserts the results of JSON.parse at runtime
export class Convert {
  public static toActeursEtOrganes(json: string): ActeursEtOrganes {
    return cast(JSON.parse(json), r("ActeursEtOrganes"))
  }

  public static acteursEtOrganesToJson(value: ActeursEtOrganes): string {
    return JSON.stringify(uncast(value, r("ActeursEtOrganes")), null, 2)
  }

  public static toActeur(json: string): Acteur {
    return cast(JSON.parse(json), r("Acteur"))
  }

  public static acteurToJson(value: Acteur): string {
    return JSON.stringify(uncast(value, r("Acteur")), null, 2)
  }

  public static toOrgane(json: string): Organe {
    return cast(JSON.parse(json), r("Organe"))
  }

  public static organeToJson(value: Organe): string {
    return JSON.stringify(uncast(value, r("Organe")), null, 2)
  }
}

function invalidValue(typ: any, val: any, key: any = ""): never {
  if (key) {
    throw Error(
      `Invalid value for key "${key}". Expected type ${JSON.stringify(
        typ,
      )} but got ${JSON.stringify(val)}`,
    )
  }
  throw Error(
    `Invalid value ${JSON.stringify(val)} for type ${JSON.stringify(typ)}`,
  )
}

function jsonToJSProps(typ: any): any {
  if (typ.jsonToJS === undefined) {
    const map: any = {}
    typ.props.forEach((p: any) => (map[p.json] = { key: p.js, typ: p.typ }))
    typ.jsonToJS = map
  }
  return typ.jsonToJS
}

function jsToJSONProps(typ: any): any {
  if (typ.jsToJSON === undefined) {
    const map: any = {}
    typ.props.forEach((p: any) => (map[p.js] = { key: p.json, typ: p.typ }))
    typ.jsToJSON = map
  }
  return typ.jsToJSON
}

function transform(val: any, typ: any, getProps: any, key: any = ""): any {
  function transformPrimitive(typ: string, val: any): any {
    if (typeof typ === typeof val) return val
    return invalidValue(typ, val, key)
  }

  function transformUnion(typs: any[], val: any): any {
    // val must validate against one typ in typs
    const l = typs.length
    for (let i = 0; i < l; i++) {
      const typ = typs[i]
      try {
        return transform(val, typ, getProps)
      } catch (_) {}
    }
    return invalidValue(typs, val)
  }

  function transformEnum(cases: string[], val: any): any {
    if (cases.indexOf(val) !== -1) return val
    return invalidValue(cases, val)
  }

  function transformArray(typ: any, val: any): any {
    // val must be an array with no invalid elements
    if (!Array.isArray(val)) return invalidValue("array", val)
    return val.map((el) => transform(el, typ, getProps))
  }

  function transformDate(val: any): any {
    if (val === null) {
      return null
    }
    const d = new Date(val)
    if (isNaN(d.valueOf())) {
      return invalidValue("Date", val)
    }
    return d
  }

  function transformObject(
    props: { [k: string]: any },
    additional: any,
    val: any,
  ): any {
    if (val === null || typeof val !== "object" || Array.isArray(val)) {
      return invalidValue("object", val)
    }
    const result: any = {}
    Object.getOwnPropertyNames(props).forEach((key) => {
      const prop = props[key]
      const v = Object.prototype.hasOwnProperty.call(val, key)
        ? val[key]
        : undefined
      result[prop.key] = transform(v, prop.typ, getProps, prop.key)
    })
    Object.getOwnPropertyNames(val).forEach((key) => {
      if (!Object.prototype.hasOwnProperty.call(props, key)) {
        result[key] = transform(val[key], additional, getProps, key)
      }
    })
    return result
  }

  if (typ === "any") return val
  if (typ === null) {
    if (val === null) return val
    return invalidValue(typ, val)
  }
  // if (typ === false) return invalidValue(typ, val)
  while (typeof typ === "object" && typ.ref !== undefined) {
    typ = typeMap[typ.ref]
  }
  if (Array.isArray(typ)) return transformEnum(typ, val)
  if (typeof typ === "object") {
    return typ.hasOwnProperty("unionMembers")
      ? transformUnion(typ.unionMembers, val)
      : typ.hasOwnProperty("arrayItems")
        ? transformArray(typ.arrayItems, val)
        : typ.hasOwnProperty("props")
          ? transformObject(getProps(typ), typ.additional, val)
          : invalidValue(typ, val)
  }
  // Numbers can be parsed by Date but shouldn't be.
  if (typ === Date && typeof val !== "number") return transformDate(val)
  return transformPrimitive(typ, val)
}

function cast<T>(val: any, typ: any): T {
  return transform(val, typ, jsonToJSProps)
}

function uncast<T>(val: T, typ: any): any {
  return transform(val, typ, jsToJSONProps)
}

function a(typ: any) {
  return { arrayItems: typ }
}

function u(...typs: any[]) {
  return { unionMembers: typs }
}

function o(props: any[], additional: any) {
  return { props, additional }
}

// function m(additional: any) {
//   return { props: [], additional }
// }

function r(name: string) {
  return { ref: name }
}

const typeMap: any = {
  ActeursEtOrganes: o(
    [
      { json: "organes", js: "organes", typ: a(r("Organe")) },
      { json: "acteurs", js: "acteurs", typ: a(r("Acteur")) },
    ],
    false,
  ),
  Acteur: o(
    [
      { json: "schemaVersion", js: "schemaVersion", typ: u(undefined, "") },
      { json: "uid", js: "uid", typ: "" },
      { json: "etatCivil", js: "etatCivil", typ: r("EtatCivil") },
      {
        json: "profession",
        js: "profession",
        typ: u(undefined, r("Profession")),
      },
      { json: "uriHatvp", js: "uriHatvp", typ: u(undefined, "") },
      {
        json: "adresses",
        js: "adresses",
        typ: u(undefined, a(u(r("AdresseElectronique"), r("AdressePostale")))),
      },
      { json: "mandats", js: "mandats", typ: u(undefined, a(r("Mandat"))) },
      { json: "photo", js: "photo", typ: u(undefined, r("Photo")) },
      { json: "deports", js: "deports", typ: u(undefined, a(r("Deport"))) },
    ],
    false,
  ),
  AdresseElectronique: o(
    [
      { json: "xsiType", js: "xsiType", typ: "" },
      { json: "uid", js: "uid", typ: "" },
      { json: "type", js: "type", typ: "" },
      { json: "typeLibelle", js: "typeLibelle", typ: "" },
      { json: "poids", js: "poids", typ: u(undefined, "") },
      {
        json: "adresseDeRattachement",
        js: "adresseDeRattachement",
        typ: u(undefined, ""),
      },
      { json: "valElec", js: "valElec", typ: u(undefined, "") },
    ],
    false,
  ),
  AdressePostale: o(
    [
      { json: "xsiType", js: "xsiType", typ: "" },
      { json: "uid", js: "uid", typ: "" },
      { json: "type", js: "type", typ: "" },
      { json: "typeLibelle", js: "typeLibelle", typ: "" },
      { json: "poids", js: "poids", typ: u(undefined, "") },
      { json: "intitule", js: "intitule", typ: u(undefined, "") },
      { json: "numeroRue", js: "numeroRue", typ: u(undefined, "") },
      { json: "nomRue", js: "nomRue", typ: u(undefined, "") },
      {
        json: "complementAdresse",
        js: "complementAdresse",
        typ: u(undefined, ""),
      },
      { json: "codePostal", js: "codePostal", typ: u(undefined, "") },
      { json: "ville", js: "ville", typ: u(undefined, "") },
    ],
    false,
  ),
  EtatCivil: o(
    [
      { json: "ident", js: "ident", typ: r("Ident") },
      { json: "infoNaissance", js: "infoNaissance", typ: r("InfoNaissance") },
      { json: "dateDeces", js: "dateDeces", typ: u(undefined, Date) },
    ],
    false,
  ),
  Ident: o(
    [
      { json: "civ", js: "civ", typ: r("Civ") },
      { json: "prenom", js: "prenom", typ: "" },
      { json: "nom", js: "nom", typ: "" },
      { json: "alpha", js: "alpha", typ: "" },
      { json: "trigramme", js: "trigramme", typ: u(undefined, "") },
    ],
    false,
  ),
  InfoNaissance: o(
    [
      { json: "dateNais", js: "dateNais", typ: u(undefined, Date) },
      { json: "villeNais", js: "villeNais", typ: u(undefined, "") },
      { json: "depNais", js: "depNais", typ: u(undefined, "") },
      { json: "paysNais", js: "paysNais", typ: u(undefined, "") },
    ],
    false,
  ),
  Mandat: o(
    [
      { json: "xsiType", js: "xsiType", typ: u(undefined, r("TypeMandat")) },
      { json: "uid", js: "uid", typ: "" },
      { json: "acteurRef", js: "acteurRef", typ: "" },
      { json: "legislature", js: "legislature", typ: u(undefined, "") },
      { json: "typeOrgane", js: "typeOrgane", typ: r("CodeTypeOrgane") },
      { json: "dateDebut", js: "dateDebut", typ: Date },
      {
        json: "datePublication",
        js: "datePublication",
        typ: u(undefined, Date),
      },
      { json: "dateFin", js: "dateFin", typ: u(undefined, Date) },
      { json: "preseance", js: "preseance", typ: u(undefined, "") },
      { json: "nominPrincipale", js: "nominPrincipale", typ: "" },
      { json: "infosQualite", js: "infosQualite", typ: r("InfosQualite") },
      { json: "organesRefs", js: "organesRefs", typ: a("") },
      { json: "suppleant", js: "suppleant", typ: u(undefined, r("Suppleant")) },
      { json: "election", js: "election", typ: u(undefined, r("Election")) },
      { json: "mandature", js: "mandature", typ: u(undefined, r("Mandature")) },
      {
        json: "collaborateurs",
        js: "collaborateurs",
        typ: u(undefined, a(r("Collaborateur"))),
      },
      { json: "libelle", js: "libelle", typ: u(undefined, "") },
      {
        json: "missionSuivanteRef",
        js: "missionSuivanteRef",
        typ: u(undefined, ""),
      },
      {
        json: "missionPrecedenteRef",
        js: "missionPrecedenteRef",
        typ: u(undefined, ""),
      },
    ],
    false,
  ),
  Collaborateur: o(
    [
      { json: "qualite", js: "qualite", typ: u(undefined, r("Civ")) },
      { json: "prenom", js: "prenom", typ: "" },
      { json: "nom", js: "nom", typ: "" },
    ],
    false,
  ),
  Election: o(
    [
      { json: "lieu", js: "lieu", typ: r("Lieu") },
      { json: "causeMandat", js: "causeMandat", typ: u(undefined, "") },
      {
        json: "refCirconscription",
        js: "refCirconscription",
        typ: u(undefined, ""),
      },
    ],
    false,
  ),
  Lieu: o(
    [
      { json: "region", js: "region", typ: u(undefined, "") },
      {
        json: "regionType",
        js: "regionType",
        typ: u(undefined, r("RegionType")),
      },
      { json: "departement", js: "departement", typ: u(undefined, "") },
      { json: "numDepartement", js: "numDepartement", typ: u(undefined, "") },
      { json: "numCirco", js: "numCirco", typ: u(undefined, "") },
    ],
    false,
  ),
  InfosQualite: o(
    [
      { json: "codeQualite", js: "codeQualite", typ: u(undefined, "") },
      { json: "libQualite", js: "libQualite", typ: "" },
      { json: "libQualiteSex", js: "libQualiteSex", typ: u(undefined, "") },
    ],
    false,
  ),
  Mandature: o(
    [
      {
        json: "datePriseFonction",
        js: "datePriseFonction",
        typ: u(undefined, Date),
      },
      { json: "causeFin", js: "causeFin", typ: u(undefined, "") },
      {
        json: "premiereElection",
        js: "premiereElection",
        typ: u(undefined, ""),
      },
      { json: "placeHemicycle", js: "placeHemicycle", typ: u(undefined, "") },
      {
        json: "mandatRemplaceRef",
        js: "mandatRemplaceRef",
        typ: u(undefined, ""),
      },
      { json: "matriculeVote", js: "matriculeVote", typ: u(undefined, "") },
    ],
    false,
  ),
  Suppleant: o(
    [
      { json: "dateDebut", js: "dateDebut", typ: Date },
      { json: "dateFin", js: "dateFin", typ: u(undefined, Date) },
      { json: "suppleantRef", js: "suppleantRef", typ: "" },
    ],
    false,
  ),
  Profession: o(
    [
      { json: "libelleCourant", js: "libelleCourant", typ: u(undefined, "") },
      { json: "socProcInsee", js: "socProcInsee", typ: r("SocProcInsee") },
    ],
    false,
  ),
  SocProcInsee: o(
    [
      { json: "catSocPro", js: "catSocPro", typ: u(undefined, "") },
      { json: "famSocPro", js: "famSocPro", typ: u(undefined, r("FamSocPro")) },
    ],
    false,
  ),
  Deport: o(
    [
      { json: "uid", js: "uid", typ: "" },
      { json: "chronotag", js: "chronotag", typ: "" },
      { json: "legislature", js: "legislature", typ: "" },
      { json: "refActeur", js: "refActeur", typ: "" },
      { json: "dateCreation", js: "dateCreation", typ: Date },
      { json: "datePublication", js: "datePublication", typ: Date },
      { json: "portee", js: "portee", typ: r("CodeEtLibelle") },
      { json: "lecture", js: "lecture", typ: r("CodeEtLibelle") },
      { json: "instance", js: "instance", typ: r("CodeEtLibelle") },
      { json: "cible", js: "cible", typ: r("Cible") },
      { json: "explication", js: "explication", typ: u(undefined, "") },
    ],
    false,
  ),
  Cible: o(
    [
      { json: "type", js: "type", typ: r("CodeEtLibelle") },
      { json: "referenceTextuelle", js: "referenceTextuelle", typ: "" },
    ],
    false,
  ),
  CodeEtLibelle: o(
    [
      { json: "code", js: "code", typ: "" },
      { json: "libelle", js: "libelle", typ: "" },
    ],
    false,
  ),
  Organe: o(
    [
      { json: "schemaVersion", js: "schemaVersion", typ: u(undefined, "") },
      { json: "xsiType", js: "xsiType", typ: u(undefined, r("TypeOrgane")) },
      { json: "uid", js: "uid", typ: "" },
      { json: "codeType", js: "codeType", typ: r("CodeTypeOrgane") },
      { json: "libelle", js: "libelle", typ: "" },
      { json: "libelleEdition", js: "libelleEdition", typ: u(undefined, "") },
      { json: "libelleAbrege", js: "libelleAbrege", typ: "" },
      { json: "libelleAbrev", js: "libelleAbrev", typ: "" },
      { json: "viMoDe", js: "viMoDe", typ: r("ViMoDe") },
      {
        json: "organeParentRef",
        js: "organeParentRef",
        typ: u(undefined, ""),
      },
      {
        json: "organePrecedentRef",
        js: "organePrecedentRef",
        typ: u(undefined, ""),
      },
      { json: "regime", js: "regime", typ: u(undefined, r("Regime")) },
      {
        json: "legislature",
        js: "legislature",
        typ: u(undefined, ""),
      },
      {
        json: "secretariat",
        js: "secretariat",
        typ: u(undefined, r("Secretariat")),
      },
      { json: "numero", js: "numero", typ: u(undefined, "") },
      { json: "lieu", js: "lieu", typ: u(undefined, r("OrganeLieu")) },
      {
        json: "regimeJuridique",
        js: "regimeJuridique",
        typ: u(undefined, ""),
      },
      {
        json: "siteInternet",
        js: "siteInternet",
        typ: u(undefined, ""),
      },
      {
        json: "nombreReunionsAnnuelles",
        js: "nombreReunionsAnnuelles",
        typ: u(undefined, ""),
      },
      {
        json: "positionPolitique",
        js: "positionPolitique",
        typ: u(undefined, r("PositionPolitique")),
      },
      { json: "preseance", js: "preseance", typ: u(undefined, "") },
      { json: "couleurAssociee", js: "couleurAssociee", typ: u(undefined, "") },
      {
        json: "listePays",
        js: "listePays",
        typ: u(undefined, a("")),
      },
    ],
    false,
  ),
  OrganeLieu: o(
    [
      { json: "region", js: "region", typ: r("Region") },
      { json: "departement", js: "departement", typ: r("Departement") },
    ],
    false,
  ),
  Departement: o(
    [
      {
        json: "codeNatureDep",
        js: "codeNatureDep",
        typ: u(undefined, r("CodeNatureDep")),
      },
      { json: "code", js: "code", typ: u(undefined, "") },
      { json: "libelle", js: "libelle", typ: u(undefined, "") },
    ],
    false,
  ),
  Region: o(
    [
      { json: "type", js: "type", typ: u(undefined, r("RegionType")) },
      { json: "libelle", js: "libelle", typ: u(undefined, "") },
    ],
    false,
  ),
  Secretariat: o(
    [
      { json: "secretaire01", js: "secretaire01", typ: u(undefined, "") },
      { json: "secretaire02", js: "secretaire02", typ: u(undefined, "") },
    ],
    false,
  ),
  ViMoDe: o(
    [
      { json: "dateDebut", js: "dateDebut", typ: u(undefined, Date) },
      { json: "dateAgrement", js: "dateAgrement", typ: u(undefined, Date) },
      { json: "dateFin", js: "dateFin", typ: u(undefined, Date) },
    ],
    false,
  ),
  Photo: o(
    [
      { json: "chemin", js: "chemin", typ: "" },
      { json: "cheminMosaique", js: "cheminMosaique", typ: "" },
      { json: "hauteur", js: "hauteur", typ: 0 },
      { json: "largeur", js: "largeur", typ: 0 },
      { json: "xMosaique", js: "xMosaique", typ: 0 },
      { json: "yMosaique", js: "yMosaique", typ: 0 },
    ],
    false,
  ),
  Civ: ["M.", "Mme"],
  TypeMandat: [
    "MandatAvecSuppleant_Type",
    "MandatMission_Type",
    "MandatParlementaire_type",
    "MandatSimple_Type",
  ],
  RegionType: [
    "Collectivités d'outre-mer et Nouvelle-Calédonie",
    "Dom",
    "Français établis hors de France",
    "Métropolitain",
  ],
  CodeTypeOrgane: [
    "API",
    "ASSEMBLEE",
    "BUREAU",
    "CIRCONSCRIPTION",
    "CJR",
    "CMP",
    "CNPE",
    "CNPS",
    "COMNL",
    "COMPER",
    "COMSENAT",
    "COMSPSENAT",
    "CONFPT",
    "CONSTITU",
    "DELEG",
    "DELEGBUREAU",
    "DELEGSENAT",
    "GA",
    "GE",
    "GEVI",
    "GOUVERNEMENT",
    "GP",
    "GROUPESENAT",
    "HCJ",
    "MINISTERE",
    "MISEVAL",
    "MISINFO",
    "MISINFOCOM",
    "MISINFOPRE",
    "OFFPAR",
    "ORGAINT",
    "ORGEXTPARL",
    "PARPOL",
    "PRESREP",
    "SENAT",
  ],
  FamSocPro: [
    "Agriculteurs exploitants",
    "Artisans, commerçants et chefs d'entreprise",
    "Autres personnes sans activité professionnelle",
    "Cadres et professions intellectuelles supérieures",
    "Employés",
    "Ouvriers",
    "Professions Intermédiaires",
    "Retraités",
    "Sans profession déclarée",
  ],
  TypeOrgane: [
    "GroupePolitique_type",
    "OrganeCirconscription_type",
    "OrganeExterne_Type",
    "OrganeExtraParlementaire_type",
    "OrganeParlementaireInternational",
    "OrganeParlementaire_Type",
  ],
  CodeNatureDep: ["H", "M", "O", "T"],
  PositionPolitique: ["Majoritaire", "Minoritaire", "Opposition"],
  Regime: [
    "Assemblée consultative provisoire",
    "Assemblées nationales constituantes",
    "3ème République",
    "4ème République",
    "5ème République",
  ],
}
