// To parse this data:
//
//   import { Convert, Document, DossierParlementaire, DossiersLegislatifs } from "./file";
//
//   const document = Convert.toDocument(json);
//   const dossierParlementaire = Convert.toDossierParlementaire(json);
//   const dossiersLegislatifs = Convert.toDossiersLegislatifs(json);
//
// These functions will throw an error if the JSON doesn't
// match the expected interface, even if the JSON is valid.

import { Acteur, Mandat, Organe } from "./acteurs_et_organes"
import { PointOdj, Reunion } from "./agendas"
import { Scrutin } from "./scrutins"
import { CodeActe } from "../shared_types/codes_actes"

export interface DossiersLegislatifs {
  textesLegislatifs: Document[]
  dossiersParlementaires: DossierParlementaire[]
}

export interface DossierParlementaire {
  schemaVersion?: string
  xsiType?: DossierParlementaireXsiType
  uid: string
  legislature: string
  titreDossier: TitreDossier
  procedureParlementaire: ProcedureParlementaire
  initiateur?: Initiateur
  actesLegislatifs?: ActeLegislatif[]
  fusionDossier?: FusionDossier
  indexation?: Indexation
  plf?: EtudePlf[]
}

export enum DossierParlementaireXsiType {
  DossierCommissionEnqueteType = "DossierCommissionEnquete_Type",
  DossierIniativeExecutifType = "DossierIniativeExecutif_Type",
  DossierLegislatifType = "DossierLegislatif_Type",
  DossierMissionControleType = "DossierMissionControle_Type",
  DossierMissionInformationType = "DossierMissionInformation_Type",
  DossierResolutionAn = "DossierResolutionAN",
}

export interface EtudePlf {
  uid: string
  organeRef: string
  texteAssocie?: string
  rapporteurs?: Rapporteur[]
  missionMinefi?: MissionMinefi
  ordreDiqs: string
  ordreCommission: string
}

export interface MissionMinefi {
  typeMission: TypeMission
  codeMission?: string
  libelleLong: string
  libelleCourt: string
  typeBudget: TypeBudget
  missions?: MissionMinefi[]
}

export enum TypeBudget {
  BudgetAnnexe = "Budget annexe",
  BudgetGénéral = "Budget général",
  CompteDeConcoursFinancier = "Compte de concours financier",
  CompteSpécial = "Compte spécial",
  PremièrePartie = "Première partie",
}

export enum TypeMission {
  MissionPrincipale = "mission principale",
  MissionSecondaire = "mission secondaire",
  PartieDeMission = "partie de mission",
}

export enum Qualite {
  Auteur = "auteur",
  Cosignataire = "cosignataire",
  Rapporteur = "rapporteur",
  RapporteurGénéral = "rapporteur général",
  RapporteurPourAvis = "rapporteur pour avis",
  RapporteurSpécial = "rapporteur spécial",
}

export enum TypeActeLegislatif {
  AdoptionEuropeType = "Adoption_Europe_Type",
  ConclusionEtapeCcType = "ConclusionEtapeCC_Type",
  ConvocationCongresType = "ConvocationCongres_Type",
  CreationOrganeTemporaireType = "CreationOrganeTemporaire_Type",
  DecisionMotionCensureType = "DecisionMotionCensure_Type",
  DecisionRecevabiliteBureauType = "DecisionRecevabiliteBureau_Type",
  DecisionType = "Decision_Type",
  DeclarationGouvernementType = "DeclarationGouvernement_Type",
  DepotAccordInternationalType = "DepotAccordInternational_Type",
  DepotAvisConseilEtatType = "DepotAvisConseilEtat_Type",
  DepotInitiativeNavetteType = "DepotInitiativeNavette_Type",
  DepotInitiativeType = "DepotInitiative_Type",
  DepotLettreRectificativeType = "DepotLettreRectificative_Type",
  DepotMotionCensureType = "DepotMotionCensure_Type",
  DepotMotionReferendaireType = "DepotMotionReferendaire_Type",
  DepotRapportType = "DepotRapport_Type",
  DiscussionCommissionType = "DiscussionCommission_Type",
  DiscussionSeancePubliqueType = "DiscussionSeancePublique_Type",
  EtapeType = "Etape_Type",
  EtudeImpactType = "EtudeImpact_Type",
  MotionProcedureType = "MotionProcedure_Type",
  NominRapporteursType = "NominRapporteurs_Type",
  ProcedureAccelereType = "ProcedureAccelere_Type",
  PromulgationType = "Promulgation_Type",
  RenvoiCmpType = "RenvoiCMP_Type",
  RenvoiPrealableType = "RenvoiPrealable_Type",
  RetraitInitiativeType = "RetraitInitiative_Type",
  SaisieComAvisType = "SaisieComAvis_Type",
  SaisieComFondType = "SaisieComFond_Type",
  SaisineConseilConstitType = "SaisineConseilConstit_Type",
}

export interface ActeLegislatif {
  xsiType: TypeActeLegislatif
  uid: string
  codeActe: CodeActe
  libelleActe: LibelleActe
  organeRef?: string
  organe?: Organe // Added by Tricoteuses
  dateActe?: Date
  //
  anneeDecision?: string
  auteurMotion?: string
  auteursRefs?: string[]
  casSaisine?: TypeDeclaration
  codeLoi?: string
  contributionInternaute?: ContributionInternaute
  decision?: TypeDeclaration
  depotInitialLectureDefinitiveRef?: string
  depotInitialLectureDefinitive?: Document // Added by Tricoteuses
  formuleDecision?: string
  infoJo?: InfoJo
  infoJoce?: InfoJoce
  infoJoRect?: InfoJo[]
  initiateur?: Initiateur
  motif?: Motif
  numDecision?: string
  odjRef?: string
  odj?: PointOdj // Added by Tricoteuses
  provenanceRef?: string
  provenance?: Organe // Added by Tricoteuses
  rapporteurs?: Rapporteur[]
  referenceNor?: string
  reunionRef?: string
  reunion?: Reunion // Added by Tricoteuses
  statutAdoption?: TypeDeclaration
  statutConclusion?: TypeDeclaration
  texteAdopteRef?: string
  texteAdopte?: Document // Added by Tricoteuses
  texteAssocieRef?: string
  texteAssocie?: Document // Added by Tricoteuses
  texteEuropeen?: TexteEuropeen
  texteExterneAssocie?: TexteExterneAssocie
  texteLoiRef?: string
  textesAssocies?: TexteAssocie[]
  titreLoi?: string
  typeDeclaration?: TypeDeclaration
  typeMotion?: TypeDeclaration
  typeMotionCensure?: TypeDeclaration
  urlConclusion?: string
  urlEcheancierLoi?: string
  urlLegifrance?: string
  voteRefs?: string[]
  votes?: Scrutin[] // Added by Tricoteuses
  //
  actesLegislatifs?: ActeLegislatif[]
}

export interface LibelleActe {
  nomCanonique: string
  libelleCourt?: string
}

export interface Rapporteur {
  acteurRef: string
  acteur?: Acteur // Added by Tricoteuses
  typeRapporteur: Qualite
  etudePlfRef?: string
}

export interface TypeDeclaration {
  famCode: string
  libelle: string
}

export interface TexteAssocie {
  typeTexte: TypeTexte
  texteAssocieRef: string
  texteAssocie?: Document // Added by Tricoteuses
}

export enum TypeTexte {
  Bta = "BTA",
  Tap = "TAP",
}

export interface ContributionInternaute {
  dateOuverture?: string
  dateFermeture?: string
}

export interface InfoJoce {
  refJoce: string
  dateJoce: string
}

export interface InitiateurActeur {
  acteurRef: string
  acteur?: Acteur // Added by Tricoteuses
  mandatRef?: string
  mandat?: Mandat // Added by Tricoteuses
}

export enum Motif {
  EnApplicationDeLArticle612DeLaConstitution = "En application de l'article 61§2 de la Constitution",
}

export interface TexteEuropeen {
  typeTexteEuropeen: string
  titreTexteEuropeen: string
}

export interface InfoJo {
  typeJo: TypeJo
  dateJo: string
  numJo: string
  urlLegifrance?: string
  referenceNor?: string
}

export enum TypeJo {
  JoLoiDecret = "JO_LOI_DECRET",
}

export interface InfoJoRectElement {
  typeJo: TypeJo
  dateJo: string
  numJo: string
}

export interface FusionDossier {
  cause: Cause
  dossierAbsorbantRef: string
}

export enum Cause {
  DossierAbsorbé = "Dossier absorbé",
  ExamenCommun = "Examen commun",
}

export interface Indexation {
  themes: Themes
}

export interface Themes {
  "@niveau": string
  theme: Theme
}

export interface Theme {
  libelleTheme: string
}

export interface Initiateur {
  acteurs?: InitiateurActeur[]
  organeRef?: string
  organe?: Organe // Added by Tricoteuses
}

export interface ProcedureParlementaire {
  code: string
  libelle: string
}

export interface TitreDossier {
  titre: string
  titreChemin?: string
  senatChemin?: string
}

export interface TexteExterneAssocie {
  typeDoc: TexteExterneAssocieTypeDoc
  texteRef: string
  titreDoc: string
  urlDoc: string
}

export enum TexteExterneAssocieTypeDoc {
  Decret = "Décret",
}

export interface Document {
  schemaVersion?: string
  xsiType: TypeDocument
  uid: string
  legislature?: string
  cycleDeVie: CycleDeVie
  denominationStructurelle: DocumentDenominationStructurelle
  provenance?: Provenance
  titres: Titres
  divisions?: Division[]
  dossierRef: string
  dossier?: DossierParlementaire // Added by Tricoteuses
  classification: Classification
  auteurs: Auteur[]
  correction?: Correction
  notice: Notice
  indexation?: Indexation
  imprimerie?: Imprimerie
  coSignataires?: CoSignataireElement[]
  depotAmendements?: DepotAmendements
  organesReferents?: string[]
  rapportPublie?: boolean
  subdivisions?: Subdivision[] | null
}

export enum TypeDocument {
  AccordInternationalType = "accordInternational_Type",
  AvisConseilEtatType = "avisConseilEtat_Type",
  DocumentEtudeImpactType = "documentEtudeImpact_Type",
  RapportParlementaireType = "rapportParlementaire_Type",
  TexteLoiType = "texteLoi_Type",
}

export interface Subdivision {
  id: string
  type?: string
  niveau: number
  niveauRelatif: number
  titres?: ContenuSubdivision[]
  alineas?: ContenuSubdivision[]
}

export interface ContenuSubdivision {
  texte?: string
  html?: string
}

export interface Auteur {
  acteur?: AuteurActeur
  organeRef?: string
  organe?: Organe // Added by Tricoteuses
}

export interface AuteurActeur {
  acteurRef: string
  acteur?: Acteur // Added by Tricoteuses
  qualite: Qualite
}

export interface Classification {
  famille?: Famille
  type: ProcedureParlementaire
  sousType?: SousType
  statutAdoption?: StatutAdoption
}

export interface Famille {
  depot: ProcedureParlementaire
  classe: ProcedureParlementaire
  espece?: ProcedureParlementaire
}

export interface SousType {
  code: string
  libelle?: string
  libelleEdition?: string
}

export enum StatutAdoption {
  Adoptcom = "ADOPTCOM",
  Adoptseance = "ADOPTSEANCE",
  Rejetseance = "REJETSEANCE",
}

export interface CoSignataireElement {
  acteurRef?: string
  dateCosignature: string
  dateRetraitCosignature?: string
  edite: boolean
  organe?: CoSignataireOrgane
}

export interface CoSignataireOrgane {
  organeRef: string
  etApparentes: boolean
}

export interface Correction {
  typeCorrection: TypeCorrection
  niveauCorrection?: string
}

export enum TypeCorrection {
  Rectifié = "Rectifié",
}

export interface CycleDeVie {
  chrono: Chrono
}

export interface Chrono {
  dateCreation?: Date
  dateDepot?: Date
  datePublication?: Date
  datePublicationWeb?: Date
}

export enum DocumentDenominationStructurelle {
  Allocution = "Allocution",
  Avis = "Avis",
  Déclaration = "Déclaration",
  Lettre = "Lettre",
  Message = "Message",
  Motion = "Motion",
  ProjetDeLoi = "Projet de loi",
  PropositionDeLoi = "Proposition de loi",
  PropositionDeRésolution = "Proposition de résolution",
  Rapport = "Rapport",
  RapportDInformation = "Rapport d'information",
  Résolution = "Résolution",
}

export interface DepotAmendements {
  amendementsSeance: AmendementsSeance
  amendementsCommission?: AmendementsCommission[]
}

export interface AmendementsCommission {
  organeRef: string
  amendable: boolean
}

export interface AmendementsSeance {
  amendable: boolean
}

export interface Division {
  xsiType: TypeDocument
  uid: string
  legislature?: string
  cycleDeVie: CycleDeVie
  denominationStructurelle: string
  titres: Titres
  divisions?: Division[]
  dossierRef: string
  dossier?: DossierParlementaire // Added by Tricoteuses
  classification: Classification
  auteurs: Auteur[]
  organesReferents?: string[]
  correction?: Correction
  notice: Notice
  indexation?: Indexation
  imprimerie?: Imprimerie
  rapportPublie?: boolean
}

export interface Imprimerie {
  dian?: string
  isbn?: string
  nbPage?: string
  prix?: string
}

export interface Notice {
  numNotice?: string
  formule?: string
  adoptionConforme: boolean
}

export interface Titres {
  titrePrincipal: string
  titrePrincipalCourt: string
}

export enum Provenance {
  Commission = "Commission",
  Séance = "Séance",
  TexteDéposé = "Texte Déposé",
}

// Converts JSON strings to/from your types
// and asserts the results of JSON.parse at runtime
export class Convert {
  public static toDossiersLegislatifs(json: string): DossiersLegislatifs {
    return cast(JSON.parse(json), r("DossiersLegislatifs"))
  }

  public static dossiersLegislatifsToJson(value: DossiersLegislatifs): string {
    return JSON.stringify(uncast(value, r("DossiersLegislatifs")), null, 2)
  }

  public static toDocument(json: string): Document {
    return cast(JSON.parse(json), r("Document"))
  }

  public static documentToJson(value: Document): string {
    return JSON.stringify(uncast(value, r("Document")), null, 2)
  }

  public static toDossierParlementaire(json: string): DossierParlementaire {
    return cast(JSON.parse(json), r("DossierParlementaire"))
  }

  public static dossierParlementaireToJson(
    value: DossierParlementaire,
  ): string {
    return JSON.stringify(uncast(value, r("DossierParlementaire")), null, 2)
  }
}

function invalidValue(typ: any, val: any, key: any = ""): never {
  if (key) {
    throw Error(
      `Invalid value for key "${key}". Expected type ${JSON.stringify(
        typ,
      )} but got ${JSON.stringify(val)}`,
    )
  }

  throw Error(
    `Invalid value ${JSON.stringify(val)} for type ${JSON.stringify(typ)}`,
  )
}

function jsonToJSProps(typ: any): any {
  if (typ.jsonToJS === undefined) {
    const map: any = {}
    typ.props.forEach((p: any) => (map[p.json] = { key: p.js, typ: p.typ }))
    typ.jsonToJS = map
  }
  return typ.jsonToJS
}

function jsToJSONProps(typ: any): any {
  if (typ.jsToJSON === undefined) {
    const map: any = {}
    typ.props.forEach((p: any) => (map[p.js] = { key: p.json, typ: p.typ }))
    typ.jsToJSON = map
  }
  return typ.jsToJSON
}

function transform(val: any, typ: any, getProps: any, key: any = ""): any {
  function transformPrimitive(typ: string, val: any): any {
    if (typeof typ === typeof val) return val
    return invalidValue(typ, val, key)
  }

  function transformUnion(typs: any[], val: any): any {
    // val must validate against one typ in typs
    const l = typs.length
    for (let i = 0; i < l; i++) {
      const typ = typs[i]
      try {
        return transform(val, typ, getProps)
      } catch (_) {}
    }
    return invalidValue(typs, val)
  }

  function transformEnum(cases: string[], val: any): any {
    if (cases.indexOf(val) !== -1) return val
    return invalidValue(cases, val)
  }

  function transformArray(typ: any, val: any): any {
    // val must be an array with no invalid elements
    if (!Array.isArray(val)) return invalidValue("array", val)
    return val.map((el) => transform(el, typ, getProps))
  }

  function transformDate(val: any): any {
    if (val === null) {
      return null
    }
    const d = new Date(val)
    if (isNaN(d.valueOf())) {
      return invalidValue("Date", val)
    }
    return d
  }

  function transformObject(
    props: { [k: string]: any },
    additional: any,
    val: any,
  ): any {
    if (val === null || typeof val !== "object" || Array.isArray(val)) {
      return invalidValue("object", val)
    }
    const result: any = {}
    Object.getOwnPropertyNames(props).forEach((key) => {
      const prop = props[key]
      const v = Object.prototype.hasOwnProperty.call(val, key)
        ? val[key]
        : undefined
      result[prop.key] = transform(v, prop.typ, getProps, prop.key)
    })
    Object.getOwnPropertyNames(val).forEach((key) => {
      if (!Object.prototype.hasOwnProperty.call(props, key)) {
        result[key] = transform(val[key], additional, getProps, key)
      }
    })
    return result
  }

  if (typ === "any") return val
  if (typ === null) {
    if (val === null) return val
    return invalidValue(typ, val)
  }
  // if (typ === false) return invalidValue(typ, val)
  while (typeof typ === "object" && typ.ref !== undefined) {
    typ = typeMap[typ.ref]
  }
  if (Array.isArray(typ)) return transformEnum(typ, val)
  if (typeof typ === "object") {
    return typ.hasOwnProperty("unionMembers")
      ? transformUnion(typ.unionMembers, val)
      : typ.hasOwnProperty("arrayItems")
        ? transformArray(typ.arrayItems, val)
        : typ.hasOwnProperty("props")
          ? transformObject(getProps(typ), typ.additional, val)
          : invalidValue(typ, val)
  }
  // Numbers can be parsed by Date but shouldn't be.
  if (typ === Date && typeof val !== "number") return transformDate(val)
  return transformPrimitive(typ, val)
}

function cast<T>(val: any, typ: any): T {
  return transform(val, typ, jsonToJSProps)
}

function uncast<T>(val: T, typ: any): any {
  return transform(val, typ, jsToJSONProps)
}

function a(typ: any) {
  return { arrayItems: typ }
}

function u(...typs: any[]) {
  return { unionMembers: typs }
}

function o(props: any[], additional: any) {
  return { props, additional }
}

// function m(additional: any) {
//     return { props: [], additional };
// }

function r(name: string) {
  return { ref: name }
}

const typeMap: any = {
  DossiersLegislatifs: o(
    [
      {
        json: "textesLegislatifs",
        js: "textesLegislatifs",
        typ: a(r("Document")),
      },
      {
        json: "dossiersParlementaires",
        js: "dossiersParlementaires",
        typ: a(r("DossierParlementaire")),
      },
    ],
    false,
  ),
  DossierParlementaire: o(
    [
      { json: "schemaVersion", js: "schemaVersion", typ: u(undefined, "") },
      {
        json: "xsiType",
        js: "xsiType",
        typ: u(undefined, r("DossierParlementaireXsiType")),
      },
      { json: "uid", js: "uid", typ: "" },
      { json: "legislature", js: "legislature", typ: "" },
      { json: "titreDossier", js: "titreDossier", typ: r("TitreDossier") },
      {
        json: "procedureParlementaire",
        js: "procedureParlementaire",
        typ: r("ProcedureParlementaire"),
      },
      {
        json: "initiateur",
        js: "initiateur",
        typ: u(undefined, r("Initiateur")),
      },
      {
        json: "actesLegislatifs",
        js: "actesLegislatifs",
        typ: u(undefined, a(r("ActeLegislatif"))),
      },
      {
        json: "fusionDossier",
        js: "fusionDossier",
        typ: u(undefined, r("FusionDossier")),
      },
      {
        json: "indexation",
        js: "indexation",
        typ: u(undefined, r("Indexation")),
      },
      { json: "plf", js: "plf", typ: u(undefined, a(r("EtudePlf"))) },
    ],
    false,
  ),
  EtudePlf: o(
    [
      { json: "uid", js: "uid", typ: "" },
      { json: "organeRef", js: "organeRef", typ: "" },
      { json: "texteAssocie", js: "texteAssocie", typ: u(undefined, "") },
      {
        json: "rapporteurs",
        js: "rapporteurs",
        typ: u(undefined, a(r("Rapporteur"))),
      },
      {
        json: "missionMinefi",
        js: "missionMinefi",
        typ: u(undefined, r("MissionMinefi")),
      },
      { json: "ordreDiqs", js: "ordreDiqs", typ: "" },
      { json: "ordreCommission", js: "ordreCommission", typ: "" },
    ],
    false,
  ),
  MissionMinefi: o(
    [
      { json: "typeMission", js: "typeMission", typ: r("TypeMission") },
      { json: "codeMission", js: "codeMission", typ: u(undefined, "") },
      { json: "libelleLong", js: "libelleLong", typ: "" },
      { json: "libelleCourt", js: "libelleCourt", typ: "" },
      { json: "typeBudget", js: "typeBudget", typ: r("TypeBudget") },
      {
        json: "missions",
        js: "missions",
        typ: u(undefined, a(r("MissionMinefi"))),
      },
    ],
    false,
  ),
  ActeLegislatif: o(
    [
      { json: "xsiType", js: "xsiType", typ: r("TypeActeLegislatif") },
      { json: "uid", js: "uid", typ: "" },
      {
        json: "codeActe",
        js: "codeActe",
        typ: r("CodeActe"),
      },
      { json: "libelleActe", js: "libelleActe", typ: r("LibelleActe") },
      { json: "organeRef", js: "organeRef", typ: u(undefined, "") },
      { json: "dateActe", js: "dateActe", typ: u(undefined, Date) },
      {
        json: "actesLegislatifs",
        js: "actesLegislatifs",
        typ: u(undefined, a(r("ActeLegislatif"))),
      },
      { json: "anneeDecision", js: "anneeDecision", typ: u(undefined, "") },
      {
        json: "auteurMotion",
        js: "auteurMotion",
        typ: u(undefined, ""),
      },
      {
        json: "auteursRefs",
        js: "auteursRefs",
        typ: u(undefined, a("")),
      },
      {
        json: "casSaisine",
        js: "casSaisine",
        typ: u(undefined, r("TypeDeclaration")),
      },
      { json: "codeLoi", js: "codeLoi", typ: u(undefined, "") },
      {
        json: "contributionInternaute",
        js: "contributionInternaute",
        typ: u(undefined, r("ContributionInternaute")),
      },
      {
        json: "decision",
        js: "decision",
        typ: u(undefined, r("TypeDeclaration")),
      },
      {
        json: "depotInitialLectureDefinitiveRef",
        js: "depotInitialLectureDefinitiveRef",
        typ: u(undefined, ""),
      },
      { json: "formuleDecision", js: "formuleDecision", typ: u(undefined, "") },
      { json: "infoJo", js: "infoJo", typ: u(undefined, r("InfoJo")) },
      { json: "infoJoce", js: "infoJoce", typ: u(undefined, r("InfoJoce")) },
      {
        json: "infoJoRect",
        js: "infoJoRect",
        typ: u(undefined, a(r("InfoJo"))),
      },
      {
        json: "initiateur",
        js: "initiateur",
        typ: u(undefined, r("Initiateur")),
      },
      { json: "motif", js: "motif", typ: u(undefined, r("Motif")) },
      { json: "numDecision", js: "numDecision", typ: u(undefined, "") },
      { json: "odjRef", js: "odjRef", typ: u(undefined, "") },
      { json: "provenanceRef", js: "provenanceRef", typ: u(undefined, "") },
      {
        json: "rapporteurs",
        js: "rapporteurs",
        typ: u(undefined, a(r("Rapporteur"))),
      },
      { json: "referenceNor", js: "referenceNor", typ: u(undefined, "") },
      { json: "reunionRef", js: "reunionRef", typ: u(undefined, "") },
      {
        json: "statutAdoption",
        js: "statutAdoption",
        typ: u(undefined, r("TypeDeclaration")),
      },
      {
        json: "statutConclusion",
        js: "statutConclusion",
        typ: u(undefined, r("TypeDeclaration")),
      },
      {
        json: "texteAdopteRef",
        js: "texteAdopteRef",
        typ: u(undefined, ""),
      },
      { json: "texteAssocieRef", js: "texteAssocieRef", typ: u(undefined, "") },
      {
        json: "texteEuropeen",
        js: "texteEuropeen",
        typ: u(undefined, r("TexteEuropeen")),
      },
      {
        json: "texteExterneAssocie",
        js: "texteExterneAssocie",
        typ: u(undefined, r("TexteExterneAssocie")),
      },
      { json: "texteLoiRef", js: "texteLoiRef", typ: u(undefined, "") },
      {
        json: "textesAssocies",
        js: "textesAssocies",
        typ: u(undefined, a(r("TexteAssocie"))),
      },
      { json: "titreLoi", js: "titreLoi", typ: u(undefined, "") },
      {
        json: "typeDeclaration",
        js: "typeDeclaration",
        typ: u(undefined, r("TypeDeclaration")),
      },
      {
        json: "typeMotion",
        js: "typeMotion",
        typ: u(undefined, r("TypeDeclaration")),
      },
      {
        json: "typeMotionCensure",
        js: "typeMotionCensure",
        typ: u(undefined, r("TypeDeclaration")),
      },
      { json: "urlConclusion", js: "urlConclusion", typ: u(undefined, "") },
      {
        json: "urlEcheancierLoi",
        js: "urlEcheancierLoi",
        typ: u(undefined, ""),
      },
      {
        json: "urlLegifrance",
        js: "urlLegifrance",
        typ: u(undefined, ""),
      },
      {
        json: "voteRefs",
        js: "voteRefs",
        typ: u(undefined, a("")),
      },
    ],
    false,
  ),
  LibelleActe: o(
    [
      { json: "nomCanonique", js: "nomCanonique", typ: "" },
      { json: "libelleCourt", js: "libelleCourt", typ: u(undefined, "") },
    ],
    false,
  ),
  Rapporteur: o(
    [
      { json: "acteurRef", js: "acteurRef", typ: "" },
      { json: "typeRapporteur", js: "typeRapporteur", typ: r("Qualite") },
      { json: "etudePlfRef", js: "etudePlfRef", typ: u(undefined, "") },
    ],
    false,
  ),
  TypeDeclaration: o(
    [
      { json: "famCode", js: "famCode", typ: "" },
      { json: "libelle", js: "libelle", typ: "" },
    ],
    false,
  ),
  TexteAssocie: o(
    [
      { json: "typeTexte", js: "typeTexte", typ: r("TypeTexte") },
      { json: "texteAssocieRef", js: "texteAssocieRef", typ: "" },
    ],
    false,
  ),
  ContributionInternaute: o(
    [
      { json: "dateOuverture", js: "dateOuverture", typ: u(undefined, "") },
      { json: "dateFermeture", js: "dateFermeture", typ: u(undefined, "") },
    ],
    false,
  ),
  InfoJoce: o(
    [
      { json: "refJoce", js: "refJoce", typ: "" },
      { json: "dateJoce", js: "dateJoce", typ: "" },
    ],
    false,
  ),
  InitiateurActeur: o(
    [
      { json: "acteurRef", js: "acteurRef", typ: "" },
      { json: "mandatRef", js: "mandatRef", typ: u(undefined, "") },
    ],
    false,
  ),
  TexteEuropeen: o(
    [
      { json: "typeTexteEuropeen", js: "typeTexteEuropeen", typ: "" },
      { json: "titreTexteEuropeen", js: "titreTexteEuropeen", typ: "" },
    ],
    false,
  ),
  InfoJo: o(
    [
      { json: "typeJo", js: "typeJo", typ: r("TypeJo") },
      { json: "dateJo", js: "dateJo", typ: "" },
      { json: "numJo", js: "numJo", typ: "" },
      { json: "urlLegifrance", js: "urlLegifrance", typ: u(undefined, "") },
      { json: "referenceNor", js: "referenceNor", typ: u(undefined, "") },
    ],
    false,
  ),
  FusionDossier: o(
    [
      { json: "cause", js: "cause", typ: r("Cause") },
      { json: "dossierAbsorbantRef", js: "dossierAbsorbantRef", typ: "" },
    ],
    false,
  ),
  Indexation: o([{ json: "themes", js: "themes", typ: r("Themes") }], false),
  Themes: o(
    [
      { json: "@niveau", js: "@niveau", typ: "" },
      { json: "theme", js: "theme", typ: r("Theme") },
    ],
    false,
  ),
  Theme: o([{ json: "libelleTheme", js: "libelleTheme", typ: "" }], false),
  Initiateur: o(
    [
      {
        json: "acteurs",
        js: "acteurs",
        typ: u(undefined, a(r("InitiateurActeur"))),
      },
      { json: "organeRef", js: "organeRef", typ: u(undefined, "") },
    ],
    false,
  ),
  ProcedureParlementaire: o(
    [
      { json: "code", js: "code", typ: "" },
      { json: "libelle", js: "libelle", typ: "" },
    ],
    false,
  ),
  TitreDossier: o(
    [
      { json: "titre", js: "titre", typ: "" },
      { json: "titreChemin", js: "titreChemin", typ: u(undefined, "") },
      { json: "senatChemin", js: "senatChemin", typ: u(undefined, "") },
    ],
    false,
  ),
  TexteExterneAssocie: o(
    [
      { json: "typeDoc", js: "typeDoc", typ: r("TexteExterneAssocieTypeDoc") },
      { json: "texteRef", js: "texteRef", typ: "" },
      { json: "titreDoc", js: "titreDoc", typ: "" },
      { json: "urlDoc", js: "urlDoc", typ: "" },
    ],
    false,
  ),
  Document: o(
    [
      { json: "schemaVersion", js: "schemaVersion", typ: u(undefined, "") },
      { json: "xsiType", js: "xsiType", typ: r("TypeDocument") },
      { json: "uid", js: "uid", typ: "" },
      { json: "legislature", js: "legislature", typ: u(undefined, "") },
      { json: "cycleDeVie", js: "cycleDeVie", typ: r("CycleDeVie") },
      {
        json: "denominationStructurelle",
        js: "denominationStructurelle",
        typ: r("DocumentDenominationStructurelle"),
      },
      {
        json: "provenance",
        js: "provenance",
        typ: u(undefined, r("Provenance")),
      },
      { json: "titres", js: "titres", typ: r("Titres") },
      {
        json: "divisions",
        js: "divisions",
        typ: u(undefined, a(r("Division"))),
      },
      { json: "dossierRef", js: "dossierRef", typ: "" },
      {
        json: "classification",
        js: "classification",
        typ: r("Classification"),
      },
      { json: "auteurs", js: "auteurs", typ: a(r("Auteur")) },
      {
        json: "correction",
        js: "correction",
        typ: u(undefined, r("Correction")),
      },
      { json: "notice", js: "notice", typ: r("Notice") },
      {
        json: "indexation",
        js: "indexation",
        typ: u(undefined, r("Indexation")),
      },
      {
        json: "imprimerie",
        js: "imprimerie",
        typ: u(undefined, r("Imprimerie")),
      },
      {
        json: "coSignataires",
        js: "coSignataires",
        typ: u(undefined, a(r("CoSignataireElement"))),
      },
      {
        json: "depotAmendements",
        js: "depotAmendements",
        typ: u(undefined, r("DepotAmendements")),
      },
      {
        json: "organesReferents",
        js: "organesReferents",
        typ: u(undefined, a("")),
      },
      { json: "rapportPublie", js: "rapportPublie", typ: u(undefined, false) },
      {
        json: "subdivisions",
        js: "subdivisions",
        typ: u(null, u(undefined, a(r("Subdivision")))),
      },
    ],
    false,
  ),
  Subdivision: o(
    [
      { json: "id", js: "id", typ: "" },
      { json: "type", js: "type", typ: u(undefined, "") },
      { json: "niveau", js: "niveau", typ: 0 },
      { json: "niveauRelatif", js: "niveauRelatif", typ: 0 },
      {
        json: "titres",
        js: "titres",
        typ: u(null, a(r("ContenuSubdivision"))),
      },
      {
        json: "alineas",
        js: "alineas",
        typ: u(null, a(r("ContenuSubdivision"))),
      },
    ],
    false,
  ),
  ContenuSubdivision: o(
    [
      { json: "texte", js: "texte", typ: u(undefined, "") },
      { json: "html", js: "html", typ: u(undefined, "") },
    ],
    false,
  ),
  Auteur: o(
    [
      { json: "acteur", js: "acteur", typ: u(undefined, r("AuteurActeur")) },
      {
        json: "organeRef",
        js: "organeRef",
        typ: u(undefined, ""),
      },
    ],
    false,
  ),
  AuteurActeur: o(
    [
      { json: "acteurRef", js: "acteurRef", typ: "" },
      { json: "qualite", js: "qualite", typ: r("Qualite") },
    ],
    false,
  ),
  Classification: o(
    [
      { json: "famille", js: "famille", typ: u(undefined, r("Famille")) },
      { json: "type", js: "type", typ: r("ProcedureParlementaire") },
      { json: "sousType", js: "sousType", typ: u(undefined, r("SousType")) },
      {
        json: "statutAdoption",
        js: "statutAdoption",
        typ: u(undefined, r("StatutAdoption")),
      },
    ],
    false,
  ),
  Famille: o(
    [
      { json: "depot", js: "depot", typ: r("ProcedureParlementaire") },
      { json: "classe", js: "classe", typ: r("ProcedureParlementaire") },
      {
        json: "espece",
        js: "espece",
        typ: u(undefined, r("ProcedureParlementaire")),
      },
    ],
    false,
  ),
  SousType: o(
    [
      { json: "code", js: "code", typ: "" },
      { json: "libelle", js: "libelle", typ: u(undefined, "") },
      { json: "libelleEdition", js: "libelleEdition", typ: u(undefined, "") },
    ],
    false,
  ),
  CoSignataireElement: o(
    [
      {
        json: "acteurRef",
        js: "acteurRef",
        typ: u(undefined, ""),
      },
      { json: "dateCosignature", js: "dateCosignature", typ: "" },
      {
        json: "dateRetraitCosignature",
        js: "dateRetraitCosignature",
        typ: u(undefined, ""),
      },
      { json: "edite", js: "edite", typ: false },
      {
        json: "organe",
        js: "organe",
        typ: u(undefined, r("CoSignataireOrgane")),
      },
    ],
    false,
  ),
  CoSignataireOrgane: o(
    [
      { json: "organeRef", js: "organeRef", typ: "" },
      { json: "etApparentes", js: "etApparentes", typ: false },
    ],
    false,
  ),
  Correction: o(
    [
      {
        json: "typeCorrection",
        js: "typeCorrection",
        typ: r("TypeCorrection"),
      },
      {
        json: "niveauCorrection",
        js: "niveauCorrection",
        typ: u(undefined, ""),
      },
    ],
    false,
  ),
  CycleDeVie: o([{ json: "chrono", js: "chrono", typ: r("Chrono") }], false),
  Chrono: o(
    [
      { json: "dateCreation", js: "dateCreation", typ: u(undefined, Date) },
      { json: "dateDepot", js: "dateDepot", typ: u(undefined, Date) },
      {
        json: "datePublication",
        js: "datePublication",
        typ: u(undefined, Date),
      },
      {
        json: "datePublicationWeb",
        js: "datePublicationWeb",
        typ: u(undefined, Date),
      },
    ],
    false,
  ),
  DepotAmendements: o(
    [
      {
        json: "amendementsSeance",
        js: "amendementsSeance",
        typ: r("AmendementsSeance"),
      },
      {
        json: "amendementsCommission",
        js: "amendementsCommission",
        typ: u(undefined, a(r("AmendementsCommission"))),
      },
    ],
    false,
  ),
  AmendementsCommission: o(
    [
      { json: "organeRef", js: "organeRef", typ: "" },
      { json: "amendable", js: "amendable", typ: false },
    ],
    false,
  ),
  AmendementsSeance: o(
    [{ json: "amendable", js: "amendable", typ: false }],
    false,
  ),
  Division: o(
    [
      { json: "xsiType", js: "xsiType", typ: r("TypeDocument") },
      { json: "uid", js: "uid", typ: "" },
      { json: "legislature", js: "legislature", typ: u(undefined, "") },
      { json: "cycleDeVie", js: "cycleDeVie", typ: r("CycleDeVie") },
      {
        json: "denominationStructurelle",
        js: "denominationStructurelle",
        typ: "",
      },
      { json: "titres", js: "titres", typ: r("Titres") },
      {
        json: "divisions",
        js: "divisions",
        typ: u(undefined, a(r("Division"))),
      },
      { json: "dossierRef", js: "dossierRef", typ: "" },
      {
        json: "classification",
        js: "classification",
        typ: r("Classification"),
      },
      { json: "auteurs", js: "auteurs", typ: a(r("Auteur")) },
      {
        json: "organesReferents",
        js: "organesReferents",
        typ: u(undefined, a("")),
      },
      {
        json: "correction",
        js: "correction",
        typ: u(undefined, r("Correction")),
      },
      { json: "notice", js: "notice", typ: r("Notice") },
      {
        json: "indexation",
        js: "indexation",
        typ: u(undefined, r("Indexation")),
      },
      {
        json: "imprimerie",
        js: "imprimerie",
        typ: u(undefined, r("Imprimerie")),
      },
      { json: "rapportPublie", js: "rapportPublie", typ: u(undefined, false) },
    ],
    false,
  ),
  Imprimerie: o(
    [
      { json: "dian", js: "dian", typ: u(undefined, "") },
      { json: "isbn", js: "isbn", typ: u(undefined, "") },
      { json: "nbPage", js: "nbPage", typ: u(undefined, "") },
      { json: "prix", js: "prix", typ: u(undefined, "") },
    ],
    false,
  ),
  Notice: o(
    [
      { json: "numNotice", js: "numNotice", typ: u(undefined, "") },
      { json: "formule", js: "formule", typ: u(undefined, "") },
      { json: "adoptionConforme", js: "adoptionConforme", typ: false },
    ],
    false,
  ),
  Titres: o(
    [
      { json: "titrePrincipal", js: "titrePrincipal", typ: "" },
      { json: "titrePrincipalCourt", js: "titrePrincipalCourt", typ: "" },
    ],
    false,
  ),
  DossierParlementaireXsiType: [
    "DossierCommissionEnquete_Type",
    "DossierIniativeExecutif_Type",
    "DossierLegislatif_Type",
    "DossierMissionControle_Type",
    "DossierMissionInformation_Type",
    "DossierResolutionAN",
  ],
  TypeBudget: [
    "Budget annexe",
    "Budget général",
    "Compte de concours financier",
    "Compte spécial",
    "Première partie",
  ],
  TypeMission: [
    "mission principale",
    "mission secondaire",
    "partie de mission",
  ],
  Qualite: [
    "auteur",
    "cosignataire",
    "rapporteur",
    "rapporteur général",
    "rapporteur pour avis",
    "rapporteur spécial",
  ],
  TypeActeLegislatif: [
    "Adoption_Europe_Type",
    "ConclusionEtapeCC_Type",
    "ConvocationCongres_Type",
    "CreationOrganeTemporaire_Type",
    "DecisionMotionCensure_Type",
    "DecisionRecevabiliteBureau_Type",
    "Decision_Type",
    "DeclarationGouvernement_Type",
    "DepotAccordInternational_Type",
    "DepotAvisConseilEtat_Type",
    "DepotInitiativeNavette_Type",
    "DepotInitiative_Type",
    "DepotLettreRectificative_Type",
    "DepotMotionCensure_Type",
    "DepotMotionReferendaire_Type",
    "DepotRapport_Type",
    "DiscussionCommission_Type",
    "DiscussionSeancePublique_Type",
    "Etape_Type",
    "EtudeImpact_Type",
    "MotionProcedure_Type",
    "NominRapporteurs_Type",
    "ProcedureAccelere_Type",
    "Promulgation_Type",
    "RenvoiCMP_Type",
    "RenvoiPrealable_Type",
    "RetraitInitiative_Type",
    "SaisieComAvis_Type",
    "SaisieComFond_Type",
    "SaisineConseilConstit_Type",
  ],
  CodeActe: Object.values(CodeActe),
  TypeTexte: ["BTA", "TAP"],
  Motif: ["En application de l'article 61§2 de la Constitution"],
  TypeJo: ["JO_LOI_DECRET"],
  Cause: ["Dossier absorbé", "Examen commun"],
  TexteExterneAssocieTypeDoc: ["Décret"],
  TypeDocument: [
    "accordInternational_Type",
    "avisConseilEtat_Type",
    "documentEtudeImpact_Type",
    "rapportParlementaire_Type",
    "texteLoi_Type",
  ],
  StatutAdoption: ["ADOPTCOM", "ADOPTSEANCE", "REJETSEANCE"],
  TypeCorrection: ["Rectifié"],
  DocumentDenominationStructurelle: [
    "Allocution",
    "Avis",
    "Déclaration",
    "Lettre",
    "Message",
    "Motion",
    "Projet de loi",
    "Proposition de loi",
    "Proposition de résolution",
    "Rapport",
    "Rapport d'information",
    "Résolution",
  ],
  Provenance: ["Commission", "Séance", "Texte Déposé"],
}
