import { load } from "../../src/file_systems"
import { assert } from "chai"
import temp from "temp"
import fs from "fs-extra"
import { SchemaBugs } from "../../src/bugs"
import { BasePlugin } from "../../src/bugs"
import { Plugin as Dossier00001 } from "./dossier-00001"

// Automatically track and cleanup files at exit
temp.track()

suite("bugs")

test("#SchemaBugsFix", async function () {
  const d = new SchemaBugs({ "bugs-dir": "tests/bugs", schema: "dossier" })
  const bug = new Dossier00001({})
  d.plugins = [bug]

  const tmpDir = temp.mkdirSync("clean")
  fs.copySync("tests/bugs/dossier-00001", `${tmpDir}`)

  const f = `${tmpDir}/fail.json`
  // true means a bug was found and fixed
  assert.deepEqual(await d.fix([f]), { "dossier-00001": true })
  // false means the bug is not found, because it was fixed
  assert.deepEqual(await d.fix([f]), { "dossier-00001": false })
})

test("#SchemaBugs", async function () {
  const d = new SchemaBugs({ "bugs-dir": "tests/bugs", schema: "dossier" })
  const bug = new Dossier00001({})
  d.plugins = [bug]

  class Plugin extends BasePlugin {
    count: any

    constructor(options: any) {
      super(options, "test")
      this.count = {}
    }

    async fix(_content: any, filename: any): Promise<any> {
      this.count["fix"] = [filename]
      return null
    }

    preCheck(): any {
      this.count["preCheck"] = 1
    }

    check(content: any, filename: any): any {
      this.count["check"] = [content, filename]
      return null
    }

    postCheck(analysis: any): any {
      this.count["postCheck"] = analysis
    }
  }

  const i = new Plugin({})
  d.plugins = [i]

  const filename = "tests/bugs/dossier-00001/ok.json"
  const content = "content"

  await d.fix([filename])
  d.preCheck()
  const report = d.check(content, filename)
  const analysis = { filename: report }
  d.postCheck(analysis)

  assert.deepEqual(i.count["fix"], [filename])
  assert.equal(i.count["preCheck"], 1)
  assert.deepEqual(i.count["check"], [content, filename])
  assert.deepEqual(i.count["postCheck"], analysis)
})

test("#PluginDossier00001", function () {
  const bug = new Dossier00001({})
  const d = "tests/bugs/dossier-00001"

  {
    const f = `${d}/fail.json`
    const content = load(f)
    assert.equal(bug.fix(content, f), true, f)
  }

  {
    const f = `${d}/ok.json`
    const content = load(f)
    assert.equal(bug.fix(content, f), false, f)
  }
})
