import { assert } from "chai"
import {
  datasetDirectorySchema,
  getSchemas,
  getDatasets,
  diffUpstream,
  masterUrl,
} from "../src/datasets"

suite("dataset")

test("#datasetDirectorySchema", async function () {
  assert.deepEqual(datasetDirectorySchema("parent/Agenda_XV"), [
    ["parent/Agenda_XV/**/*.json", "agenda"],
  ])
  assert.deepEqual(datasetDirectorySchema("parent/Dossiers_Legislatifs_XV"), [
    ["parent/Dossiers_Legislatifs_XV/documents/**/*.json", "document"],
    ["parent/Dossiers_Legislatifs_XV/dossiers/**/*.json", "dossier"],
  ])
  for (const dataset of [
    "acteurs_mandats_organes",
    "AMO30_tous_acteurs_tous_mandats_tous_organes_historique",
  ])
    assert.deepEqual(datasetDirectorySchema(dataset), [
      [`${dataset}/organes/**/*.json`, "organe"],
      [`${dataset}/acteurs/**/*.json`, "acteur"],
    ])
})

test("#getSchemas", async function () {
  assert(getSchemas().includes("dossier"))
})

test("#getDatasets", async function () {
  const datasets = getDatasets()
  assert(datasets.length > 1)
  for (const dataset of datasets)
    assert(datasetDirectorySchema(dataset).length > 0)
})

test("#URLs", async function () {
  assert.equal(diffUpstream("invalid"), "invalid")
  assert.equal(masterUrl("invalid"), "invalid")
  assert.include(
    diffUpstream("../Agenda_XV_nettoye/FILENAME"),
    "tricoteuses/data",
  )
  assert.include(
    diffUpstream(
      "../AMO30_tous_acteurs_tous_mandats_tous_organes_historique_nettoye/FILENAME",
    ),
    "acteurs_mandats_organes",
  )

  assert.include(masterUrl("../Agenda_XV_nettoye/FILENAME"), "master/FILENAME")
})
